
;;; compare-files.el

;; Copyright (C) 2014-2015 Davin Pearson

;; Emacs Lisp Archive Entry
;; Filename: compare-files.el
;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: Windows Backup
;; Version: 1.0

;;; Commentary:

;; This file is not part of GNU Emacs.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Known Bugs:

;; None so far!

;;; Code:

;;;
;;; NOTE: This code is for use in Windows only as the drive spec is
;;; assumed to occupy position 0 of the file strings
;;;
(defun d-round (x)
  (/ (round (* x 100.0)) 100.0))

(defvar *compare-files-deep* nil)
(defvar *buf-message* (format "~/buf-%s.txt" (d-time--get-stamp)))
(defvar *buf-message-count* 0)

;; (find-file *buf-message*)
;; (setq rest '("abc"))
;; (buf-message "abc" 123)
;; (buf-message '(nil))
(defun buf-message (&rest rest)
  (save-excursion
    (find-file *buf-message*)
    (goto-char (point-max))
    (if (eq (car rest) t)
        (progn
          (apply 'insert '("Success\n"))
          (apply 'message '("Success")))
      (save-excursion
        (if (and rest (car rest) (stringp (car rest)))
            (insert (apply 'format rest))
          (insert "nil")
          )
        (insert "\n")
        (apply 'message rest)
        (when (= (mod *buf-message-count* 50) 0)
          (save-buffer))
        (incf *buf-message-count*)
        )
      )
    ) ;; END SAVE-EXCURSION!
  )   ;; END DEFUN! buf-message

(require 'archive-compare)

;; (compare-files-inner-nondirs ptr-c ptr-k progress)
(defun compare-files-inner-nondirs (ptr-c ptr-k &optional progress)
  (let (str-c str-k (skip nil) substr1 substr2)
    (when progress
      (setq count 0)
      (setq len   (length ptr-c)))
    (d-assert (compare-lists ptr-c ptr-k))
    ;;(compare-files-inner-subdirs (car ptr-c) (car ptr-k))
    (while (or ptr-c ptr-k)
      (recenter)
      (d-assert (and (stringp (car ptr-c)) (not (file-directory-p (car ptr-c)))))
      (d-assert (and (stringp (car ptr-k)) (not (file-directory-p (car ptr-k)))))
      (setq len-c (if (string-match d-unix-source (car ptr-c))
                      (length d-unix-source)
                    1))
      (setq len-k (if (string-match d-unix-target (car ptr-k))
                      (length (substring (car ptr-k)
                                         (match-beginning 0) (match-end 0)))
                    1))
      (when (file-exists-p "c:/")
        (setq substr1 (if (car ptr-c)
                          (substring (car ptr-c) 1)
                        nil))
        (setq substr2 (if (car ptr-k)
                          (substring (car ptr-k) 1)
                        nil)))
      (when (and (string-match "/media/www/" (car ptr-c))
                 (string-match "/media/www/" (car ptr-k)))
        (d-assert (string-match "/media/www/[a-zA-Z0-9_-]*/albums/internal" (car ptr-c)))
        (setq substr1 (substring (car ptr-c) (match-end 0)))
        (d-assert (string-match "/media/www/[a-zA-Z0-9_-]*/albums/internal" (car ptr-c)))
        (setq substr2 (substring (car ptr-c) (match-end 0)))
        ;; -------------------------------------------------------------
        (message "*** substr1=%s substr2=%s" substr1 substr2)
        (debug)
        (if (and ptr-c ptr-k)
            (if (and (stringp (car ptr-c))
                     (stringp (car ptr-k))
                     (string= substr1 substr2))
                (progn
                  (setq fc (nth 7 (file-attributes (car ptr-c))))
                  (setq fk (nth 7 (file-attributes (car ptr-k))))
                  (if (/= fc fk)
                      (buf-message "File size mismatch #1 %s, %s." (car ptr-c) (car ptr-k))
                    (when *compare-files-deep*
                      (setq result (d-compare-files (car ptr-c) (car ptr-k)))
                      (if result
                          (buf-message "Success")
                        (buf-message "Failure")
                        (error "*compare-files-deep* error"))))
                  ;;(debug "Bic Runga: Heal me now")
                  )
              (buf-message "File mismatch #2 %s, %s." (car ptr-c) (car ptr-k))
              (error "File#2 mismatch")
              )
          (buf-message "File mismatch #3 %s, %s." (car ptr-c) (car ptr-k))
          (error "file#3 mismatch")
          )
        (setq ptr-k (cdr ptr-k))
        (setq ptr-c (cdr ptr-c))
        (when progress
          (incf count)
          ;; (setq count 10) (setq len 100)
          (buf-message "Stage %d Progress %0.2f%%" skip (d-round (/ (* 100.0 count) len)))
          ;;(debug "David Bowie's China Girl")
          )
        ) ;; END WHEN!
      )   ;; END WHILE! (or ptr-c ptr-k)
    )     ;; END LET!
  )       ;; END DEFUN! compare-files-inner-nondirs

(setq skip nil)

;; (compare-files-inner-subdirs ptr-c ptr-k progress)
(defun compare-files-inner-subdirs (ptr-c ptr-k &optional progress)
  (let ((count 0)
        (len   0)
        (len-c 0)
        (len-k 0))
    (d-assert (compare-lists ptr-c ptr-k))
    (when (and ptr-c ptr-k)
      (setq len-c (if (string-match (concat "^" d-unix-source) (car ptr-c))
                      (length d-unix-source)
                    1))
      ;;                              0123456789012345678
      ;;             (match-beginning 0)     (match-end 0)
      ;;                              V                 V
      (setq len-k (if (string-match (concat "^" d-unix-target) (car ptr-k))
                      (length (substring (car ptr-k)
                                         (match-beginning 0) (match-end 0)))
                    1)))
    ;;(debug)
    ;; ---------------------------------------------------------------
    (cond
     ((null skip)
      ;; do nothing
      )
     ((= 0 skip)
      ;; do nothing
      )
     ((= 1 skip)
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      )
     ((= 2 skip)
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      )
     ((= 3 skip)
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      )
     ((>= 4 skip)
      (error "Skip out of bounds skip=4"))
     )
     (when progress
      (setq count 0)
      (setq len   (length ptr-c)))
    ;;(debug "Bic Runga / Sorry")
    (while (or ptr-c ptr-k)
      (setq substr1 (if (car ptr-c)
                        (substring (car ptr-c) len-c)))
      (setq substr2 (if (car ptr-c)
                        (substring (car ptr-k) len-k)))
      (if (and ptr-c ptr-k
               (stringp (car ptr-c))
               (stringp (car ptr-k))
               (string= substr1 substr2)) ;; (string= "abc" nil)
          ;; (string= nil nil)
          (let (ptr2-c ptr2-k)
            (setq ptr2-c (directory-files-subdirs (car ptr-c) t))
            (setq ptr2-k (directory-files-subdirs (car ptr-k) t))
            (let ((skip nil))
              (compare-files-inner-subdirs ptr2-c ptr2-k))
            ;;(debug "Halloween")
            (setq ptr2-c (directory-files-nondirs (car ptr-c) t))
            (setq ptr2-k (directory-files-nondirs (car ptr-k) t))
            (d-assert (compare-lists ptr2-c ptr2-k))
            (let ((skip nil))
              (compare-files-inner-nondirs ptr2-c ptr2-k))
            )
        (buf-message "File mismatch #4 %s, %s" (car ptr-c) (car ptr-k))
        (error "mismatch"))
      ;;(debug "John Coltrane: Ascension")
      (setq ptr-c (cdr ptr-c))
      (setq ptr-k (cdr ptr-k))
      (when progress
        (incf count)
        (buf-message "Stage %d Progress %0.2f%%" skip (d-round (/ (* 100.0 count) len)))
        )
      (when skip
        (incf count)
        (setq ptr-c (cdr ptr-c))
        (setq ptr-k (cdr ptr-k))
        (incf count)
        (setq ptr-c (cdr ptr-c))
        (setq ptr-k (cdr ptr-k))
        (incf count)
        (setq ptr-c (cdr ptr-c))
        (setq ptr-k (cdr ptr-k))
        )
      ) ;; END WHILE!
    )   ;; END LET!
  )     ;; END DEFUN! compare-files-inner-subdirs (ptr-c ptr-k &optional progress)

;; (compare-lists (setq ptr-1 '("a:/bcd")) (setq ptr-2 '("b:/bcx")))
;; (compare-lists (setq ptr-1 '("a:/bcd")) (setq ptr-2 '("b:/bcd")))
;; (compare-lists ptr-1 ptr-2)
(defun compare-lists (ptr-1 ptr-2)
  (let ((done  nil)
        (found t)
        (i     0)
        (len   nil)
        (len-1 nil)
        (len-2 nil))
    ;; ---------------------------------------------------------------
    (setq i 0)
    (setq len (length (car ptr-1)))
    (while (< i len)
      (when (> (aref (car ptr-1) i) 255)
        (setq found nil)
        (buf-message "File error #5 %s contains a spurious character %c (%d)" (car ptr-1) (aref (car ptr-1) i) (aref (car ptr-1) i)))
      (incf i))
      ;; -------------------------------------------------------------
    (setq i 0)
    (setq len (length (car ptr-2)))
    (while (< i len)
      (when (> (aref (car ptr-2) i) 255)
        (setq found nil)
        (buf-message "File error #6 %s contains a spurious character %c (%d)" (car ptr-2) (aref (car ptr-2) i) (aref (car ptr-2) i)))
      (incf i))
    ;; ---------------------------------------------------------------
    (when (and ptr-1 ptr-2)
      (setq len-1 (if (string-match d-unix-source (car ptr-1))
                      (length d-unix-source)
                    1))
      (setq len-2 (if (string-match d-unix-target (car ptr-2))
                      (length (substring (car ptr-2)
                                         (match-beginning 0) (match-end 0)))
                    1)))
    ;; ---------------------------------------------------------------
    ;;(debug "Dire Straits: You and your friend")
    (while (and (not done) ptr-1 ptr-2)
      (when (not (and ptr-1
                      ptr-2
                      (consp ptr-1)
                      (consp ptr-2)
                      (stringp (car ptr-1))
                      (stringp (car ptr-2))
                      (string= (substring (car ptr-1) len-1)
                               (substring (car ptr-2) len-2))))
        (buf-message "File error name mismatch #7\n (car ptr-1)=%s,\n (car ptr-2)=%s." (car ptr-1) (car ptr-2))
        (buf-message "ptr-1=%s" ptr-1)
        (buf-message "ptr-2=%s" ptr-2)
        (debug "abc (car ptr-1) (car ptr-2)" (car ptr-1) (car ptr-2))
        ;;(debug "Billie Holiday: What a Night, What a Moon, What a Girl")
        (setq found nil)
        (setq done t))
      (setq ptr-1 (cdr ptr-1))
      (setq ptr-2 (cdr ptr-2))) ;; END WHILE! (and (not done) ptr-1 ptr-2
    found))                     ;; END DEFUN! compare-lists

(progn
  (setq d-unix-source "/media/www/WD2TB-BLU/")
  (setq d-unix-target "/media/www/K2TB/")
  )

(defun compare-lists-dome (skip)
  (let (*buf-message-count* dir drive ptr-c ptr-k)
    (setq *buf-message-count* 0)
    (setq dir "albums/internal")
    (setq drive "f")
    (message "*** skip=%s" skip)
    (sit-for 1)
    (if (file-exists-p "c:/")
        (progn
          (setq ptr-c (directory-files-subdirs (concat "c:/"      dir) t))
          (setq ptr-k (directory-files-subdirs (concat drive ":/" dir) t)))
      (progn
        (setq ptr-c (directory-files-subdirs (concat d-unix-source dir) t))
        (setq ptr-k (directory-files-subdirs (concat d-unix-target dir) t)))
      )
    (d-assert (compare-lists ptr-c ptr-k))
    (compare-files-inner-subdirs ptr-c ptr-k 'PROGRESS)
    (setq ptr-c (directory-files-nondirs (concat "c:/" dir)      t))
    (setq ptr-k (directory-files-nondirs (concat drive ":/" dir) t))
    (d-assert (compare-lists ptr-c ptr-k))
    (compare-files-inner-nondirs ptr-c ptr-k 'PROGRESS)
    (save-some-buffers 'NO-QUESTIONS)
    ) ;; END LET!
  )   ;; END DEFUN! compare-lists-dome

(defun compare-files ()
  (interactive)
  (compare-lists-dome 0)
  (compare-lists-dome 1)
  (compare-lists-dome 2)
  (compare-lists-dome 3)
  ) ;; END DEFUN! compare-files ()

;;(setq list (directory-files-deep "c:/albums" t "\\.m3u$"))
;;(setq list (directory-files-deep "f:/albums" t "\\.m3u$"))
;;(setq list (directory-files-deep "f:/albums" t "\\.cue$"))
;;(setq list (directory-files-deep "f:/albums" t "Thumbs\\.db$"))
;;(mapcar 'delete-file list)
(provide 'compare-files)
