
;;; d-defmacro.el

;; Copyright (C) 2017 Davin Pearson

;; Emacs Lisp Archive Entry
;; Filename: d-defmacro.el
;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: defmacros for defining macros in J.T.W.
;; Version: 1.0

;;; Commentary:

;; This file is part of GNU Java Training Wheels.
;;
;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Install Instructions:

;; See the following URL for the latest info and a tarball:
;;
;; <http://davin.50webs.com/research/2010/mopa2e2.html#d-defmacro>
;;
;; Then untar the tarball to a folder pointed to by the Emacs variable
;; load-path and add the following line to your ~/.emacs file.
;;
;; (require 'd-defmacro)

;;; Known Bugs:

;; None so far!

;;; Code:

;;(load-file "~/lisp++-projects/c++2lisp++-stage-1-purge-comments.el")
;;(load-file (concat (car load-path) "lisp++-mode.el"))
;;(load-file "~/lisp++-projects/lisp++2c++-cclass.el")

(safe-require 'd-flm)

(setq d-macro-list nil)

(defmacro d-defmacro (name &rest macro-form)
  `(progn
     (setq d-macro-list (cons (quote , name) d-macro-list))
     (defmacro , name (&rest rest)
       ,@ macro-form
          )
     ))

;;(setq type "int")
;;(setq vari "v")
;;(setter int i)

(d-defmacro
 getter
 (setq type (nth 0 rest))
 (setq vari (nth 1 rest))
 (d-assert (cdr rest))
 (d-assert (not (cdddr rest)))
 (if (not (stringp type))
     (setq type (prin1-to-string type)))
 (if (not (stringp vari))
     (setq vari (prin1-to-string vari)))
 (setq prop nil)
 (setq var (d-read-str (concat "getter-setter-prop--" type "--" vari)))
 (when (not (and (boundp var) var))
   (set var t)
   (setq prop (concat "private " type " private_" vari ";")))
 (concat "public " type " get" (d-string-capitalise vari) "() "
         "{ return private_" vari "; }" prop "\n"))

(d-defmacro
 setter
 (setq type (nth 0 rest))
 (setq vari (nth 1 rest))
 (d-assert (cdr rest))
 (d-assert (not (cdddr rest)))
 (if (not (stringp type))
     (setq type (prin1-to-string type)))
 (if (not (stringp vari))
     (setq vari (prin1-to-string vari)))
 (setq prop nil)
 (setq var (d-read-str (concat "getter-setter-prop--" type "--" vari)))
 (when (not (and (boundp var) var))
   (set var t)
   (setq prop (concat "private " type " private_" vari ";")))
 (concat "public void set" (d-string-capitalise vari) "(" type " " vari ")"
         " { this.private" vari " = " vari "; }" prop "\n"))

;; (d-compress-args '("100" "200" "300" ")"))
(defun d-compress-args (rest)
  (let ((ptr    rest)
        (result "(")
        (count  0)) ;; (setq count 0)
    (while ptr
      (when (not (string= (car ptr) ")"))
        (setq result (concat result (if (/= count 0) ",") (car ptr)))
        (incf count))
      (setq ptr (cdr ptr)))
    (setq result (concat result ")"))
    (cons result count)
    ) ;; END LET!
  )   ;; END DEFUN! d-compress-args

(defun d-get-class-list ()
  (interactive)
  (save-excursion
    (save-match-data
      (let (indent-str class-or-interface class-name p1 p2 list)
        (goto-char (point-min)) ;;           1
        (while (re-search-forward (concat "\\(^[ \t]*\\)"
                                          "\\(public[ \t]+\\|abstract[ \t]+\\|"
                                          "final[ \t]+\\|\\)*"
                                          "\\(class\\|interface\\) +"
                                          "\\([A-Z][a-zA-Z0-9_]*\\)") nil t)
          ;;                                 3                        4
          (setq indent-str         (buffer-substring-no-properties (match-beginning 1)
                                                                   (match-end 1)))
          (setq class-or-interface (buffer-substring-no-properties (match-beginning 3)
                                                                   (match-end 3)))
          (setq class-name         (buffer-substring-no-properties (match-beginning 4)
                                                                   (match-end 4)))
          (save-excursion
            (beginning-of-line)
            (setq p1 (point))
            (cond
             ((save-excursion
                (forward-line 1)
                (beginning-of-line)
                (looking-at "^[ \t]*{"))
              (forward-line)
              (beginning-of-line)
              (forward-sexp)
              ;;(if (string= class-name "Singleton")
              ;;    (d-debug "Public Enemy / Mind Terrorist"))
              (setq p2 (point)))
             ((save-excursion
                (forward-line 1)
                (beginning-of-line)
                (looking-at "^[ \t]*begin\\>"))
              (re-search-forward (concat "^" indent-str "end[ \t]*$") nil t)
              (setq p2 (point)))
             )
            (setq list (cons (list class-or-interface class-name p1 p2) list))))
        list))))

(defun d-are-we-inside-class (class)
  (d-assert (stringp (nth 0 class)))
  (d-assert (stringp (nth 1 class)))
  (and (>= (point) (nth 2 class))
       (<= (point) (nth 3 class))))

(defun d-find-matching-class (class-list)
  (block nil
    (let ((ptr class-list)) ;; (setq ptr class-list)
      (while ptr
        (when (d-are-we-inside-class (car ptr))
          ;;(message "* found d-are-we-inside-class class-list=%s (car ptr)=%s" ptr (car ptr))
          ;;(d-error "Foomatic")
          (return (car ptr)))
        (setq ptr (cdr ptr))))))

(defun d-get-enclosing-class ()
  (let (class-list)
    (setq class-list (d-find-matching-class (d-get-class-list)))
    ;;(d-error "Alien Syndrome / class-list=%s" class-list)
    class-list))

;; (setq compress-args (d-compress-args '("100" "200" "300")))
(d-defmacro
 singleton_design_pattern
 (let (ctor compress-args compressed-args compressed-count
            list-of-classes matching-class count location)
   (setq class (nth 1 (d-get-enclosing-class)))
   (d-error (and "Public Enemy / How to Kill a Radio Consultant" class))
   (with-temp-buffer
     ;;(when (get-buffer "*singleton*")
     ;;  (kill-buffer "*singleton*"))
     ;;(switch-to-buffer (generate-new-buffer "*singleton*"))
     (setq b2 (current-buffer))
     ;;(message "* rest=%s" rest)
     (setq ctor (nth 0 rest))
     (insert ctor)
     (goto-char (point-min))
     (while (re-search-forward "^\\([ \t]*\\)constructor[ \t]*(" nil t)
       (replace-match (concat "\\1constructor " class "(") 'fixedcase))
     (goto-char (point-min))
     (d-assert (flm-re-search-forward--no-comments-no-strings "(" nil t))
     (setq begy (point))
     (setq compress-args (d-compress-args (cdr rest)))
     (setq compressed-args (car compress-args))
     (setq compressed-count (cdr compress-args))
     (setq location (flm-re-search-forward--no-comments-no-strings "(" nil t))
     (forward-char -1)
     (forward-sexp)
     (setq endy (point))
     (goto-char begy)
     (setq count 0)
     (condition-case err
         (while (<= (point) endy)
           (cond
            ;; -------------------------------------------------------
            ((looking-at "[a-zA-Z0-9_]")
             (skip-chars-forward "a-zA-Z0-9_")
             ;;(message "* [a-zA-Z0-9_] (point)=%s line=(%s) count=%s"
             ;;  (point) (d-current-line-as-string) count)
             )
            ;; -------------------------------------------------------
            ((looking-at "[ \t\r\n]")
             (skip-chars-forward " \t\r\n")
             ;;(message "skip-chars-forward  \\t\\r\\n (point)=%s" (point))
             ;;(d-debug "Public Enemy / Don't Believe the Hype")
             )
            ;; -------------------------------------------------------
            ((looking-at ",")
             (incf count)
             ;;(message "* (point)=%s line=(%s) incf count=%s" (point)
             ;;  (d-current-line-as-string) count)
             (forward-char)
             ;;(d-debug "Cold Lampin' with Flavor")
             )
            ;; -------------------------------------------------------
            ((looking-at "/\\*")
             (forward-sexp))
            ;; -------------------------------------------------------
            ((looking-at "\"")
             ;;(error "* inside string")
             (forward-sexp))
            ;; -------------------------------------------------------
            ((looking-at "//")
             (forward-line)
             (beginning-of-line))
            ;; -------------------------------------------------------
            ((looking-at "(")
             (forward-sexp))
            ;; -------------------------------------------------------
            ((looking-at ")")
             (forward-char)
             )
            ;; -------------------------------------------------------
            ((looking-at "<")
             (forward-sexp))
            ;; -------------------------------------------------------
            ((looking-at "{")
             (let ((debug-on-error nil))
               (error "{ found in arg list")))
            ;; -------------------------------------------------------
            (t
             (message "Misc case (point)=%s" (point))
             (forward-char))))
       (error
        (message "Error err=%s" (prin1-to-string err))))
     (incf count) ;; NOTE: one more than the number of commas
     (let ((debug-on-error nil))
       (when (/= count compressed-count)
         (d-debug "(/= count compressed-count): count=%s compressed-count=%s" count compressed-count)))
     ;;(d-debug "Public Enemy / Raise the Roof (point)=%s" (point))
     (setq ctor (buffer-substring-no-properties (point-min) (point-max)))
     (setq str (concat "private " ctor
                       "private classVar " class " private_instance;"
                       "public function " class " getInstance()"
                       "{"
                       "if (private_instance != null) then "
                       "{"
                       "return private_instance;"
                       "}"
                       "else"
                       "{"
                       "return private_instance = new " class compressed-args ";"
                       "}"
                       "}"))
     ;;(message "str=%s" str)
     str
     ) ;; END WITH-TEMP-BUFFER!
   )   ;; END LET!
 )     ;; END D-DEFMARO! singleton_design_pattern

(defun split-string-into-csv (str)
  "Note: csv stands for Comma Separated Values"
  (with-temp-buffer
    ;;(when (get-buffer "*csv*")
    ;;  (kill-buffer "*csv*"))
    ;(set-buffer (generate-new-buffer "*csv*"))
    ;;(switch-to-buffer (current-buffer))
    (setq b3 (current-buffer))
    ;;(switch-to-buffer b3)
    (d-assert (stringp str))
    (insert str)
    (jtw-mode)
    ;;(d-debug "Public Enemy / Public Enemy No. 1")
    ;;(let ((debug-on-error nil))
    ;;  (error "Prince / Forever in my life"))
    (let ((done  nil)
          (endy  nil)
          (p0    (goto-char (1+ (point-min))))
          (p1    nil)
          (list  nil)
          (depth 0))
      (while (not endy)
        (while (not done)
          (message "* schmu depth=%s looking-at=\"%s\""
                   depth
                   (buffer-substring-no-properties (point) (jtw--clamp-point (+ (point) 10))))
          (condition-case err
              (cond
               ((looking-at "{")
                (condition-case err
                    (forward-sexp)
                  (error
                   (forward-char)
                   (incf depth))))
               ((looking-at ",")
                (forward-char 1)
                (when (= depth 0)
                  (setq done t)))
               ((looking-at "<")
                (condition-case err
                    (progn
                      (forward-sexp)
                      (cond
                       ((save-excursion
                          (backward-char)
                          (looking-at ">"))
                        ;; DO NOTHING!
                        )
                       ((save-excursion
                          (backward-char)
                          (looking-at ")"))
                        (decf depth)
                        )))
                  (error
                   (forward-char)
                   (incf depth))))
               ((looking-at "[a-zA-Z0-9_]+")
                (skip-chars-forward "a-zA-Z0-9_"))
               ((looking-at "[ \t\r\n]+")
                (skip-chars-forward " \t\r\n"))
               ((eobp)
                (setq done t)
                (setq endy t))
               ((and (looking-at ")") (> depth 0))
                (decf depth)
                (when (= depth 0)
                  (setq done t)
                  (setq endy t)
                  ))
               ((looking-at "(")
                (condition-case err
                    (forward-sexp)
                  (error
                   (forward-char)
                   (incf depth))))
               ((looking-at "[")
                (condition-case err
                    (forward-sexp)
                  (error
                   (forward-char 1)
                   (incf depth))))
               ((looking-at "\\]")
                (forward-char)
                (decf depth))
               ((looking-at "//")
                (forward-sexp))
               ((looking-at "/\\*")
                (forward-sexp))
               ((looking-at "\"")
                (forward-sexp))
               (t
                (forward-char)
                ))
            (error
             ;;(message "Error err=%s" (prin1-to-string err))
             (cond
              ((eq (car err) 'invalid-regexp)
               ;;(d-debug "invalid-regexp %s" (prin1-to-string err))
               (forward-char)
               (setq done t))
              ((eq (car err) 'end-of-buffer)
               ;;(d-debug "end-of-buffer %s" (prin1-to-string err))
               (setq done t)
               (setq endy t))
              ((eq (car err) 'scan-error)
               (let ((debug-on-error nil))
                 (error "scan error %s" (prin1-to-string err)))
               (setq done t)
               (setq endy t))
              (t
               (let ((debug-on-error nil))
                 (error "Misc error: %s" err)))
               ))))
        (setq done nil)
        (setq p1 (point))
        (setq str (buffer-substring-no-properties p0 (1- p1)))
        (setq p0 p1)
        ;;(d-debug "foomatic")
        ;;(d-assert (null list))
        (setq list (cons str list))
        ;;(sit-and-message 3 "list=%s" list)
        )
      ;;(d-debug "Prince / It's Gonna Be a beautiful night")
      (setq list (nreverse list)))))

(defun splat-list (list)
   ;;(setq args (eval args))
   (let ((done   nil)
         (i      0)
         (result nil))
     (while (not done)
       (if (nth i list)
           (setq result (cons (nth i list) result))
         (setq done t))
       (incf i)
       )
     (setq list (mapcar (function (lambda (x) `(quote ,x))) list))
     list))

 (defun fcall (func &rest args)
   (eval `(,func ,@args))
   )

(d-defmacro
 foreach
 (setq vrbl (nth 0 rest))
 (setq list (nth 1 rest))
 (message "vrbl=%s" vrbl)
 (message "list=%s" list)
 (d-assert (null (cdddr rest)))
 ;;(d-assert (null (nth 3 rest)))
 (concat "for (Iterator " vrbl "= " list ".getIterator(); "
         "!" vrbl ".isDone(); "
         vrbl ".next())")
 )

(d-defmacro
 null_macro
 (message "(nth 0 rest)=%s" (nth 0 rest))
 (concat "public property String s = " (prin1-to-string (nth 0 rest)) ";"))

(provide 'd-defmacro)
