
;;; d-htmlize.el

;; Copyright (C) 2014-2015 Davin Pearson

;; Emacs Lisp Archive Entry
;; Filename: d-htmlize.el
;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: htmlize code (inner module ~/dlisp/imported-stuff/htmlize.el not written by me)
;; Version: 1.0

;;; Commentary:

;; This file is not part of GNU Emacs.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Known Bugs:

;; None so far!

;;; Code:

(message "loaded d-htmlize target=%s" target)

(defun message (format-string &rest args) (interactive))

;;(message 123)
;;(message "123")
;;(message (format "abc=%s" (point)))
(defun checkpoint (msg &rest rest)
  (message "*** checkpoint %s"
	   (format (funcall 'message msg rest))))

(if (not (string-match "/dlisp/" (car load-path)))
    (if (file-exists-p "/media/www/C1TB/home/dlisp/imported-stuff/htmlize.el")
        (progn
          (setq load-path (cons "/media/www/C1TB/home/dlisp/" load-path)))
      (setq load-path (cons "~www/dlisp" load-path))
      ))

(d-emergency-set-load-path)

(load-file (concat (car load-path) "/imported-stuff/htmlize.el"))

(setq prefs-home-emacs-p t)
(setq prefs-minimal-syntax-highlighting-p t)
(setq prefs-maximal-syntax-highlighting-p t)
(setq prefs-font-libd-p t)
(setq prefs-font-xtra-p t)
;;(message "%s" 123)

(message 1)
(load-file (concat (car load-path) "/early-bindings.el"))
(message 2)
(load-file (concat (car load-path) "/d-make-faces.el"))
(message 3)
(load-file (concat (car load-path) "/jtw-mode.el"))
(message 5)
(load-file (concat (car load-path) "/d-auto-mode-alist.el"))
(message 6)
(load-file (concat (car load-path) "/d-php-mode.el"))
(message 7)

(defun htmlize--splat (old-text new-text)
  (save-match-data
    (save-excursion
      (goto-char (point-min))
      (while (search-forward old-text nil t)
        (replace-match new-text nil nil)))))

;; (setq filename "a.cc")
(defun d-htmlize (filename)
  (interactive "fEnter filename: ")
  ;;(find-file filename)
  ;;(jtw-mode)
  ;;(message "*** major-mode=%s" major-mode)
  (message 8)
  (message "*** before concat filename=%s" filename)
  (setq filename (concat (file-name-directory default-directory) filename))
  (message "*** after concat filename=%s" filename)
  ;;(find-file filename)
  (message 9)
  (message "*** before saving")
  ;;(htmlize-buffer (find-file filename))
  (htmlize-file filename)
  (save-some-buffers 'NO-QUESTIONS)
  (kill-buffer)
  (message "*** after saving")
  (message 10)
  (setq filename-2 (concat filename ".html"))
  (message 11)
  (find-file filename-2)
  (message "dh-6")
  (message 12)
  (read-only-mode -1)
  (message "filename-2=%s" filename-2)
  (message 11)
  (progn
    (goto-char (point-min))
    (re-search-forward "<pre>" nil t)
    (beginning-of-line)
    (delete-region (point-min) (point)))
  (progn
    (goto-char (point-min))
    (re-search-forward "</pre>" nil t)
    (beginning-of-line)
    (forward-line 1)
    (delete-region (point) (point-max))
    )
  (save-buffer)
  (kill-buffer nil)
  (d-quote
   (goto-char (point-min))
   (insert "<html>\n")
   (insert "<head>\n")
   (insert "   <link rel=\"stylesheet\" href=\"stylesheet10.css\" type=\"text/css\">\n")
   (insert "</head>\n")
   (insert "<body>\n")
   (goto-char (point-max))
   (insert "</body></html>\n"))
  ;;
  ;; NOTE: removal of extra syntax highlighting
  ;;
  ;; WARNING: not robust in the case of NOTE: inside strings
  ;;
  ;;(htmlize--splat "class=\"d-face-special-reference\"" "class=\"flcf\"")
  ;;(htmlize--splat "class=\"fgXwhite\"" "class=\"flcf\"")
  (d-quote
   (save-excursion
     (find-file filename-2)
     (message "*** file %s has the following line count %s"
              filename-2
              (count-lines-region (point-min) (point-max)))
     (message "*** BEGINNING OF PRINTOUT")
     (goto-char (point-min))
     (setq count 0)
     (while (not (eobp))
       (message "* LINE %d: %s" count (d-current-line-as-string))
       (forward-line 1)
       (beginning-of-line)
       (incf count))
     (message "*** END OF PRINTOUT")))
  )

;;(setq target "a.cc")
;;(setq target "a.jtw")
;; (setq target "/d/home/hairy-lemon/src/davin")

;; (setq target "c:/a/bc.txt")
(defun canonicalise (target)
  (progn
    (if (not (string-match "^\\([a-z]\\):/" target))
        (progn
          ;;(message "foo")
          target)
      (setq drive (substring target (match-beginning 1) (match-end 1)))
      (setq sub (substring target (match-end 0)))
      (concat drive ":/" sub))))

(message "target pre-canonicalise=%s" target)

(setq target (canonicalise target))

(message "target post-canonicalise=%s" target)

(message "target=%s" target)

(message "dh-pre")

(tde)

(d-htmlize target)

(message "dh-post")

(provide 'd-htmlize)
