
;;; Commentary:

;; This file provides some emergency bindings in case your .emacs
;; has a bug in it.  Simply invoke emacs with the following arguments:

;; emacs --debug-init --load ~www/dlisp/early-bindings.el
;;
;;; Known Bugs:

;; none so far!

;;; Code:

(require 'cl)

(assert (fboundp 'incf))
(assert (fboundp 'cdddr))

(setq *prefix* default-directory)

(when *prefix*
  (when (not (fboundp 'd-emergency-set-load-path))
    (defun d-emergency-set-load-path ()
      (message "Inside file: early-bindings.el defun: d-emergency-set-load-path")
      (assert (and "David Bowie / China Girl" (boundp '*prefix*)))
      (assert *prefix*)
      (setq load-path (cons (expand-file-name *prefix*)
                            load-path))
      (message "** file: early-bindings.el (car load-path)=%s" (car load-path))
      ) ;; END DEFUN! d-emergency-set-load-path
    )   ;; END WHEN! (when (not (fboundp 'd-emergency-set-load-path))
  )     ;; END WHEN! *prefix*

(d-emergency-set-load-path)

;; BUGGER THIS!
;;(assert (string= default-directory (getenv "PWD")))

 ;; prefs-*-p online

;; NOTE: Note this file contains no (load-file \"package-name\") or
;; any (require 'package-name) with the exception of (require 'cl).

(defvar prefs-advanced-user-p nil
  "Enables ESC ESC (Eval Expression), backquote as a dabbrev key,
aliases and (not d-movement)

not prefs-advanced-user-p => f5 for html indent buffer

")


(defvar prefs-calendar-online-p nil
  "Whether or not to set the calendar online")

(defvar prefs-home-emacs-p nil
  "Sets READONLY and d-log navigation"
  ;; was d-movement.el
  )

(defvar prefs-minimal-syntax-highlighting-p t
  "If set, sets the display theme to show
  prefs-minimal-syntax-highlighting-p.  Note that this variable
  is independent of pref-maximal-syntax-highlighting-p.")

(defvar prefs-maximal-syntax-highlighting-p t
  "If set, sets the display theme to show
  prefs-maximal-syntax-highlighting-p.  Note that this variable
  is independent of pref-minimal-syntax-highlighting-p.")

(defvar prefs-font-libd-p nil
  "Fontification of libd online")

(defvar prefs-font-xtra-p t
  "Extra syntax highlighting online (in case not d-htmlize.el)")

(defvar prefs-font-lisp++-p nil
  "Turns on Lisp++ syntax highlighting")

(defvar prefs-font-variables-and-properties-p nil
  "If t, turns on syntax highlighting of variables in the face:
d-face-el-local-variables")

(defvar prefs-font-lisp-stuff-p nil
  "If t, turns on syntax highlighting of Lisp code.")

(defvar prefs-windows-bindings-p t
  "To load cua.el (if t) and C-o bound to dlgopen-open-files
and C-z for UNDO
")

(defvar prefs-scratch-java-online-p nil
  "Java keywords appear in the scratch buffer")

(defvar prefs-scratch-libd-online-p nil
  "LIBD keywords appear in the scratch buffer")

(defvar prefs-dlisp-keybindings-online-p t
  "Load dlisp keybindings (for debugging Elisp)")

(defvar prefs-safe-comands-online-p t
  "Load dlisp safe commands (for debugging Elisp)")

(defvar prefs-large-font-p t
  "Whether or not to set the large font")

 ;; debug-on-error

(setq debug-on-error t)

 ;; emacs-dialect-*-p online

(setq emacs-dialect-xemacs-p
      (and (boundp 'xemacsp)
           (if (string-match "/usr/share/xemacs-" (car (last load-path))) t)))

(setq emacs-dialect-gnuemacs-p (eq system-type 'gnu/linux))

(defvar emacs-dialect-dosemacs-p (if (string-match "msdos" (emacs-version)) t)
  "Whether or not we are currently running on an MSDOS machine"
  )

 ;; os-type-*-p online

(defvar os-type-microsoft-p (if (memq system-type '(windows-nt ms-dos)) t)
  "Whether or not we are currently running on a microsoft operating system machine"
  )

(defvar os-type-linux-p (eq system-type 'gnu/linux)
  "Whether or not we are currently running on a GNU/Linux machine")

(defvar os-type-graphical-p t
  "Whether or not we are currently running on a graphical terminal window")

(defvar os-type-mswindows-p
  (if (and os-type-microsoft-p
           (not os-type-linux-p)
           (not emacs-dialect-dosemacs-p)) t)
  "Whether or not we are currently running on an the M.S. Windows operating system."
  )

(setq os-type-windows-p os-type-mswindows-p)

(setq os-type-msdos-p
      (if emacs-dialect-dosemacs-p t))

(setq os-type-graphical-p
      (let ((answer nil))
        (save-window-excursion
          (list-colors-display)
          (set-buffer "*Colors*")
          (goto-char (point-min))
          (setq answer (re-search-forward "snow" nil t))
          (kill-buffer "*Colors*")
          (if answer t nil))))

(setq os-type-text-p
      (not os-type-graphical-p))

;;(setq os-type-msdos-p   (if (and (string= (getenv "IS_DOS") "1") (memq system-type '(ms-dos))) t))
;;(setq os-type-windows-p (memq system-type '(windows-nt ms-dos)))
;;(setq os-type-linux-p   (not os-type-windows-p))

 ;; variables online

(setq undo-limit (* 1000 1000))
(setq-default tab-width 8)
(setq delete-old-versions t)
(setq-default cursor-type 'box)

(setq inhibit-startup-screen t)
(setq inhibit-splash-screen t)
(setq yes-or-no-p-history '("yes" "no"))

(setq load-prefer-newer t) ;; prefer newer of *.el and *.elc

(progn
  (setq backup-inhibited   t) ;; turns off *~ backup files
  (setq load-prefer-newer  t) ;; prefer newer of *.elc *.el
  (setq diary-file         "~/.diary")
  (setq ls-lisp-dirs-first t) ;; like MS Windows Explorer
  (setq dired-use-ls-dired t)
  (setq kill-whole-line    t)
  (setq case-fold-search   t)
  )

(setq kill-whole-line t)
;;(setq eol-mnemonic-unix "(Unix)")
(setq eol-mnemonic-dos "(Dos)")
(setq compile-command "make ")
(setq message-log-max 10000)
(put 'eval-expression  'disabled nil)
(put 'narrow-to-region 'disabled nil)
(put 'downcase-region  'disabled nil)
(put 'upcase-region    'disabled nil)
(put 'set-goal-column  'disabled nil)
(put 'scroll-left      'disabled nil)
(put 'scroll-down      'disabled nil)
(put 'scroll-up        'disabled nil)
(put 'scroll-right     'disabled nil)
(put 'erase-buffer     'disabled nil)

(setq enable-local-eval t)

(setq backup-inhibited nil)   ;; NOTE: disables ~ style backups
(setq auto-save-timeout 1)  ;; NOTE: #*# style backups are online

(setq-default c-basic-offset 3)

(setq-default
 transient-mark-mode               nil
 indent-tabs-mode                  nil   ;;; never put tabs in d- code
 parse-sexp-ignore-comments        t
 comment-column                    40
 completion-ignore-case            t     ;;; Good for Win95 mode.
 case-fold-search                  nil
 c-basic-offset                    3
 truncate-lines                    nil
 fill-column                       70
 backup-inhited                    t     ;;; Culls *~ style comments
 )

(defalias 'tde 'toggle-debug-on-error)

(setq
 gc-cons-threshold                 2000000
 visible-bell                      t
 same-window-buffer-names          nil
 compilation-window-height         15
 line-number-mode                  t
 require-final-newline             t
 search-highlight                  t
 compilation-window-height         10
 compilation-ask-about-save        nil
 next-line-add-newlines            nil ;; Doesn't add newlines at end
 kill-whole-line                   t   ;; Makes C-k more convenient.
 next-screen-context-lines         1   ;; Overlap with PGUP/PGDN
 compile-command                   "make "
 default-major-mode                'fundamental-mode
 three-step-help                   t
 apropos-do-all                    t   ;; more extensive online help
 enable-recursive-minibuffers      nil ;; advanced customisation feature...
 column-number-mode                t
 eval-expression-debug-on-error    nil
 )

(setq completion-ignored-extensions
      (append (list
               ".old5" ".old4" ".old3" ".old2" ".old"   ".oldel" ".old.el"
               ".out"  ".exe"  ".pif"  ".com"  ".class" ".tmp"   ".tm4"
               ".num"  ".dep"  ".DEP"  ".ps"   ".dvi"   ".aux"   ".log"
               ".tga"  ".sys"
               )
              completion-ignored-extensions))

(auto-compression-mode 1)
(setq max-mini-window-height 1.0)
(progn
  (blink-cursor-mode -1)
  (blink-cursor-mode nil)
  (setq blink-cursor nil)
  )
(blink-cursor-mode t)

(setq compilation-scroll-output t)
(setq load-prefer-newer t) ;; prefers latest file out of x.el and x.elc
(setq compile-history (append '("cd ~/hairy-lemon/src/50webs-com/ && make m4 m5"
                                "cd ~/preprocessors/jtw-projects/ && make build "
                                "cd ~/lisp++-projects/ && make 2016/"
                                "cd ~/jtw-tutorials/ && make build "
                                "cd ~/book/lisp++ && make lisp++11-ebook.pdf"
                                "cd ~/book/jtw && make test.pdf"
                                "cd ~/dlisp/ && grep --color -nH -e d-super-f3 *.el"
                                "cd ~/dlisp/ && grep --color -nH -e FOO *.el"
                                "make doit")
                              (if (boundp 'compile-history)
                                  compile-history
                                )))

(global-set-key "\C-o" 'other-window)

(setq debug-ignored-errors (append
                            (list ;;"Args out of range"
                                  "No dynamic expansion for `.*' found"
                                  "No further dynamic expansion for `.*' found"
                                  "Containing expression ends prematurely"
                                  "Command attempted to use minibuffer while in minibuffer"
                                  "No mark set in this buffer"
                                  "End of history; no default available"
                                  "Can't expand minibuffer to full frame"
                                  "No.* dynamic expansion for `[a-zA-Z0-9_\t\r\n]*' found"
                                  "Attempt to split minibuffer window"
                                  "File .* does not exist"
                                  "Symbol's value as variable is void:.*"
                                  "Containing expression ends prematurely"
                                  "Canceled"
                                  )
                            debug-ignored-errors))

 ;; Elisp specific defuns

(defmacro d-quote (&rest ignored))

;;; file: lisp++2c++-lisp++2c++.el
;;; (setq ptr (d-remove-backup-and-nonfile-elements (setq list (setq ptr (buffer-list)))))
(defun d-remove-backup-and-nonfile-elements (list)
  ;; (d-debug "Calamansi + Foomatic")
  ;; (setq x 1234)
  ;; (setq x "foo.el")
  ;; (setq x ".#abc.el")
  (let ((ptr list)) ;; (setq ptr list)
    (cond
     ((stringp (car ptr))
      (d-error "Smelly cat")
      (setq ptr (mapcar
                 (function
                  (lambda (x)
                    ;;(debug "Calamansi")
                    (if (string-match
                         "^\\.#"
                         (file-name-nondirectory x))
                        nil
                      x))) ptr))
      )
     ((bufferp (car ptr)) ;; (setq ptr (buffer-list))
      ;;(setq count 0)    ;; (setq ptr (cdr ptr)) (setq x (car ptr))
      (setq ptr (mapcar (function
                         (lambda (x)
                           ;;(debug "Frogger")
                           (buffer-file-name x)
                           )) ptr))
      (setq ptr (delq nil ptr)) ;; (equal "abc" "abc")
      (setq ptr (delete-duplicates ptr :test 'string=))
      ;;(message "ptr=%s" ptr)
      ;;(debug "This is no Bach; this is a great river")
      ;;(debug "Calamansi")
      ;;(if (stringp (car ptr))
      ;;    (message "(car ptr) is a string %s" (car ptr)))
      ;; (setq x "~/.emacs")
      (setq ptr (mapcar
                 (function
                  (lambda (x)
                    ;;(debug "Calamansi")
                    (d-assert (or (not x) (stringp x)))
                    (if (and x (string-match "^\\.#"
                                             (if (stringp x)
                                                 (file-name-nondirectory x)
                                               "")))
                        nil x))) ptr))
      ;;(message "No.9 ptr=%s" ptr)
      )
     ((null (car ptr))
      (message "null type in (car ptr)=%s" (car ptr))
      )
     (t
      (d-error "Wrong type for arg (car ptr)=%s" (car ptr)))
     )
    (setq ptr (delq nil ptr))
    (setq ptr (delete-duplicates ptr :test 'string=))
    ;;(d-debug "Paul Mcartney / Live and Let Die ")
    ptr))

;; file: ~/dlisp/early-bindings.el
(defun d-os-dependent-compare-files (file1 file2)
  (let (func-ptr)
    (setq func-ptr (if os-type-microsoft-p 'string=-ignore-case 'string=))
    (funcall func-ptr file1 file2)))

;; (d-debug (format "abc=%s" 123))
(defmacro d-debug (&rest rest)
  `(when debug-on-error
     ;;(setq debug-on-error nil)
     (funcall 'debug ,@rest))
  )

(defmacro d-assert (condition)
  `(progn
     (assert ,condition)
     t)
  )

;; (setq debug-on-error t) (d-error "Smeg")
(defmacro d-error (&rest rest)
  `(let ((debug-on-error nil))
     (funcall 'error ,@rest)))

(defun insert-prin1 (p)
  (insert (prin1-to-string p)))

(defun d-read-ready (str)
  (and (stringp str) (not (string= "" str))))

(defun d-read-str (str)
  (d-assert (d-read-ready str))
  (read str))

(defmacro not-eq (o1 o2)
  `(not (eq ,o1 ,o2)))

;; (setq f 'foomatic)
(defun safe-require (f)
  (condition-case err
      (progn
        ;;(message "* About to require '%s" f)
        (require f)
        ;;(message "* Done require '%s" f)
        )
    (error
     ;;(let ((debug-on-error t))
     (message (format "**** Strawpeople / Error: %s"
                      (prin1-to-string err)
                      ))
     )))

(defun d-string-isupper (ch)
  (and (>= ch ?A) (<= ch ?Z)))

(defalias 'd-string-isuppercase 'd-string-isupper)

(defun d-string-ischar (ch)
  (or (and (>= ch ?a) (<= ch ?z))
      (and (>= ch ?A) (<= ch ?Z))))

(defalias 'd-string-ischar 'd-string-ischaracter)

(defun d-string-islower (ch)
  (and (>= ch ?a) (<= ch ?z)))

(defalias 'd-string-islowercase 'd-string-islower)

;; (d-string-capitalise (setq str "abc"))
(defun d-string-capitalise (str)
  (setq str (concat "" str)) ;; clones existing string
  (setq ch (aref str 0))
  (when (d-string-islowercase ch)
    (setq ch (- ch 32))
    (aset str 0 ch))
  str)

;; (d-string-purge-spaces "ptr< Singleton >")
(defun d-string-purge-spaces (str)
  (let (sausage i len)
    (setq sausage "")
    (setq i 0)
    (setq len (length str))
    (while (< i len)
      (if (/= (aref str i) ? )
          (setq sausage (format "%s%c" sausage (aref str i))))
      (incf i))
  sausage))

;;;
;;; (d-trim-left "\r\t abc \t\r\n  abc def  ")
;;;
(defun d-trim-left-old (s)
  (progn
    (while (and (setq ch (aref s 0))
                (or (eq ch ?\ )
                    (eq ch ?\t)
                    (eq ch ?\r)
                    (eq ch ?\n)))
      (setq s (substring s 1)))
    s)
  )

;; (d-trim-left "        abc  def        ")
;; (d-trim-left " ")
(defun d-trim-left (s)
  (let (i len ch)
    (setq i 0)
    (setq len (length s))
    (while (and (< i len)
                (setq ch (aref s i))
                (or (eq ch ?\ )
                    (eq ch ?\t)
                    (eq ch ?\r)
                    (eq ch ?\n)))
      (incf i))
    (setq s (substring s i)))
  s)

;;;
;;; (d-trim-right "abc def \r\t abc \r\t\n def \r\n")
;;; (d-trim-right s)
(defun d-trim-right-old (s)
  (progn
    (setq i (1- (length s)))
    (while (and (setq ch (aref s i))
                (or (eq ch ?\ )
                    (eq ch ?\t)
                    (eq ch ?\r)
                    (eq ch ?\n)))
      (setq s (substring s 0 i))
      (decf i))
    s)
  )

;; (d-trim-right "abc        def \n\n\n")
;; (d-trim-right " ")
;; (d-trim-right "class")
(defun d-trim-right (s)
  (let (i ch)
    (setq i (1- (length s)))
    (while (and (>= i 0)
                (setq ch (aref s i))
                (or (eq ch ?\ )
                    (eq ch ?\t)
                    (eq ch ?\r)
                    (eq ch ?\n)))
      (decf i))
    (setq s (substring s 0 (1+ i)))
    )
  )

;;;
;;; (d-trim-string (setq s "\r\t\n  abc def \n sexy rexy \n"))
;;; (d-trim-string nil)
;;; (d-trim-left "")
;;; (d-trim-right "")
(defun d-trim-string (str)
  (d-assert (not (consp str)))
  (if (null str)
      nil
    (if (symbolp str)
        (prin1-to-string str)
      (if (string= str "")
          ""
        (d-trim-left (d-trim-right str))))))

(defun d-buffer-substring (top bot)
  (buffer-substring-no-properties top bot)
  )

;; (setq prompt "abc> ")
;; (setq ch ?z)
;;; also appears in file: ~/lisp++-projects/lisp++2c++.el
(defun y-or-n-or-!-or-q-p (prompt)
  (let (ch done)
    (while (not done)
      (setq done nil)
      (setq ch (read-char prompt))
      (if (= ch ?Y) (setq ch ?y))
      (if (= ch ?N) (setq ch ?n))
      (if (= ch ?Q) (setq ch ?q))
      (if (or (= ch ?y) (= ch ?n) (= ch ?!) (= ch ?q))
          (setq done t)
        (if (not (string-match "Please" prompt))
            (setq prompt (concat "Please enter y or n or ! or q.  " prompt))))
      )
    ch)
  )

;; (setq c-basic-offset 3)
;; (get-buffer-from-file (setq filestr "~/.emacs"))
(defun get-buffer-from-file (filestr)
  (progn
    (setq filestr  (expand-file-name filestr))
    (setq buf-list (buffer-list))
    (setq ptr buf-list)
    (setq done nil)
    (setq found nil))
  (while (not done)
    (setq bfn (buffer-file-name (car ptr)))
    (when (d-os-dependent-compare-files bfn filestr)
      (setq found t)
      (setq done  t))
    (setq ptr (cdr ptr)))
  (if found
      (car ptr)))

;; (setq result (d-save-some-buffers (setq no-questions nil)))
;; (setq result (d-save-some-buffers (setq no-questions t)))
(defun d-save-some-buffers (&optional no-questions) ;; FOO! BAR? sadad
  (interactive)
  (let (ptr done (saved-file-count 0) questions buf)
    (progn ;; (setq no-questions nil) (setq saved-file-count 0)
      (setq ch ?y)
      (setq ptr (buffer-list))
      (setq ptr (d-remove-backup-and-nonfile-elements ptr))
      (setq done nil)
      (setq count 0)
      (setq questions (not no-questions)))
    ;;(debug "Foomatic")
    (d-assert (stringp (car ptr)))
    (block nil
      (while (not done)
        (when (or (eq ptr nil) done)
          (setq done t)
          (setq ptr  nil)
          (return))
        (incf count)
        ;;(message "Inside main loop of d-save-some-buffers pre y-or-n-or-!-or-q-p")
        ;;(d-assert (and (car ptr) (bufferp (car ptr))))
        ;;(let ((debug-on-error t))
        ;;  (d-assert (and buf (bufferp buf))))
        (setq buf (find-file (car ptr)))
        (switch-to-buffer buf)
        (d-assert (car ptr))
        (setq ch (if (or (not (= ch ?!)) questions)
                     (y-or-n-or-!-or-q-p
                      (format "Save buffer %s? (y/n/!/q)"
                              (file-name-nondirectory (car ptr))))
                   ?!)))
      ;;(debug "Elton John / Rocket Man")
      (cond
       ((= ch ?y)
        (progn
          (d-assert buf)
          (d-assert (bufferp buf))
          (when (buffer-modified-p buf)
            (call-interactively 'save-buffer)
            ;;(basic-save-buffer)
            ;;(kill-buffer)
            )
          (incf saved-file-count)
          ))
       ((= ch ?n)
        )
       ((= ch ?!) ;; (setq ch ?!)
        (call-interactively 'save-buffer)
        ;;(basic-save-buffer)
        ;;(kill-buffer buf)
        (incf saved-file-count)
        )
       ((= ch ?q)
        (setq done t)
        (setq ptr nil))
       ) ;; END COND!
      ;;(while (string-match "TAGS\\(<.*>\\)?$" (buffer-name (car ptr)))
      ;;  (bury-buffer))
      (setq ptr (cdr ptr))
      (if (not ptr) ;; d-assert d-error d-debug d-quote
          (setq done t))
      ;;(message "while black next step")
      ) ;; END BLOCK! nil
    (message "Saved %s file(s)" saved-file-count)
    ;;(debug "Trumatic")
    )   ;; END LET! (ptr done (saved-file-count 0) questions)
  )     ;; END DEFUN! d-save-some-buffers

(defun d-kill-some-buffers (&optional no-questions) ;; FOO! BAR? sadad
  (interactive)
  (let (ptr done (saved-file-count 0) questions buf kill-count)
    (progn ;; (setq no-questions nil) (setq saved-file-count 0)
      (setq ch ?y)
      (setq ptr (buffer-list))
      (setq ptr (d-remove-backup-and-nonfile-elements ptr))
      (setq done nil)
      (setq count 0)
      (setq questions (not no-questions))
      (setq kill-count 0))
    ;;(debug "Foomatic")
    (block nil
      (while (not done)
        (when (or (eq ptr nil) done)
          (setq done t)
          (setq ptr  nil)
          (return))
        (incf count)
        ;;(message "Inside main loop of d-save-some-buffers pre y-or-n-or-!-or-q-p")
        ;;(d-assert (and (car ptr) (bufferp (car ptr))))
        ;;(let ((debug-on-error t))
        ;;  (d-assert (and buf (bufferp buf))))
        (setq buf (find-file (car ptr)))
        (when (not (buffer-modified-p buf))
          (incf kill-count)
          (kill-buffer buf))
        ;;(while (string-match "TAGS\\(<.*>\\)?$" (buffer-name (car ptr)))
        ;;  (bury-buffer))
        (setq ptr (cdr ptr))
        (if (not ptr) ;; d-assert d-error d-debug d-quote
            (setq done t))
        ) ;; END WHILE! (not done)
      )   ;; END BLOCK! nil
    (message "Killed %s buffer(s)" kill-count)
    )     ;; END LET! (ptr done (saved-file-count 0) questions)
  )       ;; END DEFUN! d-kill-some-buffers

(progn
  (global-set-key "\C-x\C-s" 'save-some-buffers)
  (global-set-key "\C-xs"    'save-some-buffers)
  )

(defun d-load-file (file)
  (load-file file))

;;; NOTE: Inits frame to appear as fullscreen maximized
(add-to-list 'default-frame-alist '(fullscreen . maximized))

;;; (safe-expand-file-name nil)
;;; (safe-expand-file-name "d:/")
;;; (safe-expand-file-name (setq file "d:/home/"))
;;; (safe-expand-file-name "d:/home")
;;; (safe-expand-file-name "d:/home/.emacs")
;;; (safe-expand-file-name "d:/home//text/..")
;;;
;;; NOTE: never returns a trailing slash
;;;
;;; (setenv "HOME" "d:/")
;;; (safe-expand-file-name "d:/")
;;; (safe-expand-file-name "d:/frog")
;;; (safe-expand-file-name "d:/FROG")
;;; (safe-expand-file-name "~/.emacs")
;;; (safe-expand-file-name "~/dlisp")
;;;
(defun safe-expand-file-name (filename)
  "Replaces ~/ with expanded file name"

  (save-match-data
    (setq filename (if filename (expand-file-name filename)))
    ;;
    ;; NOTE: removes trailing slash
    ;;
    (if (and filename (string-match "/$" filename))
        (setq filename (substring filename 0 -1)))

    filename))

;;;
;;; (safe-compress-file-name nil)
;;; (safe-compress-file-name "d:/")
;;; (safe-compress-file-name "d:/home")
;;; (safe-compress-file-name (setq filename "d:\\home"))
;;; (safe-compress-file-name (setq filename "d:\\home\\"))
;;; (safe-compress-file-name "d:\\home\\fddf")
;;; (safe-compress-file-name "d://home//frog")
;;; (safe-compress-file-name "d:/HOME/fddf")
;;; (safe-compress-file-name "d:/home/fddf/")
;;; (safe-compress-file-name "d:/home//fddf")
;;; (safe-compress-file-name "d:/fred/fddf")
;;;
;;; NOTE: never returns a trailing slash
;;;
;;; (setenv "HOME" "d:/")
;;; (safe-compress-file-name "d:/")
;;; (safe-compress-file-name "d:/frog")
;;; (safe-compress-file-name "d:/home//text/..")
;;; (safe-compress-file-name "/home/www/text")
;;; (safe-compress-file-name "/home/davin/text")
;;;
(defun safe-compress-file-name (filename)
  "Replaces expanded ~/ expression with ~/"

  (save-match-data

    (assert (getenv "HOME"))
    (assert (file-directory-p (getenv "HOME")))

    (let* ((filename (safe-expand-file-name filename))
           (new-home (safe-expand-file-name (getenv "HOME"))))

      ;;(debug)

      (progn
        ;;
        ;; NOTE: adds trailing slash
        ;;
        (if (and filename (file-directory-p filename) (not (string-match "/$" filename)))
            (setq filename (concat filename "/")))
        (if (and new-home (not (string-match "/$" new-home)))
            (setq new-home (concat new-home "/"))))

      ;;(debug)
      ;; (safe-compress-file-name "d:/home")
      ;; (setq new-home "d:/home/")
      ;; (setq filename "d:/home/")

      ;;
      ;; NOTE: replaces HOME folder with ~/
      ;;
      (if (and filename (string-match (concat "^" new-home "\\(.*$\\)") filename))
          (setq filename (concat "~/" (substring filename (match-beginning 1) (match-end 1)))))

      (progn
        ;;
        ;; NOTE: removes trailing slash
        ;;
        (when (and filename (string-match "/$" filename))
          ;;(d-foo)
          (setq filename (substring filename 0 -1)))
        )

      filename)))

;;; (get-buffer-window)
;;; (d-is-buffer-visible (current-buffer))
(defun d-is-buffer-visible (buffer)
  (and buffer (get-buffer-window buffer)))

(defun safe-load-file (filename)
  (let ((cmd (list 'load-file filename)))
    (safe-command (eval cmd))))

 ;; safe commands

(defun safe-getenv (var)
  "Useful as the second argument to string-match"
  (setq var (getenv var))
  (if (eq var nil) "" var))

;;;
;;; NOTE: standard notation follows...
;;;
;;; File/directory spec is  (setq dir-1 "d:/home") or (setq dir-1 "d:") (note no trailing slash)
;;; File access is          (concat dir-1 "/" file-1)
;;; Relative directory spec (setq rel-1 "abc/def")
;;; Therefore joiner is     (concat dir-1 "/" rel-1)
;;; Shell script joiner is  $ABC/$DEF or $ABC/foo (note $ABCfoo is an error)
;;; Makefile joiner is      $$ABC/$$DEF or $$ABC/foo (note $$ABCfoo is an error)
;;; Makefile joiner is      $(ABC)/$(DEF) or $(ABC)/foo (note $(ABC)foo works without error)
;;;
;;; BUG: d: denotes the current directory in the d drive in some cases (outside of Emacs)
;;;

;; (setq to "~/src/davin/research/2004")
;; (setq from "~/four")
;;(safe-make-link "~/src/davin_pearson/research/2004" "~/four")
(defun safe-make-link (to from)
  ;;(debug "smeg")
  (assert (stringp to))
  (assert (stringp from))
  (if (and (file-exists-p to)
           (not (file-exists-p (concat from ".lnk"))))
      (shell-command (concat "ln -s " (expand-file-name to) " " (expand-file-name from)))))

(defun safe-window-restore ()
  (if (fboundp 'w32-send-sys-command)
      (w32-send-sys-command 61728)
    )
  )

(defun safe-window-maximise ()
  (if (fboundp 'w32-send-sys-command)
      (w32-send-sys-command 61488)
    )
  )

;;; (d-ding)
(defun d-ding ()
  (let ((visible-bell t))
    (beep))
  )

;;(message "d4")
;; (d-nbeeps 3 "hello joe")
(defun d-nbeeps (n &rest msg)
  (let ((string (concat "*** BEEPS: " (apply 'format msg))))
    (decf n)
    (decf n)
    (d-ding)
    (message string)
    (sleep-for 1)
    (while (> n 0)
      (d-ding)
      (sleep-for 1)
      (decf n))
    (ding t)))

(defun d-beeps (msg &rest rest)
  "Defined by me"
  ;;(debug "Castaway")
  (let ((d-message-on t))
    ;;(d-debug "Antonio Vivaldi / The Four Seasons / Summer")
    (d-ding)
    (message "*** BEEPS: %s" (apply 'format msg rest))
    (sleep-for 1)
    (d-ding)
    (message "")
    (sleep-for 1)
    (message "*** BEEPS: %s" (apply 'format msg rest))
    (d-ding)))

(defun d-beep (msg &rest rest)
  "Defined by me"
  (let ((d-message-on t))
    (d-ding)
    (message "*** BEEPS: %s" (apply 'format msg rest))
    (sleep-for 1)
    ))

;;; (prin1-to-string (setq foo 'bar))

;; (safe-load-file "~/dlisp/imported-stuff/tabbar.el")
;; (safe-command (safe-load-file "~/dlisp/imported-stuff/tabbar.el"))

(defun safe-load-library (filename)
  (let ((cmd (list 'load-library filename)))
    (safe-command (eval cmd))))

(defun safe-find-file (file)
  (if (file-exists-p file)
      (if (fboundp 'd-find-file)
          (d-find-file file)
        (find-file file))))

(defun safe-downcase (string)
  (if string
      (downcase string)
    nil))

;; (safe-command (forward-sexp 1))
;; (safe-command (message "foo"))
;; (safe-command (safe-load-file "~/dlisp/imported-stuff/zzdotemacs.el"))
;; (safe-command (setq cmd (make-directory "/media/www/G16GB/home/bak/" 'parents)))
;; (safe-command (setq cmd (make-directory "/media/www/BLU16GB/home/bak/" 'parents)))
(defmacro safe-command (cmd &optional quietp)
  ;;(debug "Tuesday")
  `(condition-case err
       ,cmd
     (error
      (if (not ,quietp)
          (d-beeps "Command: %s failed, reason = %s, current-buffer = %s"
                   (quote ,cmd)
                   err
                   (buffer-name)
                   )))))

(defun read-ready (str)
  (and (stringp str) (not (string= "" str))))

(defun d-current-line-as-string ()
  (buffer-substring-no-properties (point-at-bol) (point-at-eol)))

(defun string<-ignore-case (str1 str2)
  (string< (upcase str1) (upcase str2)))

(global-set-key [(control l)] 'd-recenter)

(defun d-recenter ()
  (interactive)
  (recenter)
  (progn
    (safe-window-restore)
    (safe-window-maximise)
    )
  )

(global-set-key "\C-b" 'd-kill-buffer)

(defun kill-buffer-no-questions (&optional buf)
  (interactive)
  (set-buffer-modified-p nil)
  (kill-buffer buf)
  )

(defun d-kill-buffer ()
  (interactive)
  (unwind-protect
      (if (or (string=      (buffer-name) "*Messages*")
              (string=      (buffer-name) "*scratch*")
              (string=      (buffer-name) "*compilation*")
              (string-match "TAGS\\(<.*>\\)?$" (buffer-name))
              (eq major-mode 'dired-mode))
          (bury-buffer)
        (kill-buffer nil))
    ;;(if (fboundp 'd-speedbar--kill-speedbar-buffers)
    ;;    (run-with-timer 1 nil 'd-speedbar--kill-speedbar-buffer))
    ))

(define-key
  global-map
  [tool-bar kill-buffer]
  '(menu-item
    "Maybe Kill Buffer"
    d-kill-buffer
    :enable 1
    :visible 1
    :help "Maybe Kill Buffer"
    :image (image :type xpm :file "../etc/images/close.xpm")
    ))

(defun d-cadar (list)
  (car (cdr (car list))))

(global-set-key [f2] 'd-f2-save)

(defun d-f2-save ()
  "Kills some \"\\*.*\\*\" buffers"
  (interactive)
  (message "Welcome to the defun d-f2-save")
  (let (ptr list)
    (delete-other-windows)
    (save-some-buffers 'NO-QUESTIONS)
    (setq list (buffer-list))
    (setq ptr list)
    (while ptr
      (if (and (buffer-file-name (car ptr))
               (not (string-match "^ " (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*scratch*") (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*Messages*")  (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*Help*")      (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*info*")      (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*Backtrace*") (buffer-name (car ptr))))
               (not (string-match (regexp-quote "*grep*")      (buffer-name (car ptr))))
               (save-excursion
                 (set-buffer (car ptr))
                 (if (eq major-mode 'dired-mode)
                     t)))
          (progn
            ;;(debug "Foomatic")
            (kill-buffer)))
      (setq ptr (cdr ptr)))))

(global-set-key [f4] 'd-f4-save)

(defun d-f4-save ()
  (interactive)
  (let (ptr list)
    ;;(delete-other-windows)
    (save-some-buffers 'NO-QUESTIONS)
    (setq list (buffer-list))
    (setq ptr list)
    (while ptr
      (if (buffer-file-name (car ptr))
          (kill-buffer (car ptr)))
      (setq ptr (cdr ptr)))))

(safe-require 'ls-lisp)

;;;
;;; (find-file "~/dlisp/keys.el")
;;; (d-currently-editing-file "d:\\home\\dlisp\\keys.el")
;;; (d-currently-editing-file "d:\\home\\not\\editing.el")
;;; (d-currently-editing-file "d:\\HOME\\DLISP\\DIAGNOSE.EL")
;;;
(defun d-currently-editing-file (filename)
  "If not editing that filename, retures nil
otherwise returns the buffer object corresponding to the given filename."
  (setq filename (safe-expand-file-name filename))

  (if os-type-microsoft-p
      (setq filename (safe-downcase filename)))

  (let* ((list  (buffer-list))
         (ptr   list)
         (found nil))

    (while (and ptr (not found))
      (let ((curname (buffer-file-name (car ptr))))
        (setq curname (safe-expand-file-name curname))
        (if os-type-microsoft-p (setq curname (safe-downcase curname)))
        (if (string= curname filename)
            (setq found (car ptr))))
      (setq ptr (cdr ptr)))
    found))

(defun d-what-line ()
  "Different from what-line in that it returns the result rather
than printing it"
  (save-excursion
    (beginning-of-line)
    (1+ (count-lines 1 (point)))))

;;;
;;; COOL:
;;;
(defun d-splat (old-text new-text)
  (save-match-data
    (save-excursion
      (goto-char (point-min))
      (while (search-forward old-text nil t)
        (replace-match new-text nil nil)))))

;;;
;;; COOL:
;;;
(defun d-splat-re (old-text new-text)
  (save-match-data
    (save-excursion
      (goto-char (point-min))
      (while (re-search-forward old-text nil t)
        (replace-match new-text nil nil)))))

(add-hook 'c-mode-common-hook 'd-patch-control-f)

(defun d-patch-control-f ()
  (local-set-key "\C-d" 'd-control-f)
  (local-set-key "\C-f" 'd-control-f)
  )

(defun d-control-f (filename)
    "Fixed the bug where you try to load the same file into Emacs, instead loads the directory into dired mode."
    (interactive "FFind file/folder: ")
    (if (string-match "/$" filename)
        (find-file (substring filename 0 -1))
      (find-file filename))
    ;;  (if (string= (safe-expand-file-name (buffer-file-name))
    ;;               (safe-expand-file-name filename))
    ;;      (dired (file-name-directory filename))
    ;;    (find-file filename)))
    )

 ;; function keys

(global-set-key [(control f1)] 'd-control-f1)

(defun d-control-f1 ()
  (interactive)
  (setq p1 (point))
  (setq p2 (car mark-ring))
  (kill-region p1 p2)
  (delete-region p1 p2))

(defun d-f3 ()
  (interactive)
  (save-some-buffers 'NO-QUESTIONS)
  )

(global-set-key [f3] 'd-f3)

;;d-assert d-debug d-error
;;assert debug error

(if (not (assq 'debug-on-error minor-mode-alist))
    (setq minor-mode-alist (cons '(debug-on-error " DEBUG")
                                 minor-mode-alist)))

(defun d-kill-buffer ()
  (interactive)
  (d-save-some-buffers 'NO-QUESTIONS)
  (unwind-protect
      (progn
        (if (or (string=      (buffer-name) "*Messages*")
                (string=      (buffer-name) "*scratch*")
                (string=      (buffer-name) "*compilation*")
                (string-match "TAGS\\(<.*>\\)?$" (buffer-name))
                (eq major-mode 'dired-mode))
            (bury-buffer)
          (kill-buffer nil)))
    (kill-emacs)
    ;;(if (fboundp 'd-speedbar--kill-speedbar-buffers)
    ;;    (run-with-timer 1 nil 'd-speedbar--kill-speedbar-buffer))
    ))

(define-key
  global-map
  [tool-bar kill-buffer]
  '(menu-item
    "Maybe Kill Buffer"
    d-kill-buffer
    :enable 1
    :visible 1
    :help "Maybe Kill Buffer"
    :image (image :type xpm :file "../etc/images/close.xpm")
    ))

(progn
  (global-set-key "\C-b" 'save-buffers-kill-emacs)
  (global-set-key [(pause)] 'save-buffers-kill-emacs)
  ;;(global-unset-key "\C-x\C-c")
  )

(defun d-current-line-as-string ()
  (buffer-substring-no-properties (point-at-bol)
                                  (point-at-eol)))

(defun d-current-line-width ()
  (length (d-current-line-as-string)))

;; (setq str1 "abc")
;; (setq str2 "Abc")
;; (string=-ignore-case str1 str2)
(defun string=-ignore-case (str1 str2)
  (string= (upcase str1) (upcase str2)))

(defun string<-ignore-case (str1 str2)
  (string< (upcase str1) (upcase str2)))

(defun d-current-buffer-to-lines ()
  (let ((answer nil))
    (save-excursion
      (goto-char (point-min))
      (while (< (point) (point-max))
        (setq answer (cons (d-current-line-as-string) answer))
        (forward-line 1)))
    (reverse answer)))

(defun d-sort-buffer ()
  (interactive)
  (let ((list))
    (setq list (d-current-buffer-to-lines))
    (setq list (stable-sort list 'string<))
    (erase-buffer)
    (let ((ptr list))
      (while ptr
        ;;(debug)
        (insert (car ptr) "\n")
        (setq ptr (cdr ptr))))))

(defun d-delete-line ()
  (delete-region (point-at-bol) (point-at-eol))
  (if (looking-at "\n")
      (delete-char 1))
  )

(defun d-delta-looking-at (regexp delta)
  (save-excursion
    (forward-char delta)
    (looking-at regexp)))

(defun d-font-lock-remove-keywords ()
  (if (fboundp 'font-lock-add-keywords)
      (font-lock-add-keywords nil nil 'set)
    (setq font-lock-keywords nil)))

(defun d-foo (&optional who-cares)
  "Useful function for debugging"
  (interactive)
  (message "###############")
  (sleep-for 0.1)
  (message "               ")
  (sleep-for 0.1)
  (message "###############")
  (sleep-for 0.1)
  (message "               ")
  (sleep-for 0.1)
  (message "###############")
  (sleep-for 0.1)
  (message "               ")
  nil
  )

(defun d-insert-prin1 (x)
  (insert (prin1-to-string x) "\n"))

(defun d-is-first-line ()
  "Faster than (eq (d-what-line) 1)"
  (save-excursion
    (beginning-of-line)
    (bobp)))

(defun d-is-last-line ()
  (save-excursion
    (end-of-line)
    (eobp)))

;;; (d-kill-adjacent-duplicates '("abc" "abc" "abc" "def" "abc" "abc"))
;;; (d-kill-adjacent-duplicates '("abc" "abc" "abc" "abc" "abc" "abc" "def" "abc" "abc"))
;;; (d-kill-adjacent-duplicates '("abc" "abc" "abc" "abc" "abc" "abc" "def" "abc" "abc"))
(defun d-kill-adjacent-duplicates (list)
  ;;(setq list '("abc" "abc" "def" "abc"))
  (let ((ptr list))
    (while ptr
      (while (string= (car ptr) (cadr ptr))
        ;;(debug)
        (setcar ptr (cadr ptr))
        (setcdr ptr (cddr ptr)))
      (setq ptr (cdr ptr)))
    list))

;;(defun d-kill-adjacent-duplicates (list)
;;  (d-kill-adjacent-duplicates--inner (d-kill-adjacent-duplicates--inner list)))

;;;
;;; (d-last-string-match "src" "~/hairy-lemon/src/davin/webdesign/tutorial-4/src/site/index.hts") ;; yes
;;; (d-last-string-match "src" "~/hairy-lemon/src/davin/webdesign/tutorial-4/not/site/index.hts") ;; yes
;;; (d-last-string-match "src" "~/hairy-lemon/not/davin/webdesign/tutorial-4/not/site/index.hts") ;; no
;;;
(defun d-last-string-match (regexp string)
  (let ((i    (length string))
        (done nil))
    (while (and (>= i 0) (not done))
      (if (string-match regexp string i)
          (setq done t))
      (decf i))
    done))

;;(defun d-what-line ()
;;  (read (substring (what-line) 5)))

;;(global-set-key [kp-enter] 'd-random-play-emacs-midi)


 ;; HISTORY VARIABLES

;;(checkpoint "d7")

(if (or (not (boundp 'compile-history))
        (not compile-history))
    (setq compile-history '("make ")))

(setq file-name-history (cons (if emacs-dialect-gnuemacs-p
                                  (concat (getenv "HOME") "/bak/")
                                "~/bak/") file-name-history))

(if (string= (user-real-login-name) "root")
    (setq file-name-history (cons "~www/" file-name-history)))

(if emacs-dialect-gnuemacs-p
    (setq file-name-history (cons "~/" file-name-history)))

(if (boundp 'safe-compress-file-name)
    (setq file-name-history (cons (concat (safe-compress-file-name "~/dlisp") "/") file-name-history)))

(when (and emacs-dialect-gnuemacs-p os-type-microsoft-p)
  (setq file-name-history (cons "c:/" file-name-history))
  ;;(setq file-name-history (cons (concat (safe-compress-file-name (getenv "HOME")) "/") file-name-history))
  )

;;(if os-type-linux-p (setq file-name-history (cons "~/src/davin/research/2004/" file-name-history)))

(when emacs-dialect-gnuemacs-p
  ;;(setq extended-command-history (cons "eval-buffer" extended-command-history))
  (setq extended-command-history (cons "meal-timer" extended-command-history))4
  (setq extended-command-history (cons "bak" extended-command-history))
  (setq extended-command-history (cons "grep" extended-command-history))
  (setq dired-shell-command-history
	(if (boundp 'dired-shell-command-history)
	    (cons "du -h" dired-shell-command-history)
	  '("du -h")))
  )

(when emacs-dialect-xemacs-p
  ;;(setq read-command-history (cons "eval-buffer" read-command-history))
  (setq read-command-history (cons "bak" read-command-history))
  (setq read-command-history (cons "grep" read-command-history))
  )

;;;
;;; NOTE: mouse wheel in XEmacs
;;;
(if emacs-dialect-xemacs-p
    (when window-system
      ;; enable wheelmouse support by default
      (mwheel-install)
      (setq mouse-wheel-scroll-amount '(2 . 1))
      ;; use extended compound-text coding for X clipboard
      (set-selection-coding-system 'compound-text-with-extensions)
      (set-scroll-bar-mode 'right)
      (set-frame-name (concat "Gnu Emacs " (capitalize (user-real-login-name))))))

 ;; LINKS and CRUD

;;(checkpoint "d8")

(defun d-make-links ()
  (when os-type-microsoft-p
    ;;(d-make-links--year "1999")
    ;;(d-make-links--year "2003")
    ;;(d-make-links--year "2004")
    ;;(d-make-links--year "2005")
    ;;(safe-make-link (concat webdata  "/src/davin/") "~/davin")
    ;;(safe-make-link (concat "~/zallegro/2004/Tritus-II/")    "~/tritus")
    ;;(safe-make-link (concat "~/zallegro/2005/dd3/")          "~/dd3")
    ;;(safe-make-link (concat "~/zallegro/2005/tests/")        "~/tests")
    )

  (when os-type-linux-p
    ;;(safe-make-link "/usr/share/emacs/site-lisp/dlisp/" "~/dlisp")
    ;;(safe-make-link "/bak-dos"  "~/bak-dos")
    ;;(safe-make-link "/bak-unix" "~/bak-unix")
    ;;(safe-make-link "~davin/src/davin/research/2003" "~/3")
    ;;(safe-make-link "~davin/src/davin/research/2004" "~/4")
    ;;(if (string= (user-real-login-name) "root")
    ;;    (safe-make-link "~davin/web" "~/web"))
    ;;(safe-make-link "/mnt/cdrom" "~/cdrom")
    )
  )

(d-make-links)

(if (not (assq 'debug-on-error minor-mode-alist))
    (setq minor-mode-alist (cons '(debug-on-error " DEBUG") minor-mode-alist)))

(when prefs-home-emacs-p
  ;;(setenv "WEBDATA" "c:/home/hairy-lemon")
  (setq webdata "~/hairy-lemon"))

(global-font-lock-mode 1)
(add-hook 'text-mode-hook 'turn-on-font-lock)

(recenter)

 ;; defuns

;;
;; NOTE: needed for sensible bindings of CAPS LOCK key
;;
(defun d-xmodmap ()
  (interactive)
  (save-window-excursion
    (shell-command (concat "xmodmap ~www/dlisp/caps/xmodmaprc"))
    (if (get-buffer "*Shell Command Output*")
        (kill-buffer "*Shell Command Output*"))
    (message "Ran xmodmap ~www/dlisp/caps/xmodmaprc")
    ;;(sit-for 1)
    ))

(setq file-name-history (append '("~/" "~/dlisp/" "~/lisp++-projects/") file-name-history))

(progn
 (setq bg-colour nil)
 (setq bg-colour-lighter nil)
 (setq bg-colour-inverted nil)
 (setq bg-colour-lighter-lighter nil)
 (setq prefs-bg-black-p nil)
 )

(require 'cl)

;;(setq load-path (cons "~www/dlisp/" load-path))

(progn
  (global-set-key [S-down-mouse-1] 'ignore)
  (global-set-key [C-down-mouse-2] 'ignore)

  (global-set-key [f1]       'info)
  (global-set-key [f2]       (function
                              (lambda ()
                                (interactive)
                                (save-some-buffers 'NO-QUESTIONS)
                                (message "Saved some buffers...")
                                (sit-for 1)
                                (d-kill-some-buffers 'NO-QUESTIONS))))
  (global-set-key [f3]       (function
                              (lambda ()
                                (interactive)
                                (save-some-buffers 'NO-QUESTIONS))))
  (global-set-key [f4]       'delete-other-windows)
  (global-set-key [f7]       'read-only-mode)
  (global-set-key [f8]       'undo)
  (global-set-key [f12]      'eval-last-sexp)
  (global-set-key [insert]   'electric-buffer-list)
  (progn
    (global-set-key [backspace] 'backward-delete-char)
    (global-set-key [delete]    'delete-char)
    (global-set-key [(control delete)] 'kill-word)
    ;;(global-set-key [(meta    delete)] 'kill-word)
    ;;(global-set-key [(shift   delete)] 'kill-word)
    )

  (defun d-find-file0 (f)
    (interactive "FFind file/folder: ")
    (if (string-match "\\*" f)
        (find-file f 'WILDCARDS)
      (find-file f)))

  (defalias 'd-find-file 'd-find-file0)

  (progn
    (global-set-key "\C-d"      'd-find-file0)
    (global-set-key "\C-f"      'd-find-file0)
    (global-set-key "\C-x\C-f"  'd-find-file0)
    (global-set-key "\C-x\C-d"  'd-find-file0)
    (global-set-key "\C-xf"     'd-find-file0)
    (global-set-key "\C-xd"     'd-find-file0)
    (global-set-key [(control tab)] 'other-window)
    ))

(global-set-key [insert] 'electric-buffer-list)

;; -------------------------------------------------

(progn
  (global-set-key [(control right)] 'forward-sexp )
  (global-set-key [(meta right   )] 'forward-sexp )
  (global-set-key [(shift right  )] 'forward-sexp )
  (global-set-key [(control left )] 'backward-sexp)
  (global-set-key [(meta left    )] 'backward-sexp)
  (global-set-key [(shift left   )] 'backward-sexp)
  ;; ------------------------------------------------
  (global-set-key [(control w)] nil)
  (global-set-key "\C-e" 'call-last-kbd-macro)
  (global-set-key "`" 'dabbrev-expand)
  )

(global-set-key "\C-xg"    'goto-line)
(global-set-key "\C-x\C-g" 'goto-line)
(global-set-key "\C-ha"    'apropos) ;; NOTE: better than command-apropos
;;(global-set-key "\C-ha"    'command-apropos)
;;(global-set-key "\C-x\C-q" 'save-buffers-kill-emacs)
;;(global-unset-key "\C-x\C-q" )
;;(Global-set-key "\C-x\C-c" (function (lambda () (interactive) (message "Press \C-x\C-q to quit") (beep) (sleep-for 1))))
;;(global-unset-key "\C-x\C-c" )

(global-set-key "\C-d" 'd-control-f)
(global-set-key "\C-f" 'd-control-f)

(defun d-patch-control-d ()
  (local-set-key "\C-d" 'd-control-f)
  (local-set-key "\C-f" 'd-control-f)
  )

(global-set-key "\C-s" 'd-control-s)

(defun d-control-s ()
  (interactive)
  (let ((table (syntax-table))
        (case-fold-search t))
    (unwind-protect
        (progn
          ;; NOTE: this line makes it work...
          (set (make-local-variable 'font-lock-syntax-table) table)
          (if (and (boundp 'c-mode-syntax-table) c-mode-syntax-table)
              (set-syntax-table c-mode-syntax-table))
          (isearch-forward))
      (set-syntax-table table))))

(global-set-key "\M-\C-s" 'd-meta-control-s)

(defun d-meta-control-s ()
  (interactive)
  (let ((table (syntax-table))
        (case-fold-search t))
    (unwind-protect
        (progn
          ;; NOTE: this line makes it work...
          (set (make-local-variable 'font-lock-syntax-table) table)
          (if (and (boundp 'c-mode-syntax-table) c-mode-syntax-table)
              (set-syntax-table c-mode-syntax-table))
          (isearch-forward t))
      (set-syntax-table table))))

(global-set-key "\C-r" 'd-control-r)

(defun d-control-r ()
  (interactive)
  (let ((table (syntax-table))
        (case-fold-search t))
    (unwind-protect
        (progn
          ;; NOTE: this line makes it work...
          (set (make-local-variable 'font-lock-syntax-table) table)
          (if (and (boundp 'c-mode-syntax-table) c-mode-syntax-table)
              (set-syntax-table c-mode-syntax-table))
          (isearch-backward))
      (set-syntax-table table))))

(global-set-key "\M-\C-r" 'd-meta-control-r)

(defun d-meta-control-r ()
  (interactive)
  (let ((table (syntax-table))
        (case-fold-search t))
    (unwind-protect
        (progn
          ;; NOTE: this line makes it work...
          (set (make-local-variable 'font-lock-syntax-table) table)
          (if (and (boundp 'c-mode-syntax-table) c-mode-syntax-table)
              (set-syntax-table c-mode-syntax-table))
          (isearch-backward t))
      (set-syntax-table table))))

(let ((b (generate-new-buffer "*eraseme*")))
  ;;
  ;; NOTE: ensures that c-mode-syntax-table is bound for C-s / C-r
  ;;
  (set-buffer b)
  (c++-mode)
  (kill-buffer b)
  )

(global-set-key "\C-w" 'really-cut)

(defun really-cut ()
  (interactive "*")
  (if (y-or-n-p "Really cut? ")
      (call-interactively 'kill-region)))

(defun insert-prin1 (args)
  (insert (prin1-to-string args)))

;; (setq x 32.456)
(defun d-show (x)
  (kill-sexp 2)
  (insert (format " %5.2f " (/ (round (* x 100)) 100.0))))

(defun d-dollar (x)
  (kill-sexp 2)
  (insert (format " $%5.2f " (/ (round (* x 100)) 100.0))))

(defun d-f1 ()
  (interactive)
  (info)
  (delete-other-windows))

(global-set-key [f1] 'd-f1)

(defun d-meta-f1 ()
  (interactive)
  (kill-ring-save (point-min) (point-max))
  (message "Copied buffer to clipboard"))

(global-set-key [(meta f1)] 'd-meta-f1)
(global-set-key [(shift f1)] 'd-meta-f1)
(global-set-key [(control f1)] 'd-meta-f1)
(global-set-key [(f2)] 'd-f2)

(defun d-f2 ()
  (interactive)
  (save-excursion
    (save-window-excursion
      (message "Saving some buffers")
      (save-some-buffers 'NO-QUESTIONS))))

(global-set-key [f4] 'delete-other-windows)
(global-set-key [f8] 'undo)

(global-set-key [f9] 'd-f9)

(defun d-f9 ()
  (interactive)
  (save-some-buffers 'NO-QUESTIONS)
  (execute-kbd-macro "\M-xcompile\n"))

(global-set-key [(control f9)] 'd-shift-f9)
(global-set-key [(meta    f9)] 'd-shift-f9)
(global-set-key [(shift   f9)] 'd-shift-f9)

(defun d-shift-f9 ()
  (interactive)
  (save-some-buffers 'NO-QUESTIONS)
  (call-interactively 'compile)
  )

(assert (boundp 'prefs-advanced-user-p))

(if prefs-advanced-user-p
    (global-set-key [(f12)] 'eval-last-sexp)
  (global-set-key [(f12)] 'electric-buffer-list))

;;(global-set-key [f12] 'backward-delete-char-untabify) ;; NOTE: was here

(progn
  ;;(global-unset-key [kp-delete])
  (global-set-key [kp-delete] 1) ;;; NOTE: better than the above way to unset the key
  )

(when prefs-advanced-user-p
  (global-set-key "\e\e" 'eval-expression)
  (global-set-key "`" 'dabbrev-expand)
  ;; for when you miss the real key
  (global-set-key "\e`" 'dabbrev-expand)
  (global-set-key "'" 'd-quote-key)
  (global-set-key "\M-/" 'dabbrev-expand)
  (global-set-key "¿" 'dabbrev-expand)

  (global-set-key "\"" 'self-insert-command)
  ;;(global-set-key "`" 'self-insert-command)

  (defun d-quote-key ()
    (interactive)
    (if (eq (char-after (1- (point))) ?')
        (progn
          (backward-delete-char 1) ;; varsity fixed:
          (insert "`"))
      (if (eq (char-after (1- (point))) ?`)
          (progn
            (backward-delete-char 1) ;; varsity fixed:
            (insert "'"))
        (self-insert-command 1))))

  )

(if (not prefs-windows-bindings-p)
    (global-set-key [(shift   up)] 'd-shift-up))

(global-set-key [(control up)]          'd-shift-up)
(global-set-key [(meta    up)]          'd-shift-up)
(global-set-key [(prior)]               'd-shift-up)

(defun d-shift-up ()
  (interactive)
  (scroll-down 2))

(global-set-key [(control down)]        'd-shift-down)
(global-set-key [(meta    down)]        'd-shift-down)
(if (not prefs-windows-bindings-p)
    (global-set-key [(shift   down)]        'd-shift-down))

(global-set-key [(next)]                'd-shift-down)

(defun d-shift-down ()
  (interactive)
  (scroll-up 2))

(global-set-key [(control next)]        'scroll-up)
(global-set-key [(meta    next)]        'end-of-defun)
(global-set-key [(shift   next)]        'd-forward-page)

(defun d-forward-page ()
  (interactive)
  (if (fboundp 'd-deposit-mark-if-small-movement) (d-deposit-mark-if-small-movement))
  (if (not (re-search-forward "^[ \t]*" nil t))
      (goto-char (point-max)))
  ;;(forward-page)
  (if (= (point) (point-max))
      (recenter (- (window-height) 4))
    (recenter (/ (window-height) 6))))

(global-set-key [(control prior)]       'scroll-down)
(global-set-key [(meta    prior)]       'beginning-of-defun)
(global-set-key [(shift   prior)]       'd-backward-page)

(defun d-backward-page ()
  (interactive)
  (beginning-of-line)
  (if (not (re-search-backward "^[ \t]*" nil t))
      (goto-char (point-min)))
  (skip-chars-forward " \t")
  (skip-chars-forward "")
  (if (fboundp 'd-deposit-mark-if-small-movement) (d-deposit-mark-if-small-movement))
  ;;(backward-page)
  (recenter (/ (window-height) 6)))

(progn
  (global-set-key [(control meta prior)]  'd-backward-page)
  (global-set-key [(control meta next)]   'd-forward-page))

(defun d-control-prior ()
  (interactive)
  (d-scroll-up (- (window-height) 2)))

(defun d-control-next ()
  (interactive)
  (d-scroll-down (- (window-height) 2)))

(defun d-scroll-up (amount)
  "Used by d-control-meta-prior"
  (scroll-down amount)
  (forward-line (- amount))
  (move-to-window-line (/ (window-height) 2)))

(defun d-scroll-down (amount)
  "Used by d-control-meta-next"
  (if (/= (point) (point-max))
      (progn
        (scroll-up amount)
        (forward-line amount)))
  (move-to-window-line (/ (window-height) 2)))

(progn
  (global-set-key [(control home)]      'beginning-of-buffer)
  (global-set-key [(control end)]       'end-of-buffer)
  (global-set-key [(meta home)]         'beginning-of-buffer)
  (global-set-key [(meta end)]          'end-of-buffer)
  (global-set-key [(shift home)]        'beginning-of-buffer)
  (global-set-key [(shift end)]         'end-of-buffer)
  )

(global-set-key [(control right)]       'forward-sexp)
(global-set-key [(meta    right)]       'forward-sexp)
(global-set-key [(shift   right)]       'forward-sexp)
(global-set-key [(control meta right)]  'forward-sexp)

(global-set-key [(control left)]        'backward-sexp)
(global-set-key [(meta    left)]        'backward-sexp)
(global-set-key [(shift   left)]        'backward-sexp)
(global-set-key [(control meta left)]   'backward-sexp)

(global-set-key [(control backspace)]   'backward-kill-word)
(global-set-key [(meta    backspace)]   'backward-kill-word)
(global-set-key [(shift   backspace)]   'backward-kill-word)

(global-set-key [(control delete)]      'kill-word)
(global-set-key [(meta    delete)]      'kill-word)
(global-set-key [(shift   delete)]      'kill-word)

(if prefs-advanced-user-p
    (global-set-key [insert] 'electric-buffer-list)
  )

(global-set-key [(control tab)]         'other-window)
(global-set-key [(shift   tab)]         'other-window)

(global-set-key [(control ?0)]          'delete-window)
(global-set-key [(control ?1)]          'delete-other-windows)
(global-set-key [(control ?@)]          'split-window-vertically) ;; NOTE: Only applicable to Dos Emacs
(global-set-key [(control ?2)]          'split-window-vertically)
(global-set-key [(control ?3)]          'split-window-horizontally)

(progn
  (global-set-key [(shift ? )] 'd-shift-space)
  (global-set-key [(s ?\s- )]  'd-s-space)

  (defun d-shift-space ()
    (interactive)
    (message "pop-mark")
    (pop-mark)
    (recenter)
    )

  (defun d-s-space ()
    (interactive)
    (message "push-mark")
    (push-mark)
    (recenter)
    )
  )

;; (setq prefs-advanced-user-p t)
(when prefs-advanced-user-p
  (defalias   'dtp 'describe-text-properties)
  ;;(defalias 'lta 'list-text-properties-at)  ;; version 21.3
  (defalias   'lta 'describe-text-properties) ;; version 22.1
  (defalias   'elm 'emacs-lisp-mode)
  (defalias   'flm 'font-lock-mode)
  (defalias   'qrr 'query-replace-regexp)

  ;;(defalias   'eb  'eval-buffer)
  (defalias   'hff 'hexl-find-file)
  )

;;(defalias 'd-buffer-substring 'buffer-substring-noproperties)

(defun tde ()
  (interactive)
  (message "tde")
  (if debug-on-error
      (setq debug-on-error nil)
    (setq debug-on-error t)))

(setq kill-whole-line t)

(setq file-name-history (cons "~/" file-name-history))

(setq c2java-version "1.21")

(blink-cursor-mode -1)

;;;
;;; NOTE: standard notation follows...
;;;
;;; File/directory spec is  (setq dir-1 "d:/home") or (setq dir-1 "d:") (note no trailing slash)
;;; File access is          (concat dir-1 "/" file-1)
;;; Relative directory spec (setq rel-1 "abc/def")
;;; Therefore joiner is     (concat dir-1 "/" rel-1)
;;; Shell script joiner is  $ABC/$DEF or $ABC/foo (note $ABCfoo is an error)
;;; Makefile joiner is      $$ABC/$$DEF or $$ABC/foo (note $$ABCfoo is an error)
;;; Makefile joiner is      $(ABC)/$(DEF) or $(ABC)/foo (note $(ABC)foo works without error)
;;;
;;; BUG: d: denotes the current directory in the d drive in some cases (outside of Emacs)
;;;
;;; (safe-expand-file-name nil)
;;; (safe-expand-file-name "d:/")
;;; (safe-expand-file-name (setq file "d:/home/"))
;;; (safe-expand-file-name "d:/home")
;;; (safe-expand-file-name "d:/home/.emacs")
;;; (safe-expand-file-name "d:/home//text/..")
;;;
;;; NOTE: never returns a trailing slash
;;;
;;; (setenv "HOME" "d:/")
;;; (safe-expand-file-name "d:/")
;;; (safe-expand-file-name "d:/frog")
;;; (safe-expand-file-name "d:/FROG")
;;; (safe-expand-file-name "~/.emacs")
;;; (safe-expand-file-name "~/dlisp")
;;;
(defun d-safe-expand-file-name (filename)
  "Replaces ~/ with expanded file name"
  (save-match-data
    (setq filename (if filename (expand-file-name filename)))
    ;;
    ;; NOTE: adds trailing slash
    ;;
    (if (and filename (not (string-match "/$" filename)))
        (setq filename (concat filename "/")))
    filename))

(when prefs-advanced-user-p
  (defalias   'dtp 'describe-text-properties)
  ;;(defalias 'lta 'list-text-properties-at)  ;; version 21.3
  (defalias   'dtp 'describe-text-properties) ;; version 22.1
  (defalias   'elm 'emacs-lisp-mode)
  (defalias   'flm 'font-lock-mode)
  (defalias   'qrr 'query-replace-regexp)

  (defalias   'eb  'eval-buffer)
  (defalias   'hff 'hexl-find-file)
  )

(setq d-default-directory default-directory)

(defun d-electric-buffer-list ()
  (interactive)
  ;;(d-delete-speedbar-window)
  (delete-other-windows)
  (call-interactively 'electric-buffer-list)
  )

(progn
  (global-set-key [insert]           'd-electric-buffer-list)
  (global-set-key [(control insert)] 'd-electric-buffer-list)
  (global-set-key [(shift insert)]   'd-electric-buffer-list)
  (global-set-key [pause]            'd-electric-buffer-list)
  (global-set-key [(control delete)]       'kill-word)
  ;;(global-set-key [(shift   delete)]       'kill-word)
  ;;(global-set-key [(alt     delete)]       'kill-word)
  (global-set-key [(control backspace)]    'backward-kill-word)
  (global-set-key [(shift   backspace)]    'backward-kill-word)
  (global-set-key [(alt     backspace)]    'backward-kill-word)
  (global-set-key [(control d)]            'find-file)
  (global-set-key [(control f)]            'find-file)
  (global-set-key [(control x)(control f)] 'find-file)
  (global-set-key [(control x)(control d)] 'find-file)
  (global-set-key [(control x)(f)]         'find-file)
  (global-set-key [(control x)(d)]         'find-file)
  (global-set-key [(f2)]                   'bury-buffer)
  (global-set-key [(f3)]                   'save-some-buffers)
  (global-set-key [(f4)]                   'delete-other-windows)
  (global-set-key [(f8)]                   'undo)
  (global-set-key [(f9)]                   'compile)
  (global-set-key [(f12)]                  'eval-last-sexp)
  (global-set-key [(control tab)]          'other-window)
  )

;; (print-symbol apple)
(defmacro print-symbol (symbol)
  (d-assert (symbolp 'symbol))
  (concat "(symbol "
          (prin1-to-string symbol)
          "="
          (if (boundp symbol)
              (prin1-to-string (eval symbol))
            "<unbound>")
          ")"))

(condition-case err
    (show-paren-mode t)
  (error
   (message "Cannot show parens %s" (cdr err))))

(defun d-recenter ()
  (interactive)
  (set-default-font "Courier New-18")
  (recenter)
  (safe-require 'd-make-faces)
  (if (fboundp 'd-make-faces)
      (d-make-faces)
    )
  )

(progn
  (global-set-key "\C-c\C-l" 'd-recenter)
  (global-set-key "\C-l"     'd-recenter)
  )

(add-to-list 'default-frame-alist '(fullscreen . maximized))

(provide 'early-bindings)



