
;;; jtw-build-class-db.el --- class database builder

;; Copyright (C) 2006-2017 Davin Pearson

;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: J.T.W. Java
;; Package: dlisp (Davin's version of elisp)
;; Version: 1.0

;;; 

;; This file is part of GNU Java Training Wheels.
;;
;; A system for building a database of functions, methods, properties
;; and classVars.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Known Bugs:

;; None so far!

;;; Code:

(require 'cl)

(message "*** Welcome to file: jtw-build-class-db.el")

(setq debug-on-error t)

;;; (setq *prefix* "~/dlisp")
(when (not (fboundp 'd-emergency-set-load-path))
  (defun d-emergency-set-load-path ()
    (assert (boundp '*prefix*))
    (assert *prefix*)
    (setq load-path (cons (expand-file-name (concat *prefix* "/../dlisp/"))
                          load-path))
    (message "** jtw-build-class-db.el (car load-path)=%s" (car load-path))))

(d-emergency-set-load-path)

(require 'early-bindings)

(message "file: jtw-build-class-db.el %s %s" (print-symbol *prefix*) (print-symbol *stump*))

(if (not (boundp 'file-comes-from))
    (setq file-comes-from nil))

(if (not file-comes-from)
    (setq file-comes-from (cons "jtw-build-class-db.el" file-comes-from)))

(setq debug-on-error t)

;;(when (not (boundp 'cadr))
;;  (let ((debug-on-error nil))
;;    (error "Error: Macro cadr is not bound")))

;;(when (not (boundp 'cl-assert))
;;  (let ((debug-on-error nil))
;;    (error "Error: Macro cl-assert is not bound")))

(progn
  (setq slot-class-name         0)
  (setq slot-class-or-interface 1)
  (setq slot-functions          2)
  (setq slot-classVars          3)
  (setq slot-methods            4)
  (setq slot-properties         5)
  )

;;(message "*stump*=%s"  (prin1-to-string *stump*))
;;(message "*prefix*=%s" (prin1-to-string *prefix*))
;;(d-assert (boundp '*stump*))
;;(d-assert *stump*)

;;(message "*** (car load-path)=%s" (car load-path))

(setq c-basic-offset 3)

;;(message "*** (car load-path)=%s" (car load-path))

(provide 'jtw-build-class-db)
(require 'd-flm)
(require 'd-defmacro)
(require 'jtw-build-java)
(require 'directory-files-deep)

;; (setq list (warn--build-method-database))
(defun warn--build-method-database ()
  (d-assert (boundp '*prefix*))
  (d-assert (boundp '*stump*))
  (d-assert *prefix*)
  (d-assert *stump*)
  (message (format (concat "Welcome to defun: warn--build-method-database "
                           "*prefix*=%s *stump*=%s")
                   *prefix* *stump*))
  (save-excursion
    (let (e file class str type package method-name list ptr name
            alist spaces spaces2 warn--master-list--firstly x1 x2
            x3 x4 find)
      (setq warn--master-list--firstly nil)
      ;;(d-assert (boundp '*stump*))
      (d-assert (boundp '*prefix*))
      (d-assert *stump*)
      (d-assert *prefix*)
      (message "Jose Carreras / Banana Split")
      ;;(message "Downs Syndrome *prefix*=%s" *prefix*)
      ;;(message "Downs Syndrome *stump*=%s"  *stump*)
      ;;(message "Sydney Opera House / *stump*=%s" (prin1-to-string *stump*))
      ;;(d-error "Downs Syndrome / Smeggerize Early exit!")
      (setq list (directory-files-deep *prefix* t "\\.jtw$"))
      ;; (message (format "* list=%s" list))
      ;; (setq x "#abc#")
      (setq list (mapcar (function
                          (lambda (x)
                            (if (string-match "^#.*#$" x)
                                nil x))) list))
      ;;(message "aa")
      (setq list (delq nil list))
      (setq ptr list)
      (while ptr
        (setq file-old (car ptr))
        (d-assert (string-match "[a-zA-Z0-9_]*\\.jtw$" file-old))
        (setq file-new (concat (substring file-old 0 -3) "java"))
        (setq e (d-currently-editing-file file-old))
        (switch-to-buffer (find-file file-old))
        ;;(debug "Robert Johnson: Hellhound on my trail")
        (goto-char (point-min))
        ;;(message "major-mode=%s" major-mode)
        (setq cbo c-basic-offset)
        (setq spaces (make-string cbo ? ))
        (setq spaces2 (concat "\\(" spaces "\\)"))
        (while (flm-re-search-forward--no-strings-no-comments
                (concat
                 "^" spaces2 "\\(\\(abstract\\|final\\|private\\|protected\\|public\\)"
                 "[ \t]+\\)*\\(method\\|property\\|function\\|classVar\\)\\>") nil t)
          (setq type (buffer-substring-no-properties (match-beginning 4) (match-end 4)))
          (skip-chars-forward " \t")
          (skip-chars-forward "a-zA-Z0-9_<,>[]")
          (skip-chars-forward " \t")
          (setq p (point))
          (skip-chars-forward "a-zA-Z0-9_")
          (if (looking-at "[ \t\r\n]*(")
              (setq was-bra t)
            (setq was-bra nil))
          ;;(message "was-bra=%s" was-bra)
          (setq name (buffer-substring-no-properties p (point)))
          (d-assert name)
	  ;;(debug "Carl Orff: Carmina Burana")
          (setq package (warn--find-enclosing-package))
          (setq find    (d-get-enclosing-class))
          (setq class-or-interface (nth 0 find))
          (setq class              (nth 1 find))
          (if (string-match "^\\.[A-Z][a-zA-Z0-9_]*" class)
              (setq class (substring class 1)))
          (when (and (not (string-match "\\.$" package)) (not (string= package "")))
            (setq package (concat package ".")))
          (when (and package (stringp package) (not (string= package "")) (not (string= package ".")))
            (setq class (concat package class))
            )
          (when (or (string= "pkg.inner" package) (string= "pkg.pkg.inner" package))
            ;;(debug "The Eagles / Hotel California")
            ;;(debug "The Cars / Just what I needed")
            )
          ;;(debug (format "That's fairly interesting name=%s" name))
          (when (null (assoc class warn--master-list--firstly))
            (setq x1 (list 'functions))
            (setq x2 (list 'classVars))
            (setq x3 (list 'methods))
            (setq x4 (list 'properties))
            ;;;
            ;;; WTF: is going on here?
            ;;;
            (setq warn--master-list--firstly (cons (list class class-or-interface x1 x2 x3 x4)
                                                   warn--master-list--firstly))
	    (d-assert (equal (list class class-or-interface '(functions) '(classVars) '(methods) '(properties)) (car warn--master-list--firstly)))
            )
          (when (string= class "Interface0")
            ;;(if (string= name "getName")  (debug "Wrong location"))
            ;;(if (string= name "insert")   (debug "Wrong location"))
            ;;(if (string= name "toString") (debug "Wrong location"))
            ;;(if (and debug-on-error (string= name "foo")) (debug "Wrong location"))
            )
          (when (string= class "Interface1")
            ;;(if (and debug-on-error (string= name "meth")) (debug "Miles Davis: Pangaea"))
            )
          ;; ---------------------------------------------------------
          ;;(if (and (string= type "method") (string= class "Turtle"))
          ;;    (error "schmu"))
          ;; ---------------------------------------------------------
          ;;(debug "AC-DC / Thunderstruck"))
          (d-assert name)
          ;;(debug "Elvis Presley: I Can't Help Falling in Love With You")
          (progn
            (setq alist (assoc class warn--master-list--firstly))
            (d-assert alist)
            alist
            )
          ;; ---------------------------------------------------------
          ;;(message (format "pre alist=%s" alist))
          ;;(assoc "Interface0" warn--master-list--firstly)
          ;;(assoc "Interface1" warn--master-list--firstly)
          (when (or (string= type "function") (string= type "classVar"))
            (if was-bra
                (progn
                  (setcdr (last (nth slot-functions alist)) (cons name nil))
                  (d-assert (string= type "function"))
                  )
              (setcdr (last (nth slot-classVars alist)) (cons name nil))
              (d-assert (string= type "classVar"))
              )
            )
          (when (or (string= type "method") (string= type "property"))
            (if was-bra
                (progn
                  ;;(when (and debug-on-error
                  ;;           (string= "Interface1" class))
                  ;;  (d-assert (string= name "meth"))
                  ;;  (debug "Toto: Africa"))
                  (setcdr (last (nth slot-methods alist)) (cons name nil))
                  (d-assert (string= type "method"))
                  )
              (setcdr (last (nth slot-properties alist)) (cons name nil))
              (d-assert (string= type "property"))
              )
            )
          (d-assert (string= (nth slot-class-or-interface alist) class-or-interface))
          ;;(message (format "post alist=%s" alist))
          ;;(message (format "assoc Turtle=%s"  (assoc "Turtle"  warn--master-list--firstly)))
          ;;(message (format "assoc AppleII=%s" (assoc "AppleII" warn--master-list--firstly)))
          ;;(message (format "warn--master-list--firstly=%s" warn--master-list--firstly))
          ;;(if (and debug-on-error (= (length warn--master-list--firstly) 2))
          ;;    (debug "Miles Davis / Doo Bop"))
          ) ;; END WHILE (WARN--RE-SEARCH-FORWARD--NO-STRINGS-NO-COMMENTS!
        (goto-char (point-min))
        (while (re-search-forward "\\<beginMain\\>" nil t)
          ;;(debug "Spiritualized / You know it's true")
          (progn
            (setq find (d-get-enclosing-class))
            (setq package (warn--find-enclosing-package)))
          (if (string-match "\\([A-Za-z.]+\\)$" package)
              (setq package (concat (substring package 0 (match-end 1)) ".")))
          (setq class (concat package (nth 1 find)))
          (if (assoc class warn--master-list--firstly)
              (progn
                (setcdr (last (nth slot-functions (assoc class warn--master-list--firstly)))
                        (cons "main" nil))
                ;;(debug "Pineapple")
                )
            (setq class-or-interface (nth 0 find))
            (setq x1 (list 'functions))
            (setq x2 (list 'classVars))
            (setq x3 (list 'methods))
            (setq x4 (list 'properties))
            ;;;
            ;;; WTF: is going on here?
            ;;;
            (setq warn--master-list--firstly (cons (list class class-or-interface x1 x2 x3 x4)
                                                   warn--master-list--firstly))
            ;;(setq warn--master-list--firstly (cons (list class '(functions) '(classVars) '(methods) '(properties))
            ;;                                       warn--master-list--firstly))
            (message (format "Created new term in warn--master-list--firstly (car warn--master-list--firstly=%s)" (car warn--master-list--firstly)))
            (d-assert (equal (list class class-or-interface '(functions) '(classVars) '(methods) '(properties)) (car warn--master-list--firstly)))
            (setcdr (last (nth slot-functions (car warn--master-list--firstly)))
                    (cons "main" nil))
            ;;(if (and (not (string= (caar warn--master-list--firstly) "MyFirstProgram"))
            ;;         (not (string= (caar warn--master-list--firstly) "MySecondProgram")))
            ;;    (debug "Calamansi"))
            )) ;; RE-SEARCH-FORWARD BEGINMAIN!
          ;;(error "Monsters, Inc")
          ;; ADD ME! x1
          (if e (kill-buffer))
        ;; REMOVE ME! x1
        ;;(setq ptr nil)
        (setq ptr (cdr ptr))) ;; END WHILE PTR!
      ;;(if debug-on-error (debug "Soprano"))
      (warn--trim-duplicates warn--master-list--firstly)
      ;;(debug "foo")
      (defun d-strings-equal-structs (x y)
        (string= x (car y)))
      (when (> (count class warn--master-list--firstly :test 'd-strings-equal-structs) 1)
        (setq debug-on-error nil)
        (warn--log-message "Error 49: class %s has multiple instances" class))
      ;; (debug "Queen / Freddie Mercury")
      (message "Leaving the defun: warn--build-method-database")
      warn--master-list--firstly
      ) ;; END LET!
    )   ;; END SAVE-EXCURSION!
  )     ;; DEFUN! warn--build-method-database

;; (setq zeg (warn--cull-imports (setq file "pkg/Beer.jtw") zog))
;; (setq file "MyFirstProgram.jtw")
;; (setq warn--master-list nil)
;; NOTE DEFINES: warn--master-list and warn--master-list--secondly
(defun warn--cull-imports (file warn--master-list)
  ;;(error "Should never happen #2")
  (save-excursion
    (let ((e (warn--are-we-editing-p file)) package count
          import-decl imported-packages ptr ptr2 ptr3 class
          class2 import-package-or-class args
          warn--master-list--secondly)
      ;; -------------------------------------------------------------
      (setq warn--master-list--secondly warn--master-list)
      ;;(debug "Forrest Gump: Peas and Carrots")
      (find-file file)
      (d-assert (string-match "\\.java$" file))
      (setq file (substring file 0 (match-beginning 0)))
      (goto-char (point-min))
      (setq package (warn--find-enclosing-package))
      (goto-char (point-min))
      (setq str "^[ \t]*import[ \t]+\\([a-z.]+\\)\\.\\*;")
      (setq imported-packages nil)
      (beginning-of-line)
      (setq count 0)
      (while (save-match-data
               (if (flm-re-search-forward--no-strings-no-comments str nil t)
                   (setq import-decl (buffer-substring-no-properties
                                      (match-beginning 1)
                                      (match-end 1)))))
        (setq imported-packages (cons import-decl imported-packages))
        ;;(message "Dawn Upshaw %s" count)
        (incf count)
        ) ;; END WHILE!
      (goto-char (point-min))
      (setq str "^[ \t]*import[ \t]+\\([a-z.]+\\.[A-Z][a-zA-Z0-9_]*\\);")
      (while (save-match-data
               (if (flm-re-search-forward--no-strings-no-comments str nil t)
                   (setq import-decl (buffer-substring-no-properties
                                      (match-beginning 1)
                                      (match-end 1)))))
        (setq imported-packages (cons import-decl imported-packages))
        ;;(message "Cheryl Studer %s" count)
        (incf count)
        ) ;; END WHILE!
      ;;(if debug-on-error (debug "Bryan Ferry: Love is the drug"))
      ;;(debug "Jean Michel Jarre: Oxygene")
      (setq count 0)
      (setq ptr imported-packages)
      (while ptr
        (when (car ptr)
          (setq import-package-or-class (car ptr))
          (when (string-match "^[a-z.]+\\.[A-Z][a-zA-Z0-9_]*;$" import-package-or-class)
            (setq ptr2 warn--master-list--secondly)
            ;;(setq ptr2 '(("pkg.Wine" class
            ;;              (functions "foo")
            ;;              (classVars "data")
            ;;              (methods)
            ;;              (properties))))
            (while ptr2
              (setq class (caar ptr2))
              (setq import-decl (concat "^\\(" (regexp-quote import-package-or-class) "\\)"))
              (when (string-match import-decl class)
                (setq class2 (substring class (match-end 1)))
                (setq ptr3 (copy-list (car ptr2)))
                (setf (caar ptr3) class2)
                (setq warn--master-list--secondly (cons ptr3 warn--master-list--secondly))
                (setq ptr2 (cdr ptr2))
                )))
          (when (string-match "^[a-z.]*$" import-package-or-class)
            (setq ptr2 warn--master-list--secondly)
            ;;(setq ptr2 '(("pkg.Wine" class
            ;;              (functions "foo")
            ;;              (classVars "data")
            ;;              (methods)
            ;;              (properties))))
            (while ptr2
              (setq class (caar ptr2))
              (setq import-decl (concat "^\\(" (regexp-quote import-package-or-class) "\\.\\)[A-Z][a-zA-Z0-9_]*$"))
              ;;(string-match "^\\(pkg\\)\\.[A-Z][a-zA-Z0-9_]*$" class)
              ;;(string-match "^\\(pkg\\)\\.Wine$"               class)
              ;;(string-match "^\\([a-z.]*\\)\\.Wine$"           class)
              (when (string-match import-decl class) ;; DEBUG ME INTERACTIVE MODE!
                ;;(setq whole  (substring class (match-beginning 0) (match-end 0)))
                ;;(setq pre    (substring class (match-beginning 0) (match-end 1)))
                (setf class2 (substring class (match-end 1)))
                ;;(debug "Light of the day")
                (setq ptr3 (copy-list (car ptr2)))
                (setf (car ptr3) class2)
                (setq warn--master-list--secondly (cons ptr3 warn--master-list--secondly))
                )
              (setq ptr2 (cdr ptr2)))
            ))
        (setq ptr (cdr ptr))
        )
      ;;; TODO: check for doubled up classes
      (goto-char (point-min))
      ;; -------------------------------------------------------------
      (while (flm-re-search-forward--no-strings-no-comments
              "[^/\n\r][^/\n\r][ \t]*\\(\\<[a-z.]*[A-Z][a-zA-Z0-9_]*\\)\\.\\([a-zA-Z_][a-zA-Z0-9_]*\\)[ \t]*(" nil t)
        (setq class    (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
        (setq function (buffer-substring-no-properties (match-beginning 2) (match-end 2)))
        (setq ptr (assoc class warn--master-list--secondly))
        (when ptr
          (when (null (find function (cdr (nth slot-functions ptr)) :test 'string=))
            (warn--log-message (format "Error 46: Function %s.%s() not found" class function))
            ;;(debug "The Beatles: The fool on the hill")
            )
          )
        )
      ;; -------------------------------------------------------------
      (goto-char (point-min))
      ;;(debug "Foomatic")
      (while (flm-re-search-forward--no-strings-no-comments
              "[^/\n\r][^/\n\r][ \t]*\\(\\<[a-z.]*[A-Z][a-zA-Z0-9_]*\\)\\.\\([a-zA-Z_][a-zA-Z0-9_]*\\)[ \t]*[^a-zA-Z0-9_( \t]" nil t)
        (setq class    (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
        (setq classVar (buffer-substring-no-properties (match-beginning 2) (match-end 2)))
        (setq ptr (assoc class warn--master-list--secondly))
        (when (not (string-match "^jtw$" classVar))
          (when ptr
            (when (null (find classVar (cdr (nth slot-classVars ptr)) :test 'string=))
              (warn--log-message (format "Error 47: ClassVar %s.%s not found" class classVar))
              ;;(debug "Townsville, Australia")
              )
            )
          )
        )
      ;; -------------------------------------------------------------
      (if (not e) (kill-buffer nil))
      ;;(debug "Wallow in my own crapulence")
      warn--master-list--secondly
      ) ;; END LET!
    ) ;; END SAVE-EXCURSION!
  )

(defun warn--trim-duplicates (list)
  (let (ptr class functions classVars methods properties)
    (setq ptr list)
    (while ptr
      (setq car (car ptr))
      (setq functions  (nth slot-functions  car))
      (setq classVars  (nth slot-classVars  car))
      (setq methods    (nth slot-methods    car))
      (setq properties (nth slot-properties car))
      (setf (nth slot-functions  car) (delete-duplicates functions  :test 'string=))
      (setf (nth slot-classVars  car) (delete-duplicates classVars  :test 'string=))
      (setf (nth slot-methods    car) (delete-duplicates methods    :test 'string=))
      (setf (nth slot-properties car) (delete-duplicates properties :test 'string=))
      (setq ptr (cdr ptr))
      )
    list
    )
  )

(defun load-class-db ()
  (save-excursion
    (let (str list)
      (message "begin defun: load-class-db *stump*=%s" *stump*)
      ;;(message "Freestyler")
      (d-assert *stump*)
      (d-assert *prefix*)
      ;;(error "Ricky Lee Jones: Kids looking for something")
      (find-file (concat *prefix* "/class-db.el"))
      (setq str (buffer-substring-no-properties (point-min) (point-max)))
      (if (> (length str) 0)
          (setq list (d-read-str str)))
      (setq list (warn--trim-duplicates list))
      ;;(d-assert (>= (length list) 1))
      ;;(message "list=%s" list)
      (message "ending defun: load-class-db")
      list)
    ))

(defun save-class-db ()
  (message "Welcome to the defun: save-class-db")
  (save-excursion
    (let (list ptr)
      (message "About to call warn--build-method-database")
      (setq list (warn--build-method-database))
      (message "Called warn--build-method-database")
      ;;(d-assert (boundp '*stump*))
      (d-assert (boundp '*prefix*))
      (d-assert *stump*)
      (d-assert *prefix*)
      ;;(d-assert (>= (length list) 1))
      ;; -------------------------------------------------------------
      ;;(message (format "About to save-class-db *stump*=%s" *stump*))
      ;;(find-file (concat *stump* "/class-db.el"))
      (find-file (concat *prefix* "/class-db.el"))
      (read-only-mode -1)
      (emacs-lisp-mode)
      (erase-buffer)
      (insert ";;; Automatically generated file.  Do not edit!\n")
      (setq ptr list)
      (setq count 0)
      (insert "(\n")
      (while ptr
        (setq class              (nth slot-class-name         (car ptr)))
        (setq class-or-interface (nth slot-class-or-interface (car ptr)))
        (setq functions          (nth slot-functions          (car ptr)))
        (setq classVars          (nth slot-classVars          (car ptr)))
        (setq methods            (nth slot-methods            (car ptr)))
        (setq properties         (nth slot-properties         (car ptr)))
        (insert " (" (prin1-to-string class)
                " "  (prin1-to-string class-or-interface) "\n")
        (insert "  " (prin1-to-string functions)          "\n")
        (insert "  " (prin1-to-string classVars)          "\n")
        (insert "  " (prin1-to-string methods)            "\n")
        (insert "  " (prin1-to-string properties)         "\n")
        (insert " )\n")
        ;;(if (= count 5) (debug "Wagner: Die Walkure"))
        (incf count)
        (setq ptr (cdr ptr)))
      (insert ")\n")
      (save-buffer 'NO-QUESTIONS)
      (kill-buffer)
      list
      )
    )
  (message "Finishing defun: save-class-db")
  )

;;(d-assert (find "jtw-build-class-db.el" file-comes-from :test 'string=))

(defun doit ()
  (interactive)
  (message "Welcome to defun: doit from the file: jtw-build-class-db.el")
  ;;(error "Ricky Lee Jones / Take a Walk Around Midnight in the City")
  (setq *elaborate-jtw* nil)
  (if (and (boundp '*prefix*) (boundp '*stump*))
      (save-class-db))
  (message "*** Reached the end of defun: doit in file: jtw-build-class-db.el")
  )

;;(message "*** Scanner has reached end of file: jtw-build-class-db.el")
(provide 'jtw-build-class-db)
