
;;; jtw-build-jtw.el --- A system for translating *.jtw -> *.java

;; Copyright (C) 2006-2011 Davin Pearson

;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: J.T.W. Java
;; Package: dlisp (Davin's version of elisp)
;; Note: in files in the dlisp package, the d- prefix stands for _D_avin's customizations
;; Version: 2.0

;; This file is part of GNU Java Training Wheels.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Install Instructions:

;; See the following URL for the latest info and a tarball:
;;
;; <http://davin.50webs.com/research/2010/mopa2e2.html#jtw-build-java.el>
;;
;; Then untar the tarball to a folder pointed to by the Emacs variable
;; load-path and add the following line to your ~/.emacs file.
;;
;; (require 'jtw-build-java.el)

;;; Commentary:

;; This code compiles a *.jtw file into a *.java file, also giving
;; useful diagnostics on problematic *.jtw code.

;;; Known Bugs:

;; None so far!

;;; Code:

(require 'cl)

(message "*** Welcome to file: jtw-build-java.el")

(setq debug-on-error t)

;;; (setq *prefix* "~/dlisp")
(when (not (fboundp 'd-emergency-set-load-path))
  (defun d-emergency-set-load-path ()
    (assert (and 'valvoline (boundp '*prefix*)))
    (assert *prefix*)
    (setq load-path (cons (expand-file-name (concat *prefix* "/../dlisp/"))
                          load-path))
    (message "** jtw-build-java.el (car load-path)=%s" (car load-path))))

(message "The Late Show with David Letterman")

(d-emergency-set-load-path)

(require 'early-bindings)

;;(message "file: jtw-build-java.el %s" (print-symbol *prefix*))

(if (not (boundp 'file-comes-from))
    (setq file-comes-from nil))

(if (not file-comes-from)
    (setq file-comes-from (cons "jtw-build-java.el" file-comes-from)))

(defun checkpoint (msg &rest rest)
  (apply 'message msg rest)
  ;; do nothing
  )

;;(message "file: jtw-build-java.el %s" (print-symbol *stump*))

(safe-require 'd-flm)

(message "file: jtw-build-java.el %s%s" (print-symbol *prefix*) (print-symbol *stump*))

(fset 'yes-or-no-p 'y-or-n-p)

(require 'd-defmacro)
(require 'jtw-mode)
(require 'jtw-build-class-db)
;;(provide 'jtw-build-java)

(defvar *pp-namespace* "@foomatic555-123-456"
  "Namespace for pre-processor"
  )
(defvar *list-namespace* "@transalpine666-123"
  "Namespace for file include stack")

(defvar *java-namespace* "@telephone-1800-NEW-FUNK")

(defvar *drive-spec* "\\([a-zA-Z]:/\\|~[-a-zA-Z0-9_]*/\\|~/\\|/\\|\\)"
  "Regexp for drive letters in Windows and GNU/Linux")

;; (message "foo %s" 123)
;; (message (format (setq msg "abc%s") (setq rest '("def" "ghi"))))
;;(defun checkpoint (msg &rest rest)
;;  (message (format "*** checkpoint %s" (apply 'format msg rest)))
;;  )

(d-quote
 defadvice message (around jtw-cpp activate)
 (if (and format-string
          (not (string-match "^Mark set"                      format-string))
          (not (string-match "^Indenting region"              format-string))
          (not (string-match "^Loading "                      format-string))
          (not (string-match "^directory-files-deep scanning" format-string))
          (not (string-match "^Fontifying"                    format-string))
          )
     ad-do-it)
 )

;; (2- 10)
(defun 2- (&rest args)
  (apply '+ -2 args))

;; (d-prin1-to-string-java (a + b - 3 to 100 + foo() - bar(123 + abc)) to)
;; (d-prin1-to-string-java (a + b - 100 + foo() - bar(123 + abc)) to)
;; (d-prin1-to-string-java ("abc" + "def" + foo() - bar(123)) to)
;; (d-prin1-to-string-java (a + b - 100 + foo() - bar(123)) to)
;; (d-prin1-to-string-java (foo() - bar(123) to 7) to)
;; (d-prin1-to-string-java apple to)
;; (setq list 'apple)
;; (setq list '(apple))
;; (setq list '(apple banana to frog) to)
;; (setq terminating-symbol 'to)
(defmacro d-prin1-to-string-java (list &optional terminating-symbol)
  (let ((ptr    list)
        (result ""))
    ;; (setq result "")
    ;; (setq ptr    list)
    (cond
     ((null ptr)
      (setq result "()"))
     ((stringp ptr)
      (setq result (prin1-to-string ptr)))
     ((numberp ptr)
      (setq result (concat (prin1-to-string ptr) " ")))
     ((symbolp ptr)
      (setq result (concat (prin1-to-string ptr) " ")))
     (t
      (d-assert (consp ptr))
      (while (and ptr terminating-symbol (not (eq terminating-symbol (car ptr))))
        (setq result (concat result
                             (cond
                              ((null (car ptr))
                               "()")
                              ((numberp (car ptr))
                               (concat (prin1-to-string (car ptr)) " "))
                              ((symbolp (car ptr))
                               (concat (prin1-to-string (car ptr)) " "))
                              ((stringp (car ptr))
                               (concat (prin1-to-string (car ptr)) " "))
                              ((consp (car ptr))
                               (concat "(" (eval `(aref (eval `(d-prin1-to-string-java ,(car ptr) 'sexy-sardines)) 0)) ")"))
                              (t
                               (d-debug "Undefined symbol %s" (car ptr)))
                              ) ;; END COND!
                             )  ;; END CONCAT!
              )	                ;; END SETQ!
        (setq ptr (cdr ptr)))   ;; END WHILE!
      )			        ;; END T!
     )				;; END COND!
    `[, result , ptr]
    )                           ;; END LET!
  )                             ;; END DEFUN!

;; (warn--get-location)
(defun warn--get-location ()
  "Gets the location of the #foomatic declaration"
  (message "Called warn--get-location")
  (save-match-data
    (save-excursion
      (let (l p file line str-to-search)
        (beginning-of-line)
        (setq p (point))
        (when (buffer-file-name)
          (d-assert (string-match "\\.java$" (buffer-file-name))))
        (d-assert (setq bb (buffer-name)))
        ;;(d-debug "Prince / We are a star")
        (setq str-to-search (concat "^[ \t]*///* " *pp-namespace* "[ \t]*#location[0-9]+ (\\(" *drive-spec* "[-a-zA-Z0-9_+./]+\\):\\([0-9]+\\))"))
        ;;(d-debug "Prince / Raspberry Beret")
        (d-assert (or (and 1 (looking-at         str-to-search))
                      (and 2 (re-search-backward str-to-search nil t))))
        (d-assert (and 3 (match-beginning 1)))
        (d-assert (and 4 (match-end 1)))
        (d-assert (and 5 (match-beginning 3)))
        (d-assert (and 6 (match-end 3)))
        (setq file           (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
        (setq line (d-read-str (buffer-substring-no-properties (match-beginning 3) (match-end 3))))
        (d-assert (stringp  file))
        (d-assert (integerp line))
        (beginning-of-line)
        (setq l (count-lines (point) p))
        ;;(d-debug "Hopelessly devoted to you")
        (cons file (+ l line))
        ) ;; LET!
      )
    )
  )

(setq warn--err-msgs nil)

(defun warn--log-message (msg &optional file line)
  (message "Called warn--log-message msg=%s" msg)
  (save-excursion
    (if (and file line)
        (setq warn--err-msgs (cons (list file line (format "*** Error (%s:%d) %s\n" file line msg))
                                   warn--err-msgs))
      (let ((n (warn--get-location)))
        ;;(d-debug "Prince / Purple Rain")
        (setq warn--err-msgs (cons (list (car n) (cdr n) (format "*** Error (%s:%d) %s\n" (car n) (cdr n) msg))
                                   warn--err-msgs))
        ) ;; END LET!
      ) ;; END IF!
    ))

(defun warn-equals (a b)
  (and (string= (nth 0 a) (nth 0 b))
       (= (nth 1 a) (nth 1 b))
       (string= (nth 2 a) (nth 2 b))))

(defun warn--pred (a b)
  ;;(d-debug "Two hearts beat as one")
  (let ((case-fold-search t))
    (if (string=-ignore-case (nth 0 a) (nth 0 b))
        (< (nth 1 a) (nth 1 b))
      (string<-ignore-case (nth 0 a) (nth 0 b)))))

;;; (warn--list-to-string '((12 . "12 abc") (22 . "22 abc")))
(defun warn--list-to-string (list)
  (let ((ptr list)
        (str ""))
    (while ptr
      ;;(d-debug "The Velvet Underground / Look into her false coloured eyes")
      (setq str (concat str (nth 2 (car ptr))))
      (setq ptr (cdr ptr)))
    str))

(defun warn--looking-at (&optional peek-amount)
  (if (not peek-amount)
      (setq peek-amount 20))
  (buffer-substring-no-properties (point) (warn--clamp-point (+ (point) peek-amount))))

(defmacro warn--safe-command (&rest cmd)
  `(condition-case err
       ,(cons 'progn cmd)
     (error )))

;; (warn--clamp-point -2)
(defun warn--clamp-point (newpoint)
  (max (min (point-max) newpoint) (point-min)))

;; "(flm-inside-comment-or-string)         "
"(flm-inside-comment-or-string)             "

(defun flm-inside-comment-or-string ()
  (let (p)
    (setq p (get-char-property (warn--clamp-point (1- (point))) 'face))
    (or (eq p 'font-lock-string-face)
        (eq p 'font-lock-comment-face)
        (eq p 'font-lock-doc-face)
        (eq p 'font-lock-doc-string-face)
        (eq p 'd-face-super-comment)
        )))

(defun flm-string-contains-regexp-no-comments-no-strings (major-mode-name string contains-regexp start)
  "Returns (match-beginning 0) and (match-end 0) etc. to the caller."
  (let (found (d-message-on nil))
    (with-temp-buffer
      (apply major-mode-name nil)
      (d-assert (and (bobp) (eobp)))
      (insert string)
      (font-lock-fontify-buffer)
      (c++2lisp++-stage-1-purge-comments t)
      (goto-char (point-min))
      (goto-char start)
      (setq found (flm-re-search-forward-no-strings contains-regexp nil t))
      ) ;; END LET!
    ))

;; (warn--make-list 3)
(defun warn--make-list (n)
  (let ((list nil))
    (while (> n 0)
      (setq list (cons (cons n t) list))
      (decf n))
    list))

(defvar warn--list nil "Changed by warn--process-list")

(defun warn--goto-line (line)
  (goto-char (point-min))
  (forward-line (1- line)))

;; (warn--process-list)
(defun warn--process-list ()
  (save-excursion
    (let (ptr)
      (goto-char (point-max))
      (setq maxline (warn--what-line))
      (setq warn--list (warn--make-list maxline))
      ;;(setq warn--list nil)
      (setq ptr warn--list)
      ;;(d-debug "warn-list=%s" warn--list)
      (while ptr
        (when (car ptr)
          (warn--goto-line (caar ptr))
          (beginning-of-line)
          ;;(message "line=%d" (warn--what-line))
          (when (looking-at (concat "^   \\(\\(abstract\\|final\\|private\\|protected\\|public\\)"
                                    "[ \t]+\\)*"
                                    "\\(function\\|method\\|constructor\\)"))
            (setq abstract (save-excursion (beginning-of-line)
                                           (flm-re-search-forward--no-strings-no-comments
                                            "\\<abstract\\>"
                                            (point-at-eol) t)))
            (setcdr (car ptr) nil)
            (setq ptr (cdr ptr))
            (warn--goto-line (caar ptr))
            (when ptr
              ;;(message "smeg")
              ;;(message "abstract=%s" abstract)
              (when (not abstract)
                (while (and (not (looking-at "^   begin\\>")) ptr)
                  (setcdr (car ptr) nil)
                  (setq ptr (cdr ptr))
                  (when ptr
                    (message "culling inner line=%d" (warn--what-line))
                    (warn--goto-line (caar ptr)))
                  )))))
        (setq ptr (cdr ptr)))
      (setq ptr warn--list)
      (while ptr
        (when (car ptr)
          (warn--goto-line (caar ptr))
          (beginning-of-line)
          (if (looking-at "^   \\(\\(final\\|private\\|protected\\|public\\)[ \t]+\\)*\\(property\\|classVar\\)\\>")
              (setcdr (car ptr) nil)))
        (setq ptr (cdr ptr)))
      ;; NOTE: Needed for keyword var missing
      (setq ptr warn--list)
      (while ptr
        (when (car ptr)
          (warn--goto-line (caar ptr))
          (beginning-of-line)
          (if (looking-at "^[ \t]*catch[ \t\r\n]*(")
              (setcdr (car ptr) nil)))
        (setq ptr (cdr ptr)))
      (setq ptr warn--list)
      (while ptr
        (when (car ptr)
          (warn--goto-line (caar ptr))
          (beginning-of-line)
          (if (looking-at "^   }")
              (setcdr (car ptr) nil)))
        (setq ptr (cdr ptr)))
      warn--list)))

;;(warn--are-we-editing-p (setq filename "ptr/Beer.jtw"))
(defun warn--are-we-editing-p (filename)
  (setq filename (expand-file-name filename))
  (let ((ptr    (buffer-list))
        (found nil))
    (while (and ptr (not found))
      (if (and os-type-mswindows-p
               (buffer-file-name (car ptr))
               (string=-ignore-case (buffer-file-name (car ptr)) filename))
          (setq found t)
        (if (string= (buffer-file-name (car ptr)) filename)
            (setq found t)))
      (setq ptr (cdr ptr)))
    found)
  )

(defun warn--skip-whitespace ()
  (let ((done nil))
    (while (not done)
      (cond
       ((looking-at "[\t\r\n ]")
        (skip-chars-forward "\t\r\n "))
       ((looking-at "//")   (re-search-forward "$"    nil t))
       ((looking-at "/\\*") (re-search-forward "\\*/" nil t))
       (t
        (setq done t))))))

(d-quote
 defun warn--splat-quest (str)
 (save-match-data
   (when (not (eq nil str))
     (if (not (stringp str))
	 (setq str (prin1-to-string str)))
     (while (string-match "\\<QUEST\\>" str)
       (setq str (concat (substring str 0 (match-beginning 0))
			 "?"
			 (substring str (match-end 0)))))
     (while (string-match "\\\\" str)
       ;;(d-debug "Radiohead: Lucky")
       (setq str (concat (substring str 0 (match-beginning 0))
			 (substring str (match-end 0)))
	     ))
     )
   str))

;; (warn--splat-quest (setq str "abc"))
;; (warn--cull-quotes (setq str "abc"))
;; (warn--cull-quotes (setq str "\"abc"))
;; (warn--cull-quotes (setq str "abc\""))
;; (warn--cull-quotes (setq str "\"abc\""))
(defun warn--cull-quotes (str)
  (when (and str (stringp str) (> (length str) 0))
    (when (= (aref str 0) ?\")
      (setq str (substring str 1)))
    (when (= (aref str (1- (length str))) ?\")
      (setq str (substring str 0 (1- (length str)))))
    )
  str)

(defun warn--buffer-to-string ()
  (buffer-substring-no-properties (point-min) (point-max)))

(defun warn--current-line-as-string ()
  (buffer-substring (point-at-bol) (point-at-eol)))

(defun warn--what-line ()
  "Different from what-line in that it returns the result rather
than printing it"
  (save-excursion
    (beginning-of-line)
    (1+ (count-lines 1 (point)))))



;; (d-read-line-pre)
(defun d-read-line-pre ()
  (let (cb)
    (setq cb (current-buffer))
    (unwind-protect
        (progn
          (setq buf-name "*jtw-cpp*")
          (message "Vampire Weekend")
          (if (get-buffer buf-name)
              (kill-buffer buf-name))
          (setq *buf-read-char* (generate-new-buffer buf-name))
          (set-buffer *buf-read-char*)
          (insert-file-contents (concat *stump* ".jtw"))
          (goto-char (point-min)))
      (set-buffer cb))))

;; (warn--read-line)
(defun d-read-line ()
  (let (cb line)
    (setq cb (current-buffer))
    (unwind-protect
        (progn
          (set-buffer *buf-read-char*)
          (if (eobp)
              (error "Eobp")
            (setq line (warn--current-line-as-string))
            (forward-line 1)
            ))
      (set-buffer cb))
    line))

(defadvice d-indent-new-comment-line (around jtw2java activate)
  ;;(d-debug "splat")
  (save-match-data
    (let (file line)
      (if (save-excursion
            (beginning-of-line)
            ;;(d-debug "Split for the city")
            (flm-re-search-forward--no-strings-no-comments (concat "^[ \t]*//+ " *pp-namespace* "#location[0-9] "
                                                                   "(\\(" *drive-spec* "[-a-zA-Z0-9_./+]*\\):\\([0-9]+\\))")
                                                           (point-at-eol) t))
          (progn
            ;;(d-debug "Famous last words")
            (setq file (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
            (setq line (d-read-str (buffer-substring-no-properties (match-beginning 2) (match-end 2))))
            ;;(if (string-match "pkg.pkg" file) (error "#16 file=%s" file))
            (find-file file)
            (warn--goto-line line))
        ;;(d-debug "Frogger")
        ad-do-it))))

;; (setq file "Code.java")
(defadvice newline (around jtw2java activate)
  ;;(d-beeps "foo")
  (save-match-data
    (let (file line)
      (if (save-excursion
            (beginning-of-line)
            (flm-re-search-forward--no-strings-no-comments "^\\*\\*\\* Error (\\([a-zA-Z_][-a-zA-Z0-9_.]*\\):\\([0-9]+\\))"
                                                           (point-at-eol) t))
          (progn
            (setq file           (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
            (setq line (d-read-str (buffer-substring-no-properties (match-beginning 2) (match-end 2))))
            ;;(d-debug "Swords and Knives")
            (if (get-buffer file)
                (set-buffer (get-buffer file))
              ;;(if (string-match "pkg.pkg" file) (error "#17 file=%s" file))
              (find-file file))
            (warn--goto-line line))
        ad-do-it))))

(defun warn--get-*list-namespace* ()
  (save-excursion
    (d-assert (re-search-backward *list-namespace* nil t))
    (skip-chars-forward "#")
    (skip-chars-forward "a-z")
    (skip-chars-forward "#")
    (skip-chars-forward "0-9")
    ;;(skip-chars-forward " \t")
    ;;(d-debug "Smelly Cat")
    (eval (read (current-buffer)))))

;; (warn--canonicalise-filename (concat default-directory "../jtw-tutorials2/"))
(defun warn--canonicalise-filename (str)
  (setq str (expand-file-name str))
  (d-quote
   (while (string-match "/[^/][^./]+/../" str)
     (setq str (concat (substring str 0 (match-beginning 0))
                       "/"
                       (substring str (match-end 0))))
     ))
  str)

;;(d-assert (find "jtw-build-java.el" file-comes-from :test 'string=))

(d-quote message "*** before defun: doit file: jtw-build-java.el %s"
         (print-symbol *stump*)
         )

(defun doit ()
  (interactive)
  (message "*** Called defun: doit file: jtw-build-java.el %s"
           (print-symbol *stump*))
  ;;(message "*** BIG BIRD")
  (let (beg end new-file class m-call c-call include-file buf
            buf2 count str was-error buf-name file-stack file
            line ptr ptr2 ptr3 (case-fold-search nil))
    (progn
      (d-assert (and 'david-letterman (boundp '*prefix*)))
      (d-assert *prefix*)
      (d-assert (and 'paul-shaffer (boundp '*stump*)))
      (d-assert *stump*)
      ;;(message (format "SUPERCALIFRAGALISTIC! %s%s"
      ;;                 (print-symbol *prefix*)
      ;;                 (print-symbol *stump*)))
      (setq file-old (concat *stump* ".jtw"))
      (setq file-new (concat (substring file-old 0 -3) "java"))
      ;;(message "file-old=%s file-new=%s" file-old file-new)
      (setq *buf* (find-file file-new))
      ;;(switch-to-buffer *buf*)
      ;;(message "Robert Johnson / Malted Milk")
      (set-buffer *buf*)
      (read-only-mode -1)
      (erase-buffer)
      (setq *buf-old* (find-file file-old))
      (set-buffer *buf-old*)
      ;;(d-debug "Robert Johnson / Milkcow's Calf Blues")
      (setq str (buffer-substring-no-properties (point-min) (point-max)))
      ;;(message "(buffer-file-name *buf-old*)=%s" (buffer-file-name *buf-old*))
      ;;(message "(buffer-file-name *buf*)=%s"     (buffer-file-name *buf*))
      ;;(message "*** Peter Gabriel / Live Highlights")
      (set-buffer *buf*)
      (switch-to-buffer *buf*)
      (insert str)
      (message "*** Buffer contents=(%s)" (prin1-to-string str))
      (goto-char (point-min))
      (setq file-stack (list (cons (concat *stump* ".jtw") 1)))
      (insert "/** Automatically generated file. Do not edit! */\n")
      (insert (format "// %s '%s\n"                *list-namespace*
                      (prin1-to-string file-stack)))
      (insert (format "// %s #location0 (%s:%d)\n" *pp-namespace*   (concat *stump* ".jtw") 1))
      ;;(d-debug "Garbage / I would die for you")
      ;;(message "2")
      ;;(error "Franz Schubert / Winterriese")
      (progn
        (set-buffer *buf*)
        (goto-char (point-min))
        (setq rel-dir nil)
        (while (flm-re-search-forward--no-strings-no-comments "^[ \t]*include \"\\([^\"\t\r\n]+\\)\";?" nil t)
          (save-excursion
            (setq file-to-include (d-buffer-substring
                                   (match-beginning 1)
                                   (match-end 1)))
            ;;(setq file-to-include "pkg/foo.function")
            (setq file-to-include (concat rel-dir
                                          (warn--canonicalise-filename file-to-include)
                                          ))
            ;;(d-debug "Pneumatic")
            (setq line-to-include 1)
            (setq file-stack (warn--get-*list-namespace*))
            (defun d-strings-equal-structs (x y)
              (string= x (car y)))
            (when (find file-to-include file-stack :test 'd-strings-equal-structs)
              (d-error (format "Error 48: Infinite loop in include directives file-to-include=%s file-stack=%s" file-to-include (prin1-to-string file-stack))))
            ;; ---------------------------------------------------------
            (setq file-stack (cons (cons file-to-include line-to-include) file-stack))
            (beginning-of-line)
            (delete-region (point-at-bol) (point-at-eol))
            (setq p (point))
            (setq n (warn--get-location))
            (setq file-old (car n))
            (setq line-old (cdr n))
            (setq p-prior (save-excursion (warn--goto-line line-old) (point)))
            (setq dif (count-lines p-prior (point)))
            (setq line-new (+ line-old dif))
            (progn
              (setq file (caar file-stack))
              (setq line (cdar file-stack))
              )
            (progn
              (setq ptr3 (assoc file file-stack))
              (setq ptr2 file-stack)
              (while ptr2
                (when (eq (car ptr2) ptr3)
                  (setq ptr2 (cdr ptr2))
                  ;;(d-debug "If the devil made me here")
                  (setf (cdar ptr2) line-new)
                  ;;(setf (caar ptr2) (concat (file-name-directory file-to-include) file))
                  (setq ptr2 nil))
                (setq ptr2 (cdr ptr2))))
            (let (len n elt dir ptr done)
              (setq len (length file-stack))
              (setq n   (- len 2))
              ;;(d-debug "I would do time for you")
              (while (>= n 0)
                (setq elt (nth n file-stack))
                (setq dir (file-name-directory (car elt)))
                (setq ptr file-stack)
                (while (and ptr (not (eq (car ptr) elt)))
                  (if (not (string-match dir (caar ptr)))
                      (setf (caar ptr) (concat dir (caar ptr))))
                  (setq ptr (cdr ptr)))
                (decf n)))
            ;;;;(message "*** dir=%s" dir)
            ;; ---------------------------------------------------------
            (setq file (caar file-stack))
            (setq dir2 (file-name-directory file))
            ;;;;(message "*** dir2=%s" dir2)
            (insert (format "// %s#1 '%s\n"             *list-namespace* (prin1-to-string file-stack)))
            (insert (format "// %s#location1 (%s:%d)\n" *pp-namespace*   file line))
            (setq m (make-marker))
            (set-marker m (1+ (point)))
            ;;(message "insert-file-contents file=%s" file)
            (insert-file-contents file)
            (save-excursion
              ;;(d-debug "Carl Orff / Carmina Burana")
              (goto-char m)
              ;;(setf (caar file-stack) (concat (file-name-directory file) (caar file-stack)))
              (setq file-stack (cdr file-stack))
              (if file-stack
                  (progn
                    (setq file (caar file-stack))
                    (setq line (cdar file-stack))
                    (d-assert (string= file (caar file-stack)))
                    (d-assert (= line (cdar file-stack)))
                    (d-assert (stringp file))
                    (d-assert (numberp line)))
                (setq file-stack (list (cons (concat *stump* ".jtw") 1)))
                (setq file (caar file-stack))
                (setq line (cdar file-stack))
                )
              ;;(d-debug "Cassandra Wilson")
              (d-assert (string= file (caar file-stack)))
              (insert (format "// %s#2 '%s\n"             *list-namespace* (prin1-to-string file-stack)))
              (insert (format "// %s#location2 (%s:%d)\n" *pp-namespace*   file line))
              (set-marker m nil)
              (setq rel-dir (file-name-directory file))
              ;;(message "rel-dir=%s" rel-dir)
              )))
        )
      ;;(d-debug "Gangster Rapp")
      (set-buffer *buf*)
      (progn
        ;;(switch-to-buffer *buf*)
        (set-buffer *buf*)
        (goto-char (point-min))
        (font-lock-mode 1)
        ;;(font-lock-fontify-buffer)
        ;;(error "Come as you are")
        )
      ;;(d-debug "The kind you find in a second hand store")
      (if (fboundp 'jtw-mode)
          (jtw-mode)
        (java-mode))
      (if (fboundp 'jtw--meta-control-backslash)
          (jtw--meta-control-backslash)
        (c-set-offset 'substatement-open 0)
        (setq c-basic-offset 3)
        (indent-region (point-min) (point-max)))
      (setq warn--err-msgs nil)
      (setq warn--outside-of-class-error nil)
      (setq font-lock-keywords-case-fold-search nil)
      (setq *current-buffer* (current-buffer))
      (setq file-new (concat *prefix* *stump* ".java"))
      ;;(message "pre  load-class-db")
      (setq warn--master-list--fourthly (load-class-db))
      ;;(message "done load-class-db")
      (setq warn--master-list--fourthly (warn--cull-imports file-new warn--master-list--fourthly))
      ;;(d-debug (format "*** file=%s" file))
      ;;(d-debug "Rocky Erickson / True love cast down all evil")
      (message (format "warn--master-list--fourthly=%s" warn--master-list--fourthly))
      ;;(d-debug "Friday on my mind")
      ;;(d-debug "Genesis / No son of mine")
      (progn
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments
                "^\\(\\(public\\|private\\|protected\\|final\\|abstract\\)[ \t]+\\)*function" nil t)
          ;;(d-debug "Van Morrison")
          (setq warn--outside-of-class-error t)
          (warn--log-message "Error 38: Function outside of class")))

      (progn
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "^\\(\\(public\\|private\\|protected\\|final\\|abstract\\)[ \t]+\\)*method" nil t)
          ;;(d-debug "Ronan Keating")
          (setq warn--outside-of-class-error t)
          (warn--log-message "Error 39: Method outside of class")))

      (progn
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "^\\(\\(public\\|private\\|protected\\|final\\|abstract\\)[ \t]+\\)*property" nil t)
          (setq warn--outside-of-class-error t)
          (warn--log-message "Error 40: Property outside of class")))

      (progn
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "^\\(\\(public\\|private\\|protected\\|final\\|abstract\\)[ \t]+\\)*classVar" nil t)
          (setq warn--outside-of-class-error t)
          (warn--log-message "Error 41: classVar outside of class")))

      ;;(error "The Doors / Texas radio and the big beat")
      (progn
        (set-buffer *buf*)
        (warn--process-list))

      (save-excursion
        (set-buffer *buf*)
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "\\<begin\\>" nil t)
          (save-excursion
            (forward-char -1)
            (when (not (flm-inside-comment-or-string))
              (delete-region (match-beginning 0) (match-end 0))
              (insert "{")))))

      (save-excursion
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "\\<elseif\\>" nil t)
          (replace-match "else if" 'fixedcase)))
      (save-excursion
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "\\<beginMain\\>" nil t)
          (save-excursion
            (forward-char -1)
            (when (not (flm-inside-comment-or-string))
              (delete-region (match-beginning 0) (match-end 0))
              (insert "public static void main(String[] args)\n   {")))))

      (save-excursion
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "\\<end\\>" nil t)
          (save-excursion
            (forward-char -1)
            (when (not (flm-inside-comment-or-string))
              (delete-region (match-beginning 0) (match-end 0))
              (insert "}")))))

      (save-excursion
        (goto-char (point-min))
        (while (flm-re-search-forward--no-strings-no-comments "\\<endMain\\>" nil t)
          (save-excursion
            (forward-char -1)
            (when (not (flm-inside-comment-or-string))
              (delete-region (match-beginning 0) (match-end 0))
              (insert "}")))))


      

      (setq indent-tabs-mode nil)
      (java-mode)
      (setq c-basic-offset 3)
      (c-set-offset 'substatement-open 0)
      ;;(switch-to-buffer (current-buffer))
      (setq c-basic-offset 3)
      (setq indent-tabs-mode nil)
      ;;(setq indent-tabs-mode t)
      (font-lock-fontify-buffer)
      ;;(message "Robert Johnson / Walkin' Blues")
      (indent-region (point-min) (point-max))
      ;; ---------------------------------------------------------------
      (let (p1 p2 str form type variable T var start stop
               step-size step-size-2 this_start this_stop this_step
               this_step_size file line p-prior beg0 end0
               (case-fold-search nil) from to step keyword-to
               keyword-step-size)
        (setq strobe nil)

        (save-excursion
          (goto-char (point-min))
          (setq *superfor* 0)
          (while (re-search-forward "\\<superfor\\>" nil t)
            (message "found superfor...")
            (setq beg0 (match-beginning 0))
            (setq end0 (match-end 0))
            (font-lock-fontify-buffer)
            (when (save-excursion
                    (save-match-data
                      (re-search-forward "(" (point-at-eol) t)
                      (forward-char -1)
                      (re-search-forward "\\<var\\>" (point-at-eol) t)
                      (not (flm-inside-comment-or-string))))
              ;;superfor (var int i = 0 to 10)
              ;;(error "Smelly cat")
              (setq *current-buffer* (current-buffer))
              (setq p1 beg0)
              (skip-chars-forward " \t\r\n")
              (when (not
                     (save-match-data
                       (looking-at "{")))
                ;; EVAL HERE! vvv

                (setq p2 ;; EVAL HERE! nnn
                      (save-excursion
                        (forward-sexp 1)
                        (point)))
                (setq str (buffer-substring-no-properties end0 p2))

                (setq form (d-read-str str))

                ;;(d-debug "form")
                ;;(d-assert (consp form))
                ;;(error "The Rolling Stones / Rolling Stones plays Cuba")
                ;;(message "(deleted-region=%s)" (buffer-substring-no-properties p1 p2))
                (delete-region p1 p2)
                (incf *superfor*)
                (setq this (format "superfor_%d_" *superfor*))
                (when (not (eq (nth 0 form) 'var))
                  (warn--log-message "Error 35: Keyword var missing from superfor construct")
                  )
                (when (eq (nth 0 form) 'var)
                  (if (and (not (eq (nth 1 form) 'char))
                           (not (eq (nth 1 form) 'short))
                           (not (eq (nth 1 form) 'int))
                           (not (eq (nth 1 form) 'long))
                           (not (eq (nth 1 form) 'float))
                           (not (eq (nth 1 form) 'double)))
                      (warn--log-message (concat
                                          "Error 37:#2 argument type to superfor macro must be"
                                          " one of char/short/int/long/float/double")))
                  ;; (setq form '(var int i=0 to stop))
                  ;; (setq form '(var int i =0 to stop))
                  ;; (setq form '(var int i = 0 to stop))
                  (progn
                    (setq form-str (aref (eval `(d-prin1-to-string-java ,form sexy)) 0))
                    (when (string-match "^var[ \t]*" form-str)
                      (setq form-str (substring form-str (match-end 0))))
                    (when (string-match "^\\(char\\|short\\|int\\|long\\|float\\|double\\)\\>" form-str)
                      (setq T                       (substring form-str (match-beginning 0) (match-end 0)))
                      (setq form-str (d-trim-string (substring form-str (match-end 0))))
                      (when (string-match "[^<>]=" form-str)
                        (setq var (substring form-str 0 (1+ (match-beginning 0))))
                        (setq form-str (substring form-str (1+ (length var))))
                        ))
                    (cond
                     ((string-match "\\<to\\>" form-str)
                      ;;(message "found to")
                      (setq keyword-to 'to)
                      (setq start      (d-trim-string (substring form-str 0 (match-beginning 0))))
                      (setq form-str   (d-trim-string (substring form-str (match-end 0))))
                      )
                     ((string-match "\\<downto\\>" form-str)
                      ;;(message "found downto")
                      (setq keyword-to 'downto)
                      (setq start    (d-trim-string (substring form-str 0 (match-beginning 0))))
                      (setq form-str (d-trim-string (substring form-str (match-end 0))))
                      )
                     ) ;; END COND!
                    )
                  ;;(d-debug "Duran Duran / Girls on Film")
                  ;;(setq form '(var int i = 0 to 10 step 2))
                  (progn
                    (if (string-match "\\<step\\>" form-str)
                        (progn
                          (setq keyword-step-size t)
                          (setq stop (d-trim-string (substring form-str 0 (match-beginning 0))))
                          (setq step (d-trim-string (substring form-str (match-end 0))))
                          )
                      (setq keyword-step-size nil)
                      (setq stop (d-trim-string form-str))
                      (setq step nil)
                      )
                    ;;(setq start form)
                    ;;(when (string-match "=" start)
                    ;;  (setq start (substring start (match-end 0))))
                    ;;(when (string-match "\\<to\\>" start)
                    ;;  (setq start (d-trim-string (substring start 0 (match-beginning 0)))))
                    ;;(setq rest1 (eval `(d-prin1-to-string-java , form step)))
                    ;;(setq stop (aref rest1 0))
                    ;;(when (string-match "\\<to\\>" stop)
                    ;;  (setq stop (d-trim-string (substring stop (match-end 0)))))
                    ;;(setq keyword-step (car (aref rest1 1)))
                    ;;(when keyword-step
                    ;;  (setq step (aref rest1 1))
                    ;;  (when (eq keyword-step 'step)
                    ;;    (setq step (cadr (aref rest1 1)))
                    ;;    (if step (setq keyword-step-size 'step))))
                    ) ;; END PROGN!
                  ;;(d-debug "Art Blakey / Lou's Blues")
                  (progn ;; (warn--cull-quotes)
                    ;;(setq var         eq)
                    (setq start-2     (warn--splat-quest start))
                    (setq stop-2      (warn--splat-quest stop ))
                    (setq step-size-2 (warn--splat-quest step ))
                    ) ;; END PROGN!
                  ;; -----------------------------------------------
                  ;;(d-debug "The Pretenders / Precious")
                  (setq this_start     (concat this "start"))
                  (setq this_stop      (concat this "stop"))
                  (setq this_step      (concat this "step"))
                  (setq this_step_size (concat this "step_size"))
                  ;;(d-debug "Dire Straits / My Parties")
                  (insert (concat (concat "var " T " " this_start " = " start-2 "; ")
                                  (concat "var " T " " this_stop  " = " stop-2  "; ")
                                  (if keyword-step-size
                                      (concat "var " T " " this_step " = " step-size-2 "; "
                                              "var " T " " this_step_size " = "
                                              (cond
                                               ((eq keyword-to 'to)
                                                (concat "Math.abs(" this_step ")"))
                                               ((eq keyword-to 'downto)
                                                (concat "-Math.abs(" this_step ")"))
                                               (t
                                                (d-debug "Dire Straits / Heavy Fuel")))
                                              ";\n"
                                              )
                                    (concat "var " T " " this_step_size " = "
                                            (cond
                                             ((eq keyword-to 'to)
                                              "1")
                                             ((eq keyword-to 'downto)
                                              "-1")
                                             (t
                                              (d-debug "Dire Straits / Ticket to Heaven")))
                                            ";\n"
                                            ) ;; END CONCAT!
                                    )         ;; END IF!
                                  )           ;; END CONCAT!
                          )                   ;; END INSERT!
                  )                           ;; END PROGN!
                ;;(d-debug "Rod Stewart / Hot Legs")
                (setq line 0)
                (setq p-prior
                      (save-excursion
                        (beginning-of-line)
                        (setq str (concat "^[ \t]*//+ " *pp-namespace* "#location[0-9]"
                                          " (\\("       *drive-spec*   "[-a-zA-Z0-9_./]+\\):\\([0-9]+\\))"))
                        (if (or (looking-at str) (re-search-backward str nil t))
                            (progn
                              ;;(d-debug "Antonio Vivaldi")
                              (setq file (buffer-substring-no-properties           (match-beginning 1)
                                                                                   (match-end 1)))
                              (d-assert (stringp file))
                              (setq line (d-read-str (buffer-substring-no-properties (match-beginning 3)
                                                                                     (match-end 3))))
                              (d-assert (integerp line))
                              (point)
                              )
                          (setq file (concat *prefix* *stump* ".jtw"))
                          (setq line 1)
                          (goto-char (point-min))
                          (forward-line 2)
                          (point)
                          )))
                (setq line (+ line (count-lines p-prior (point))))
                (decf line)
                (decf line)
                (insert (format "// %s '%s\n"                  *list-namespace* (prin1-to-string file-stack)))
                (insert (format "// %s#location3 (%s:%d)\n"    *pp-namespace*   file line))
                (insert (concat "for (var " T " " var " = " this_start ";"
                                " ((" this_step_size " > 0) ? " var " <= "
                                this_stop " : " var " >= " this_stop "); "
                                var " += " this_step_size ")"))
                (if strobe (d-debug "Pretenders / The Wait"))
                ;;(d-debug "Yehudi Menuhin")
                ) ;; END WHEN!
              )     ;; END WHEN!
            )       ;; END WHILE!
          )         ;; END SAVE-EXCURSION!
        )           ;; END LET!
      ;; TODO FIXME: add check for infinite loop in defmacro directives...
      (let (p0 p1 p2 ptr str args fcall)
        (save-excursion
          (goto-char (point-min))
          (while (not (eobp))
            (beginning-of-line)
            (indent-for-tab-command)
            (forward-line)))
        (setq ptr d-macro-list)
        (while ptr
          (d-assert (car ptr))
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments (prin1-to-string (car ptr)) nil t)
            ;;(message "(not (flm-inside-comment-or-string))")
            ;;(d-debug "foomatic")
            (beginning-of-line)
            (setq p0 (point))
            (skip-chars-forward "a-zA-Z0-9_ \t\r\n")
            (setq p1 (point))
            (if (not (looking-at "("))
                (d-error "*** Not looking at \"(\" expression"))
            ;;(d-assert (looking-at "(constructor (int"))
            (forward-sexp)
            (setq p2 (point))
            (setq str (buffer-substring-no-properties p1 p2))
            (delete-region p0 p2)
            ;;(message "* str=%s" str)
            (setq args (split-string-into-csv str))
            ;;(d-debug "Public Enemy / Too Much Posse")
            ;;(message "* args=%s" args)
            ;;(message "* (splat-list args)=%s" (splat-list args))
            (setq fcall (eval `(fcall (car ptr) ,@ (splat-list args))))
            ;;(message "* fcall=%s" fcall)
            (if (stringp fcall)
                (insert fcall))
            ;;(d-debug "Sophisticated Bitch")
            )
          ;;setter(int,i)
          (setq ptr (cdr ptr)))) ;; END LET!
      ;;(d-debug "Gareth Farr / Owhiro")
      ;;(jtw--meta-control-backslash)
      (save-excursion
        (goto-char (point-min))
        (while (not (eobp))
          (beginning-of-line)
          (indent-for-tab-command)
          (forward-line)))
      (progn

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                        "private\\|protected\\|public\\)"
                                                                        "[ \t]+\\)*method\\>") nil t)
            (if (not (looking-at "[ \t]+[A-Za-z_][A-Za-z0-9_<,>]*[][]*[ \t]+[A-Za-z_][A-Za-z0-9_]*("))
                (warn--log-message "Error 1: Method needs a return type"))))

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "^[ \t]*\\([a-z]+\\.\\)*[A-Z][a-zA-Z0-9_]*[ \t]+[a-z][a-zA-Z0-9_]*" nil t)
            (save-excursion
              (forward-line -1)
              (warn--log-message "Error 35: Keyword var missing"))))

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                        "private\\|protected\\|public\\)"
                                                                        "[ \t]+\\)*function\\>") nil t)
            (if (string= 'interface (car (d-get-enclosing-class)))
                (warn--log-message "Error 42: Function cannot be present in an interface")
              (if (not (looking-at "[ \t]+[a-z.]*[A-Za-z_][A-Za-z0-9_<,>]*[][]*[ \t]+[A-Za-z_][A-Za-z0-9_]*("))
                  (warn--log-message "Error 2: Function needs a return type")))))


        (save-excursion

          (goto-char (point-min))
          (setq bstar (current-buffer))
          ;;
          (while (flm-re-search-forward--no-strings-no-comments "constructor[ \t]*(" nil t) ;; ...
            ;;
            (setq enc (d-get-enclosing-class))
            (d-assert enc)
            ;;
            ;;
            (setq class-or-interface (nth 0 enc))
            (setq class              (nth 1 enc))
            (d-assert class)
            (d-assert (stringp class))
            ;;(d-debug "Tchaikovsky / 1812 Festival Overture")
            (replace-match (concat "constructor " class "(") 'fixedcase)
            )
          ) ;; END SAVE-EXCURSION!
        ;;(d-debug "Public Enemy / Party For Your Right to Fight")
        (save-excursion
          ;;(font-lock-fontify-buffer)
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "constructor[ \t]+\\([A-Za-z0-9]+\\)" nil t)
            ;;(when (not (flm-inside-comment-or-string))
            (setq ctor-class (buffer-substring-no-properties (match-beginning 1) (match-end 1)))
            (setq encl (d-get-enclosing-class))
            (setq b1 (current-buffer))
            ;;(error "Foo")
            (setq encl-class (nth 1 encl))
            (d-assert encl-class)
            (d-assert (stringp encl-class))
            (when (not (string= ctor-class encl-class))
              ;;(d-debug "Prince / Girls & Boys ctor-class=%s encl=%s encl-class=%s" ctor-class encl encl-class)
              (warn--log-message (format "Error 3: Constructor class (%s) should be %s" ctor-class encl-class)))
            ;;(d-debug "Vanilla")
            ))

        (d-quote
         save-excursion
         (goto-char (point-min))
         (while (< (point) (point-max))
           (beginning-of-line)
           (font-lock-fontify-buffer)
           (when (flm-re-search-forward--no-strings-no-comments "\\<constructor\\>" (point-at-eol) t)
             (when (not (flm-inside-comment-or-string))

               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<property\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 4: One line cannot have"
                                                  " constructor and property"
                                                  " at the same time"))))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<classVar\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 5: One line cannot have"
                                                  " constructor and classVar"
                                                  " at the same time"))))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<method\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 6: One line cannot have"
                                                  " constructor and method"
                                                  " at the same time"))))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<function\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 7: One line cannot have"
                                                  " constructor and function at"
                                                  " the same time"))))))
           ;;(font-lock-fontify-buffer)
           (when (flm-re-search-forward--no-strings-no-comments "\\<property\\>" (point-at-eol) t)

             (when (not (flm-inside-comment-or-string))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<classVar\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 8: One line cannot have"
                                                  " property and classVar at the"
                                                  " same time"))))
               (progn
                 ;;(font-lock-fontify-buffer)
                 (beginning-of-line)
                 (if (flm-re-search-forward--no-strings-no-comments "\\<method\\>" (point-at-eol) t)
                     (if (not (flm-inside-comment-or-string))
                         (warn--log-message (concat "Error 9: One line cannot have"
                                                    " property and method at the"
                                                    " same time")))))
               (progn
                 ;;(font-lock-fontify-buffer)
                 (beginning-of-line)
                 (if (flm-re-search-forward--no-strings-no-comments "\\<function\\>" (point-at-eol) t)
                     (if (not (flm-inside-comment-or-string))
                         (warn--log-message (concat "Error 10: One line cannot have"
                                                    " property and function at the"
                                                    " same time")))))))
           (beginning-of-line)
           ;;(font-lock-fontify-buffer)
           (when (flm-re-search-forward--no-strings-no-comments "\\<classVar\\>" (point-at-eol) t)
             (when (not (flm-inside-comment-or-string))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<method\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 11: One line cannot have"
                                                  " classVar and method at the"
                                                  " same time"))))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<function\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 12: One line cannot have"
                                                  " classVar and function at the"
                                                  " same time"))))))
           ;;(font-lock-fontify-buffer)
           (beginning-of-line)
           (when (flm-re-search-forward--no-strings-no-comments "\\<method\\>" (point-at-eol) t)
             (when (not (flm-inside-comment-or-string))
               (beginning-of-line)
               (if (flm-re-search-forward--no-strings-no-comments "\\<function\\>" (point-at-eol) t)
                   (if (not (flm-inside-comment-or-string))
                       (warn--log-message (concat "Error 13: One line cannot have"
                                                  " method and function at the"
                                                  " same time"))))))
           (forward-line 1)))

        (save-excursion
          (goto-char (point-min))
          (setq warn--macro-list-concatenated (concat "^[ \t]*\\("
                                                      (mapconcat (function (lambda (x) (prin1-to-string x))) d-macro-list "\\|")
                                                      "\\)("))
          (while (and (flm-re-search-forward--no-strings-no-comments "^   \\([a-z]+\\)" nil t)
                      (save-excursion
                        (beginning-of-line)
                        (not (looking-at "^[ \t]*\\(public +\\|final +\\|abstract +\\|\\)class [A-Z][a-zA-Z0-9_]*"))))
            ;;
            (when (and (not (save-excursion
                              (forward-line -1)
                              (beginning-of-line)
                              (looking-at "^[ \t]*\\(public +\\|final +\\|abstract\\|\\)*class [A-Z][a-zA-Z0-9_]*")))
                       (not (save-excursion
                              (beginning-of-line)
                              (looking-at "^[ \t]*begin\\>")))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (looking-at warn--macro-list-concatenated)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (flm-re-search-forward--no-strings-no-comments "\\<function\\>" (point-at-eol) t)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (flm-re-search-forward--no-strings-no-comments "\\<method\\>" (point-at-eol) t)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (flm-re-search-forward--no-strings-no-comments "\\<classVar\\>" (point-at-eol) t)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (flm-re-search-forward--no-strings-no-comments "\\<property\\>" (point-at-eol) t)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (and (flm-re-search-forward--no-strings-no-comments "\\<constructor\\>" (point-at-eol) t)
                                   (not (flm-inside-comment-or-string)))))
                       (not (save-excursion
                              (beginning-of-line)
                              (looking-at (concat "^[ \t]*public static void main("
                                                  "String\\[\\] [a-z][a-zA-Z_]*)")))))
              (save-excursion
                (beginning-of-line)

                ;;(warn--log-message (concat "Error 14: This line needs one"
                ;;                           " of the following keywords: "
                ;;                           "function/method/classVar/"
                ;;                           "property/constructor"))
                ))))
        (d-quote
         when (not warn--outside-of-class-error)
         (progn
           (goto-char (point-min))
           (while (flm-re-search-forward--no-strings-no-comments "\\<function\\>" nil t)
             ;;(beginning-of-line)
             (setq *current-buffer* (current-buffer))
             ;;(switch-to-buffer *current-buffer*)
             ;;(error "Elevate me mama peek=%s" (warn--looking-at 50))
             (save-excursion
               (when (and (not (save-excursion
                                 (forward-char -2)
                                 (flm-inside-comment-or-string)))
                          (string= "class" (car (d-get-enclosing-class))))
                 (beginning-of-line)
                 (when (flm-re-search-forward--no-strings-no-comments "(" (point-at-eol) t)

                   (backward-char 1)
                   (d-assert (and 'where-ever-i-may-roam (looking-at "(")))
                   (warn--safe-command (forward-sexp))
                   (beginning-of-line)
                   (forward-line 1)
                   (if (not (looking-at "^   {"))
                       (if (looking-at "^[ ][ ][ ][ ][ ][ ]{")
                           (warn--log-message (concat "Error 15: Functions cannot"
                                                      " reside inside functions/"
                                                      "methods/constructors"))
                         (warn--log-message (concat "Error 16: Function must have"
                                                    " begin on the following line"))))))))))
        (d-quote
         when (not warn--outside-of-class-error)
         ;;(font-lock-fontify-buffer)
         (save-excursion
           (goto-char (point-min))
           (while (flm-re-search-forward--no-strings-no-comments "\\<constructor\\>" nil t)
             (save-excursion
               (when (not (flm-inside-comment-or-string))
                 (beginning-of-line)
                 (flm-re-search-forward--no-strings-no-comments "(" nil t)
                 (backward-char 1)
                 ;;(setq *current-buffer* (current-buffer))
                 ;;(switch-to-buffer *current-buffer*)
                 (d-assert (and 1 (looking-at "(")))
                 (warn--safe-command (forward-sexp))
                 (beginning-of-line)
                 (forward-line 1)
                 (if (not (looking-at "^   {"))
                     (if (looking-at "^[ \t]*{")
                         (warn--log-message (concat "Error 17: Constructors cannot"
                                                    " reside inside functions/"
                                                    "methods/constructors"))
                       (warn--log-message (concat "Error 18: Constructor must"
                                                  " have begin on the following"
                                                  " line")))))))))

        (when (not warn--outside-of-class-error)
          (save-excursion
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments "\\<method\\>" nil t)
              ;;(switch-to-buffer *current-buffer*)
              ;; ---------------------------------------------------
              ;;(setq barf-buf (current-buffer))
              ;;(d-assert (flm-inside-comment-or-string))
              ;;(d-debug "All Apologies")
              ;; ---------------------------------------------------
              (save-excursion
                (when (not (flm-inside-comment-or-string))
                  (setq cons (d-get-enclosing-class))
                  (when (and (string= (car cons) "class")
                             (not (save-excursion
                                    (beginning-of-line)
                                    (flm-re-search-forward--no-strings-no-comments "\\<abstract\\>"
                                                                                   (point-at-eol) t))))
                    (beginning-of-line)
                    (flm-re-search-forward--no-strings-no-comments "(" nil t)
                    (backward-char 1)
                    (d-assert (and 2 (looking-at "(")))
                    (warn--safe-command (forward-sexp))
                    (beginning-of-line)
                    (forward-line 1)
                    (if (not (looking-at "^   {"))
                        ;;                ...000
                        (if (looking-at "^      {")
                            t
                          ;;(warn--log-message (concat "Error 19: Methods cannot"
                          ;;                           " reside inside functions/"
                          ;;                           "methods/constructors"))
                          (warn--log-message (concat "Error 20: Method must have"
                                                     " begin on the following line")
                                             )))))))))

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "\\<property\\>" nil t)
            (when (not (flm-inside-comment-or-string))
              (beginning-of-line)
              (forward-line 1)
              (if (looking-at "^[ \t]+{")
                  (warn--log-message (concat "Error 21: Property must not have"
                                             " begin on the following line"))))))

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "\\<classVar\\>" nil t)
            (when (not (flm-inside-comment-or-string))
              (beginning-of-line)
              (forward-line 1)
              (if (looking-at "^[ \t]+{")
                  (warn--log-message (concat "Error 22: Class variable must not"
                                             " have begin on the following line"))))))


        (save-excursion
          ;;(font-lock-fontify-buffer)
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "if[ \t]+" nil t)
            (when (not (flm-inside-comment-or-string))
              (if (not (looking-at "("))
                  (warn--log-message "Error 23: Expecting ( after if statement"))
              (condition-case err
                  (forward-sexp 1)
                (error (warn--log-message (concat "Error 24: Unbalanced"
                                                  " parentheses after if"
                                                  " statement"))))
              ;;(skip-chars-forward " \t\r\n")
              (warn--skip-whitespace)
              (if (not (looking-at "\\<then\\>"))
                  (warn--log-message (concat "Error 25: Expecting then keyword"
                                             " after if statement"))))))


        (save-excursion
          (goto-char (point-min))
          (setq count 0)
          (while (flm-re-search-forward--no-strings-no-comments "[{}]" nil t)
            (save-excursion
              ;;(d-debug "Got my mojo working")
              ;;(forward-char -1)
              (if (flm-inside-comment-or-string)
                  (error "flm-inside-comment-or-string")
                (setq str (buffer-substring-no-properties (match-beginning 0) (match-end 0)))
                (if (string= str "{") (incf count))
                (if (string= str "}") (decf count))
                (if (< count 0)
                    (warn--log-message "Error 26: More ends than begins")))
              ))
          (when (> count 0)
            (goto-char (point-max))
            (warn--log-message "Error 27: Missing ends at the end of the file")
            ;;(d-debug "U2 / Sunday bloody Sunday")
            )
          )

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                        "private\\|protected\\|public\\)"
                                                                        "[ \t]+\\)*method\\>") nil t)
            (setq end (save-excursion
                        (flm-re-search-forward--no-strings-no-comments ")" nil t)
                        (point-at-eol)))
            (if (and (eq (car (d-get-enclosing-class)) 'class)
                     (save-excursion
                       (beginning-of-line)
                       (not (flm-re-search-forward--no-strings-no-comments "\\<abstract\\>" (point-at-eol) t)))
                     (save-excursion
                       (beginning-of-line)
                       (flm-re-search-forward--no-strings-no-comments ";" end t)))
                (warn--log-message "Error 28: Spurious semicolon at the end of the line"))))

        (save-excursion
          (goto-char (point-min))
          (while (flm-re-search-forward--no-strings-no-comments "\\([a-z]\\) (" nil t)
            (replace-match "\\1(" 'FIXEDCASE)))
        (save-excursion
          (goto-char (point-min))
          (when (flm-re-search-forward--no-strings-no-comments "\\<public static void main(String\\[\\] args) {" nil t)

            (beginning-of-line)
            (setq beg (point))
            (setq end (save-excursion
                        (flm-re-search-forward--no-strings-no-comments "{" nil t)
                        (condition-case err
                            (forward-sexp 1)
                          (error ))
                        (point)))

            (setq class (cdr (d-get-enclosing-class)))

            ;;(d-debug "The Yin and the Yang")
            (while (flm-re-search-forward--no-strings-no-comments "[^.a-zA-Z0-9_]\\([a-z][A-Za-z0-9_]*\\)[ \t]*(" end t)

              (when (not (flm-inside-comment-or-string))

                (setq m-call (buffer-substring-no-properties (match-beginning 1) (match-end 1)))





                                                ;; :test 'string=))

                                                ;;(d-debug "Bruce Springsteen  / Darlington County")
                                                (if (find m-call (nth slot-methods (assoc class warn--master-list--fourthly))
                                                          :test 'string=)
                                                    (progn
                                                      (warn--log-message (concat "Error 29: Cannot call a method"
                                                                                 " from the main function"))
                                                      ))
                                                (forward-char -1)
                                                ))
                (goto-char beg)

                (while (flm-re-search-forward--no-strings-no-comments (concat "\\<\\([A-Z]+[a-z][A-Za-z0-9]*\\)"
                                                                              "\\.\\([a-z][A-Za-z0-9]*\\)[ \t]*(") end t)

                  (when (not (flm-inside-comment-or-string))

                    (setq c-call (buffer-substring-no-properties (match-beginning 1) (match-end 1)))

                    (setq m-call (buffer-substring-no-properties (match-beginning 2) (match-end 2)))
                    ;;(d-debug "Rock Shop")
                    ;;(d-debug "Sonny Rollins")
                    (when (find m-call (nth slot-methods (assoc c-call warn--master-list--fourthly))
                                :test 'string=)
                      (warn--log-message (concat "Error 30: Cannot call a method"
                                                 " with a class name prefix from"
                                                 " the main function"))
                      ;;(d-debug "Sonny Rollins")
                      )

                    )))

              (save-excursion
                ;;(font-lock-fontify-buffer)
                (goto-char (point-min))
                (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                              "private\\|protected\\|public\\)"
                                                                              "[ \t]+\\)*function\\>") nil t)
                  (beginning-of-line)
                  (forward-line 1)
                  (when (looking-at "^   {")
                    (setq beg (point))
                    (setq end (save-excursion
                                (flm-re-search-forward--no-strings-no-comments "^   }" nil t)
                                (point)))
                    (setq class (cdr (d-get-enclosing-class)))
                    (while (and (re-search-forward "^   begin" nil t)
                                (progn
                                  (forward-line -1)
                                  (beginning-of-line)
                                  (looking-at "^   \\(public +\\|private +\\|protected +\\|final +\\|abstract +\\)*function\\>")))
                      (while (flm-re-search-forward--no-strings-no-comments (concat "[. \t]"
                                                                                    "\\([a-z][A-Za-z0-9_]*\\)"
                                                                                    "[ \t]*(") end t)
                        (when (not (flm-inside-comment-or-string))
                          (setq m-call (buffer-substring-no-properties
                                        (match-beginning 1)
                                        (match-end 1)))

                          ;;(d-debug "Bruce Springsteen / Cover me")
                          (when (find m-call (nth slot-methods (nth 3 (assoc class warn--master-list--fourthly)))
                                      :test 'string=)
                            ;;(d-debug "R.E.M. / Everybody Hurts")
                            (warn--log-message "Error 31: Cannot call a method from a function")))))
                    (goto-char beg)
                    (while (flm-re-search-forward--no-strings-no-comments (concat "\\<\\([A-Z]+[a-z][A-Za-z0-9]*\\)"
                                                                                  "\\.\\([a-z][A-Za-z0-9]*\\)") end t)
                      (when (not (flm-inside-comment-or-string))
                        (setq c-call (buffer-substring-no-properties
                                      (match-beginning 1)
                                      (match-end 1)))
                        (setq m-call (buffer-substring-no-properties
                                      (match-beginning 2)
                                      (match-end 2)))
                        (if (find m-call (nth slot-methods (assoc c-call warn--master-list--fourthly))
                                  :test 'string=)
                            (warn--log-message (concat "Error 32: Cannot call a method"
                                                       " with a class name prefix"
                                                       " from a function"))))))))

              (save-excursion
                ;;(font-lock-fontify-buffer)
                (goto-char (point-min))
                (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                              "private\\|protected\\|public\\)"
                                                                              "[ \t]+\\)*method\\>") nil t)

                  (beginning-of-line)
                  (forward-line 1)
                  (when (looking-at "^   {")

                    (setq beg (point))
                    (setq end (save-excursion
                                (flm-re-search-forward--no-strings-no-comments "^   }" nil t)
                                (point)))

                    (setq class (cdr (d-get-enclosing-class)))
                    (while (flm-re-search-forward--no-strings-no-comments (concat "\\<\\([A-Z]+[a-z][A-Za-z0-9]*\\)"
                                                                                  "\\.\\([a-z][A-Za-z0-9]*\\)") end t)

                      (when (not (flm-inside-comment-or-string))

                        (setq c-call (buffer-substring-no-properties
                                      (match-beginning 1)
                                      (match-end 1)))
                        (setq m-call (buffer-substring-no-properties
                                      (match-beginning 2)
                                      (match-end 2)))
                        (if (find m-call (nth slot-methods (assoc c-call warn--master-list--fourthly)) :test 'string=)
                            (warn--log-message (concat "Error 33: Cannot call a method"
                                                       " without an object from a method")))))))) ;; rest here

              (save-excursion
                (goto-char (point-min))

                (while (flm-re-search-forward--no-strings-no-comments (concat "^   \\(\\(abstract\\|final\\|"
                                                                              "private\\|protected\\|public\\)"
                                                                              "[ \t]+\\)*constructor\\>") nil t)

                  (beginning-of-line)
                  (forward-line 1)
                  (when (looking-at "^   {")
                    (setq beg (point))
                    (setq end (save-excursion
                                (flm-re-search-forward--no-strings-no-comments "^   }" nil t)
                                (point)))

                    (setq class (cdr (d-get-enclosing-class)))
                    (while (flm-re-search-forward--no-strings-no-comments (concat "\\<\\([A-Z][A-Za-z0-9]*\\)"
                                                                                  "\\.\\([a-z][A-Za-z0-9]*\\)") end t)

                      (setq c-call (buffer-substring-no-properties
                                    (match-beginning 1)
                                    (match-end 1)))
                      (setq m-call (buffer-substring-no-properties
                                    (match-beginning 2)
                                    (match-end 2)))
                      (end-of-line)
                      (forward-char -1)
                      ;;(prin1-to-string (warn--current-line-as-string)))
                      ;;(char-after (point))
                      ;;(get-char-property (1- (point)) 'face)

                      ;;(d-debug ""
                      (when (not (flm-inside-comment-or-string))

                        (if (find m-call (nth slot-methods (assoc c-call warn--master-list--fourthly))
                                  :test 'string=)
                            (warn--log-message (concat "Error 34: Cannot call a method"
                                                       " without an object from a constructor"))))))))


              (setq stage 1)
              (when (= 0 (length warn--err-msgs))
                ;;(d-debug "Viagra")
                (save-excursion ;;
                  ;;(font-lock-fontify-buffer)
                  (goto-char (point-min))
                  ;;(warn--log-message (format "case-fold-search=%s" case-fold-search))
                  (setq case-fold-search nil)
                  (while (flm-re-search-forward--no-strings-no-comments (concat "^[ ][ ][ ][ ][ ][ ][ ]*\\("
                                                                                "\\(superfor\\|for\\|while\\)[ \t]*(\\)?"
                                                                                "\\(boolean\\|char\\|float\\|"
                                                                                "double\\|int\\|long\\|short\\|"
                                                                                "[A-Z]\\|[A-Z]+[a-z][a-zA-Z0-9]*\\)"
                                                                                "\\([][]*\\)[ \t]+"
                                                                                "\\([a-z][A-Za-z0-9]*\\)") nil t)
                    (save-excursion
                      (setq n3 (buffer-substring-no-properties (match-beginning 3) (match-end 3)))
                      (setq n5 (buffer-substring-no-properties (match-beginning 5) (match-end 5)))
                      (setq peek (save-excursion
                                   (beginning-of-line)
                                   (warn--looking-at)))
                      (setq post (warn--looking-at))
                      (beginning-of-line)
                      (forward-line -1)
                      (setq stage 2)
                      (warn--log-message "Error 35: Keyword var missing")
                      )))
                (save-excursion
                  ;;(font-lock-fontify-buffer)
                  (goto-char (point-min))
                  (while (flm-re-search-forward--no-strings-no-comments "\\<var\\>" nil t)
                    (when (not (flm-inside-comment-or-string))
                      (when (and (save-excursion
                                   (beginning-of-line) ;;; for (var int i=0;i<len;i++)
                                   (not (looking-at "^[ ][ ][ ][ ][ ][ ][ ]*var")))
                                 (save-excursion
                                   (skip-chars-backward "a-z") ;; var
                                   (skip-chars-backward "(") ;; open bracket
                                   (skip-chars-backward " \t") ;; spaces
                                   (skip-chars-backward "a-z") ;; for or while
                                   (not (looking-at "\\(superfor\\|for\\|while\\)"))
                                   ;;(d-debug 123)
                                   ))
                        ;;(when (cdr (assq (warn--what-line) warn--list))
                        (save-excursion
                          ;;(forward-line -1)
                          (setq stage 2)
                          ;;(error "Foomatic")
                          (warn--log-message "Error 36: Keyword var does not belong here"))))))

                )

              )
            )

           ;;; first point of splats

          (progn
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments "\\<\\(function\\|classVar\\)\\> " nil t)
              (when (not (flm-inside-comment-or-string))
                (delete-region (match-beginning 0) (match-end 0))
                (insert " static "))))

          (progn
            ;;(font-lock-fontify-buffer)
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments (concat "\\<\\(method\\|property\\|constructor\\|then\\)[ \t\r\n]") nil t)
              (save-excursion
                (forward-char -2)
                (when (not (flm-inside-comment-or-string))
                  (delete-region (match-beginning 0) (match-end 0))))))

          (progn
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments "\\<var[ \t\r\n]+" nil t)
              (save-excursion
                (forward-char -2)
                (when (not (flm-inside-comment-or-string))
                  (delete-region (match-beginning 0) (match-end 0))))))

          (progn
            ;;(font-lock-fontify-buffer)
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments "\\<and\\>" nil t)
              (when (not (flm-inside-comment-or-string))
                (delete-region (match-beginning 0) (match-end 0))
                (insert "&&"))))

          (progn
            (goto-char (point-min))
            (while (flm-re-search-forward--no-strings-no-comments "\\<or\\>" nil t)
              (when (not (flm-inside-comment-or-string))
                (delete-region (match-beginning 0) (match-end 0))
                (insert "||"))))

          (setq was-error nil)

          (when warn--err-msgs

            (setq was-error t)
            (setq warn--err-msgs (sort warn--err-msgs 'warn--pred))
            (setq warn--err-msgs (cons (list "" 0 "** Stage 1: J.T.W. d-debugger Stage 1\n")
                                       warn--err-msgs))
            (setq warn--err-msgs (delete-duplicates warn--err-msgs :test 'warn-equals))
            (setq str (warn--list-to-string warn--err-msgs))
            (and 'dr-dre (save-some-buffers t))
            (d-error (concat "Error messages=" str))
            ;;(error "*** Error compiling J.T.W. code (smeg)")
            )
          (progn
            (java-mode)
            (d-assert (eq major-mode 'java-mode))
            (widen)
            (setq c-basic-offset 3)
            (c-set-offset 'substatement-open 0)
            (indent-region (point-min) (point-max))

            ;;(insert (format "was-error=%s\n" was-error))
            ;;(if (string-match "pkg/inner/A" *stump*)
            ;;    (d-debug "Toilet seat"))
            (if was-error
                (progn
		  ;;
                  (setq buf-name (buffer-name))
                  ;; FIXME: I am temporary
                  (and 'prince (save-some-buffers t))
                  ;;(set-buffer-modified-p nil)
                  (d-error "SNAIL DOG! Error building %s" buf-name))
              (and 'dr-beat (save-some-buffers t))
	      ;;
              ) ;; IF! was-error
            )   ;; END PROGN!
          )     ;; END SAVE-EXCURSION!
    )           ;; END LET!
  ;;(message "*** end of defun: doit file: jtw-build-java.el")
  )             ;; END DEFUN! doit

;;; (round (/ (warn--what-line) 58.0)) 30 pages
;;; (warn--what-line)

;;(message "*** scanner reached end file: jtw-build-java.el %s"
;;         (print-symbol *stump*))

(provide 'jtw-build-java)

