

;;; jtw-javac.el --- A program for receiving the output of the program: javac

;; Copyright (C) 2006-2016 Davin Pearson

;; Author/Maintainer: Davin Max Pearson <http://davin.50webs.com>
;; Keywords: javac backend
;; Version: 2.0

;; This program is part of GNU Java Training Wheels.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl-3.0.txt>.


;;; Commentary:

;; A program for receiving the output of the program: javac in the form
;; of a pipe.

;;; Known Bugs:

;; None so far!

;;; Code:

(message "*** Welcome to file: jtw-java.el")

(require 'cl)

(when (not (fboundp 'd-emergency-set-load-path))
  (defun d-emergency-set-load-path ()
         (assert (and 'king-kong (boundp '*prefix*)))
         (assert *prefix*)
         (setq load-path (cons (expand-file-name (concat *prefix* "/../dlisp/"))
                               load-path))
         (message "** jtw-javac.el (car load-path)=%s" (car load-path))))

(d-emergency-set-load-path)

(require 'early-bindings)

(message "file: jtw-javac.el %s %s" (print-symbol *prefix*) (print-symbol *stump*))

(defun checkpoint (msg &rest rest)
  (apply 'message msg rest)
  ;; do nothing
  )

(if (not (boundp 'file-comes-from))
    (setq file-comes-from nil))

(if (not file-comes-from)
    (setq file-comes-from (cons "jtw-javac.el" file-comes-from)))

(require 'early-bindings)
(require 'jtw-build-java)

(message "*** Welcome to file: jtw-javac.el %s %s"
         (print-symbol *prefix*)
         (print-symbol *stump*)
         )

(d-assert (find "jtw-javac.el" file-comes-from :test 'string=))
;;(d-assert (string= file-comes-from "jtw-javac.el"))

;;(message "*** Symbol value... %s" (print-symbol *stump*))

(defun doit ()
  (interactive)
  ;;(read-line-pre)
  ;;(message "input8: jtw-javac: *stump*=%s" *stump*)
  (message "*** Called defun: doit file: jtw-javac.el %s"
           (print-symbol *stump*))
  (let (numb said-message red-line numb file-less-suffix old-suffix new-suffix
             line-left line-right file-plus-suffix location
             (case-fold-search t) p)
    (condition-case err
        ;;(while (setq red-line (d-read-line))
        (while (setq red-line (read-from-minibuffer ""))
          (setq said-message nil)
          ;;(message "input0: red-line=%s" red-line)
          ;;(if (not (string-match "^Loading " red-line))
          (cond
           ((or (string-match (regexp-quote "Loading 00debian-vars...") red-line)
                (string-match (regexp-quote "Loading /etc/emacs/site-start.d/50autoconf.el") red-line)
                (string-match (regexp-quote "Loading /etc/emacs/site-start.d/50dictionaries-common.el") red-line)
                (string-match (regexp-quote "Loading debian-ispell...") red-line)
                (string-match (regexp-quote "Loading /var/cache/dictionaries-common/emacsen-ispell-default.el...") red-line)
                (string-match (regexp-quote "Loading /var/cache/dictionaries-common/emacsen-ispell-dicts.el...") red-line)
                (string-match (regexp-quote "Loading /etc/emacs/site-start.d/50git-core.el") red-line)
                )
            ;; do nothing
            )
           ((string-match (concat "\\(\\([a-zA-Z]:/\\|"
                                  "~/\\|/\\|\\./\\|\\)"
                                  "[a-zA-Z0-9_/]+\\)"
                                  "\\(\\.java\\):\\([0-9]+\\)")
                          red-line)
            (progn
              (setq file (substring red-line (match-beginning 0) (match-end 3)))
              ;;(message "input6: filename=%s" file)
              (save-match-data
                (if (string-match "^/" file)
                    (setq file (substring file 1))))
              ;;(message "input7: filename=%s" file)
              ;;(setq said-message t)
              (setq numb             (1- (d-read-str (substring red-line
                                                              (match-beginning 4)
                                                              (match-end 4)))))
              (setq file-less-suffix (substring red-line
                                                (match-beginning 1)
                                                (match-end 1)))
              ;;(message "input3: red-line=%s" red-line)
              ;;(message "input3: file-less-suffix=%s" file-less-suffix)
              (setq old-suffix ".java")
              (setq new-suffix ".jtw")
              (setq line-left        (substring red-line 0 (match-end 1)))
              (setq line-right       (substring red-line (match-end 4)))
              (setq file-plus-suffix (concat file-less-suffix new-suffix))
              (setq file             (concat file-less-suffix old-suffix))
              (if (string-match "./" file)
                  (setq file (substring file (match-end 0))))
              ;;(setq default-directory (file-name-directory default-directory))
              ;;(setq file (concat default-directory file))
              ;;(error "Maria Callas")
              ;;(message "input8: (file-name-directory file)=%s" (file-name-directory file))
              ;;(message "input7: file=%s" file)
              ;;(message "input7: default-directory pre=%s" default-directory)
              (d-assert (stringp file))
              ;;(message "input7: file=%s" file)
              ;;(message "input9: (file-name-directory file)=%s" (file-name-directory file))
              (when (file-name-directory file)
                (d-assert (stringp (file-name-directory file)))
                (d-assert (stringp default-directory))
                (if (string-match (file-name-directory file) default-directory)
                    (setq default-directory (substring default-directory 0 (match-beginning 0))))
                ;;(message "input7: default-directory post=%s" default-directory)
                ;;(message "input7: (file-name-nondirectory)=%s" (file-name-nondirectory file))
                )
              (d-assert (stringp file))
              (d-assert (stringp default-directory))
              ;;(message "input8: (concat default-directory file)=%s" (concat default-directory file))
              ;;(message "input8: numb=%s" numb)
              (find-file (concat default-directory file))
              ;;(message "input2: finding file=%s" file)
              ;;(debug "Desolation Row")
              (goto-line numb)
              ;;(debug "Tiger Woods")
              ;;(message "input2: Amber Dempsey")
              ;;(message "input2: (buffer-file-name)=%s" (buffer-file-name))
              (setq location (warn--get-location))
              ;;(message "input2: (cdr location)=%d" (cdr location))
              ;;(message "input2: setq location")
              (setq red-line (concat line-left new-suffix ":" (prin1-to-string (cdr location)) line-right))
              ;;(message "input2: setq red-line")
              ;;(debug "J.S. Bach / Mass in B Minor")
              (message "%s input1: %s" *java-namespace* red-line)))
           (t
            (message "%s input2: %s" *java-namespace* red-line))))
      (error
       (setq p (prin1-to-string (cdr err)))
       (if (and (not (string-match "Error reading from stdin" p))
                (not (string-match "End of file"              p))
                (not (string-match "Eobp"                     p)))
           (message "%s input4: Error=%s" *java-namespace* (cdr err)))
       )))
    ;;(message "*** end defun: doit file: jtw-javac.el %s %s"
    ;;         (print-symbol *stump*)
    ;;         (print-symbol *prefix*))
    )

;;(message "*** Scanner reached end file: jtw-javac.el")
;; (round (/ (d-what-line) 58.0)) 2 pages
(provide 'jtw-javac)
