## -*- makefile -*- An installer module for Java Training Wheels

## Copyright (C) 2016-2017 Davin Pearson

## Maintainer: Davin Pearson <davin.pearson@gmail.com>
## Keywords: J.T.W. installer module

### Commentary:

## This program is part of GNU Java Training Wheels

## Installer-related functions.  For the *.jtw to *.class converter,
## see preprocessors/jtw-projects/preprocessors.el

### Limitation of Warranty

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or (at
## your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more detail.
##
## You should have received a copy of the GNU General Public License
## along with GNU Emacs, see the file COPYING.  If not, see:
##
## <http://www.gnu.org/licenses/gpl.txt>.

### Commentary:

## This Makefile contains code for building *.jtw files into
## *.java files into *.class files and then executing those
## *.class files.

### Code:

.PRECIOUS:
.PRECIOUS: %.java %.class

default: install

install:
	@printf "Install does nothing.  Try \"bash install\" instead\n"

etags:
	etags *.el ../.emacs

JAVAC_FLAGS = -source 1.5 -Xlint:unchecked -Xlint:deprecation -Xlint:-options
JAVA_FLAGS  = -enableassertions
TELEPHONE   = @telephone-1800-NEW-FUNK
ifdef IS_DOS
SHELL = c:/cygwin/bin/bash.exe
EMACS = emacs.exe
else
SHELL = /bin/bash
EMACS = emacs
endif

BAK = $(HOME)/bak
DATESTAMP = $$(date +"%Y%m%d-%H%M%S")

build-class-db:
	@echo  "* Stage 0 : Building class database"
	$(EMACS) --batch --eval "(setq *prefix* \"$$(pwd)\")" --load $$(pwd)/../dlisp/jtw-build-class-db.el --funcall doit

#>/dev/null

%.java: %.jtw
	@printf "* Stage 1 : Debugging $*.jtw and building $*.java file *prefix*=%s *stump*=%s\n" $$(pwd) $*
	@$(EMACS) --batch --eval "(progn (setq *prefix* \"$$(pwd)\") (setq *stump* \"$*\"))" --load $$(pwd)/../dlisp/jtw-build-java.el --funcall doit

###
### was $$(find . -name "*.java")
### was $^
###
### NOTE: telephone is an arbitrary name to prevent against accidental
### aliasing with the rest of the code
###
%.class: %.java
	@printf "* Stage 2 : Debugging *.java file(s) and building *.class file(s) *stump*=%s *prefix*=%s\n" $$(pwd) $*
	@printf "Running javac... *prefix*=%s *stump*=%s \n" $$(pwd) $*
	@javac $(JAVAC_FLAGS) $$(find . -name "*.java") |& ($(EMACS) --batch --eval "(progn (setq *stump* \"$*\")(setq *prefix* \"$$(pwd)\"))" --load $$(pwd)/../dlisp/jtw-javac.el --funcall doit) |& (grep "$(TELEPHONE) input[0-9]*:" - ) |& (sed -e "s/$(TELEPHONE) input[0-9]*://g" - )

#>/dev/null

%.run: %.class
	@printf "* Stage 3 : Running $*.class File *stump*=%s\n" $*
	@printf "Running java...*prefix*=%s *stump*=%s\n" $$(pwd) $*
	java $(JAVA_FLAGS) $* |& ($(EMACS) --batch --eval "(progn (setq *stump* \"$*\")(setq *prefix* \"$$(pwd)\"))" --load $$(pwd)/../dlisp/jtw-java.el --funcall doit) |& (grep "$(TELEPHONE) input[0-9]*:" - ) |& (sed -e "s/$(TELEPHONE) input[0-9]*://g" - )

#(set +o pipefail)

clean: build-class-db
	rm -fv *.java
	rm -fv *.class

### preprocessors/jtw-projects/Makefile.mk ends here
