;;; preprocessor.el --- An installer for GNU Java Training Wheels and Lisp++

;; Copyright (C) 2016 Davin Pearson

;; Maintainer: Davin Pearson <http://davin.50webs.com>
;; Keywords: Lisp++ and Java Training Wheels installer
;; Version 1.2:

;;; Commentary:

;; This program is part of GNU Java Training Wheels and Lisp++

;; This file provides a mechanism for installing and uninstalling
;; Lisp++ and J.T.W. code compilation code.  In the code that follows,
;; the d- prefix stands for _D_avin's customisations to elisp.

;;; Limitation of Warranty

;; This program is free software: you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation, either version 3 of the
;; License, or (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more detail.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs, see the file COPYING.  If not, see:
;;
;; <http://www.gnu.org/licenses/gpl.txt>.

(require 'cl)
(setq d-source-dir (getenv "PWD"))
(setq d-program-terminated-message "Program terminated...\n")

(message "*** Welcome to file: preprocessors.el namely the J.T.W. installer module")

(message "*** Stevie Wonder / Ebony Eyes d-source-dir=%s" d-source-dir)
(message "*** Stevie Nicks / Stop Draggin' my Heart Around (getenv \"PWD\")=%s" (getenv "PWD"))
(message "*** Tracy Chapman / Talkin' 'bout a Revolution (getenv \"HOME\")=%s" (getenv "HOME"))

(message "*** before setting *prefix*, d-source-dir=%s" d-source-dir)
(setq *prefix* (concat d-source-dir "/dlisp"))
(message "*** before expand-file-name: *prefix*=%s" *prefix*)
(setq *prefix* (expand-file-name *prefix*))
(message "*** after expand-file-name *prefix*=%s" *prefix*)

(setq debug-on-error t)

(message "**** *prefix*=%s" *prefix*)

(when *prefix*
  (when (not (fboundp 'd-emergency-set-load-path))
    (defun d-emergency-set-load-path ()
           (message "Inside file: preprocessors.el defun: d-emergency-set-load-path")
           (assert (and 'china-girl (boundp '*prefix*)))
           (assert *prefix*)
           (setq load-path (cons (expand-file-name *prefix*)
                                 load-path))
           (message "** file: preprocessor.el (car load-path)=%s" (car load-path))
           ) ;; END DEFUN! d-emergency-set-load-path
    )        ;; END WHEN! (when (not (fboundp 'd-emergency-set-load-path))
  )          ;; END WHEN! *prefix*

(message "**** Tina Turner / Nutbush City Limits")

(d-emergency-set-load-path)

(message "*** before delving into file: early-bindings.el *prefix*=%s" *prefix*)
(require 'early-bindings)
(require 'd-directory-files-deep)

(d-assert (boundp 'd-source-dir))

(defun d-unsplat-file-dir (file)
  ;;(shell-command (format "chmod 755 %s" file))
  (condition-case err
      (when (file-exists-p file) ;; os-type-linux-p
        (message "*** d-unsplat-file-dir on file %s" file)
        (save-excursion
          (find-file file)
          (read-only-mode -1)
          (goto-char (point-min))
          (when (re-search-forward (regexp-quote "J.T.W.") nil t)
            (delete-region (point-at-bol) (1+ (point-at-eol))))
          (while (looking-at "[ \t\r\n]")
            (delete-char 1))
          (save-some-buffers 'NO-QUESTIONS)
          (kill-buffer)))
    (error
     (message "Error: %s" (prin1-to-string err)))))

(progn
  (setq *begin-string* ";;; BEGIN java-training-wheels stuff")
  (setq *begin-regexp* (concat "^" (regexp-quote *begin-string*) "$"))
  (setq *end-string*   ";;; END java-training-wheels stuff")
  (setq *end-regexp*   (concat "^" "[; ]* END java-training-wheels stuff$"))
  (setq *full-emacs*   nil)
  )

;; (d-add-to-dotemacs)
(defun d-add-to-dotemacs (dotemacs)
  (interactive)
  (message "Welcome to d-add-to-dotemacs dotemacs=%s" dotemacs)
  (d-assert (string-match "\\.emacs$" dotemacs))
  (find-file dotemacs)
  (goto-char (point-min))
  (when (and (progn
               (goto-char (point-min))
               (not (re-search-forward *begin-regexp* nil t)))
            (progn
              (goto-char (point-min))
              (not (re-search-forward *end-regexp* nil t))))
    (message "Found *begin-regexp* or *end-regexp*")
    (goto-char (point-max))
    (setq p1 (point))
    (skip-chars-backward " \t\r\n")
    (setq p2 (point))
    (delete-region p1 p2)
    (insert "\n" *begin-string* "\n")
    (d-assert (boundp *full-emacs*))
    (insert ";;; Please do not remove or edit the above line or this will prevent the\n"
            ";;; preprocessor system from working.\n")
    (setq d-target-dir (d-remove-trailing-slash d-target-dir))
    (message "d-target-dir=%s %s" d-target-dir (print-symbol *prefix*))
    (if (not *full-emacs*)
	(progn
          (insert         "(setq d-target-dir " (prin1-to-string *prefix*) ")\n")
          (insert         "(setq load-path (append (list (expand-file-name (concat *prefix* \"../dlisp\")))\n")
          (insert         "   load-path))\n")
	  (insert         ";; Set the following variables to nil to turn off Davin's syntax highlighting prefs.\n")
          (insert         "(setq prefs-minimal-syntax-highlighting-p t) ;; keep essential syntax highlighting\n")
          (insert         "(setq prefs-maximal-syntax-highlighting-p t) ;; keep most syntax highlighting\n")
          (insert         "")
	  (insert         "(require 'early-bindings)\n")
	  (insert         "(require 'd-make-faces)\n")
	  (insert         "(require 'jtw-mode)\n")
	  )
      (progn ;; ELSE BRANCH!
        (insert           "(setq d-target-dir " (prin1-to-string *prefix*) ")\n")
        (insert           "(setq load-path (append (list (expand-file-name (concat *prefix* \"../dlisp\")))\n")
        (insert           "   load-path))\n")

        (insert           "")
	(insert           "(d-emergency-set-load-path)\n")
        (insert           ";; Set the following variables to nil to turn off Davin's syntax highlighting prefs.\n")
	(insert           "(setq prefs-minimal-syntax-highlighting-p t) ;; keep essential syntax highlighting\n")
	(insert           "(setq prefs-maximal-syntax-highlighting-p t) ;; keep most syntax highlighting\n")
        (insert           "")
	(insert           "(require 'd-start)\n")
	)
      )
    (message "*** d-target-dir=%s" d-target-dir)
    (insert *end-string* "\n")
    (insert ";;; Please do not remove or edit the above line or this will prevent the\n"
            ";;; preprocessor system from working.\n")
    )
  (save-buffer)
  (kill-buffer)
  )

;; (d-remove-from-dotemacs (setq dotemacs "~www/.emacs"))
(defun d-remove-from-dotemacs (dotemacs)
  (interactive)
  (progn
    (message "Welcome to d-remove-from-dotemacs dotemacs=%s" dotemacs)
    (when (not (string-match "\\.emacs$" dotemacs))
      (d-error (concat "Error .emacs is not a valid .emacs (%s) file\n"
                       d-program-terminated-message)
               dotemacs))
    (find-file dotemacs)
    (when (re-search-forward *begin-regexp* nil t)
      ;;(forward-line -1)
      (beginning-of-line)
      (setq p1 (point))
      (when (re-search-forward *end-regexp* nil t)
        ;;(forward-line 1)
        (beginning-of-line)
        (while (looking-at ";;;.*$")
          (forward-line 1))
        (setq p2 (point))
        (message "Deleting region=(%s)" (buffer-substring-no-properties p1 p2))
        (delete-region p1 p2)
        (save-buffer)
        (kill-buffer)
        (message "Here is the contents of your ~/.emacs file after suppressing J.T.W. mode code.")
        (d-shell-command "cat ~/.emacs" nil 'PRINT-OUTPUT)
        ))))

(defun do-uninstall ()
  (interactive)
  (when os-type-mswindows-p
    (message "Warning MS-Windows detected.")
    (message "Installation is not guaranteed to work without errors ")
    (message "if your shell does not have write access to the c:/Program Files* folders\n"))
  (if os-type-linux-p
      (progn
        ;;(getenv "PWD")
        (setq answer-0
	      (y-or-n-p
	       "Enter installation prefix dir or press y ENTER for /usr): "))
        (cond
         ((string= "y" answer-0)
          (setq d-target (getenv "PWD"))
          )
         ((string= "n" answer-0)
          (setq d-target d-usr)
          )
         ((string= "" answer-0)
          (setq d-target (getenv "PWD")))
         )
        (if (or (string= d-target-dir "") (string= d-target-dir "y"))
	    (setq d-target-dir "/usr")))
    ;; else branch of if os-type-linux-p ... else
    (d-assert os-type-mswindows-p)
    (setq d-target-dir (read-from-minibuffer "Enter installation prefix dir (or press y ENTER for c:/): "))
    (if (or (string= d-target-dir "") (string= d-target-dir "y"))
	(setq d-target-dir "c:/preprocessors")))
  (setq d-target-dir (d-remove-trailing-slash d-target-dir))
  ;; -----------------------------------------------------------------
  (d-remove-from-dotemacs (expand-file-name (concat "~" *user-name*)))
  (message "Finished the preprocessors.el uninstaller module")
  )

;;(message "args=%s" args)
;;(setq args "--prefix=~/jtw")
;;(setq d-target-dir "~/jtw")
;;(setq d-target-dir "/usr")

;; (yes-or-no-p "Really smeg?")
;;
;; NOTE: Patches yes-or-no-p to y-or-n-p so that (yes | (bash install))
;; works by providing the default options for the install/uninstall scripts.
(fset 'yes-or-no-p 'y-or-n-p)

(if (not (boundp 'args))
    (setq args ""))

;; (yes2y-no2n (read-from-minibuffer "Foo> "))
;; (setq y-or-n "yes")
(defun yes2y-no2n (y-or-n)
  (let ((case-fold-search t))
    (setq y-or-n
          (cond
           ((null y-or-n)
            "")
           ((string-match "^y\\(es\\)?$" y-or-n)
            "y")
           ((string-match "^n\\(o\\)?$" y-or-n)
            "n")
           (t
            y-or-n)))))

(defun d-check-for-missing-folders (filename)
  ;; (d-check-for-missing-folders (setq filename "~/dlisp"))
  ;; (d-check-for-missing-folders (setq filename "~/abc.el"))
  (when (not (file-exists-p filename))
    (setq is-a-dir (file-directory-p filename))
    (d-error (concat "Missing " (if is-a-dir "folder " "file ") (format "%s" filename) " from archive\n"
		     "Try re-untaring the tarball preprocessors-YYYYmmdd-HHMMSS.tar.gz,\n"
		     "where YYYY is the year the tarball was modified, mm is the month the\n"
		     "file was modified, dd is the day the tarball was modified and so on\n"
		     "for HH MM and SS\nThen re-run the installer module.  "
                     d-program-terminated-message
                     "  Type bash install for more information.\n"))
    t))

(defun prefixify (str)
  (with-temp-buffer
    (insert str)
    (goto-char (point-min))
    ;;; prevents the first line from being quoted
    (forward-line 1)
    (while (not (eobp))
      (beginning-of-line)
      (insert "> ")
      (forward-line 1)
      (end-of-line))
    (d-buffer-substring (point-min) (point-max))
    ))

(defun d-shell-command (cmd &optional please-wait print-output)
  (let (b1 b2 str)
    (save-excursion
      (if please-wait
          (message "Executing shell command: %s\nPlease wait..." cmd)
        (message "Executing shell command: %s" cmd))
      (shell-command cmd)
      (when print-output
            (set-buffer "*Shell Command Output*")
            (goto-char (point-min))
            (insert "Here are the contents of the shell command output:")
            (setq b1 (point-min))
            (setq b2 (point-max))
            (d-assert b1)
            (d-assert b2)
            (when (and b1 b2)
              (setq str (d-buffer-substring b1 b2))
              (message "str=(%s)" (prefixify str))
              ) ;; END WHEN!
            )
      nil
      )   ;; END SAVE-EXCURSION!
    )     ;; END LET!
  )       ;; END DEFUN! d-shell-command (cmd &optional please-wait print-output)

;; (d-add-trailing-slash (setq str "/abc/def/"))
(defun d-add-trailing-slash (str)
  (save-match-data
    (setq str
          (if (string-match "/$" str)
              str
            (concat str "/")))))

;; (d-remove-trailing-slash (setq str "/abc/def/"))
(defun d-remove-trailing-slash (str)
  (save-match-data
    (setq str
          (if (string-match "/$" str)
              (substring str 0 (match-beginning 0))
            str))))

(setq pwd (getenv "PWD"))

(defun doit () ;; A.K.A. defun: install
  (interactive)
  (setq debug-on-error t)
  (d-assert d-source-dir)
  ;;(d-assert d-target-dir)
  (let (str pwd user-dir)
    (let (answer-0 done)
      ;;(message "*** Stevie Wonder / All Day Sucker d-target-dir=%s" d-target-dir)
      (when os-type-mswindows-p ;; (setq *user-name* "www")
        (message "Warning MS-Windows detected.  Installation is not guaranteed to work without errors\n")
        (message "if your shell does not have write access to the c:/Program Files* folders\n")
        (setq answer-0 (y-or-n-p "Continue? "))
        (if (not answer-0)
            (message "You chose option Y to continue")
          (d-error "\nYou chose option N to abort.\n" d-program-terminated-message))
        ) ;; END WHEN! os-type-mswindows-p
      (when (or (not (boundp '*user-name*)) (not *user-name*) (string= *user-name* ""))
        (d-error "*** User name not specified.  Type \"bash install --help\" OR \"bash uninstall --help\" for more information." d-program-terminated-message))
      (setq pwd (d-add-trailing-slash (getenv "PWD")))
      (message "*** (getenv \"PWD\")=%s" (getenv "PWD"))
      ;;(d-error "early termination")
      (message "*** (getenv \"HOME\")=%s" (getenv "HOME"))
      (message "*** d-source-dir=%s" d-source-dir)
      ;;(message "maggot *user-name*=%s" *user-name*)
      ;;(message "*** (getenv HOME)=%s" (getenv "HOME"))
      (setq *root-user-p* (if (string-match "^/root$" (getenv "HOME")) t))
      (when (and os-type-linux-p (not *root-user-p*))
        ;;(d-debug "Calamansi")
        ;;(message "(current-buffer)=%s" (current-buffer))
        (message (format "You really should be logged in as the root user to run this script."))
        (message (format "If you are not logged in as the root user then you will"))
        (message (format "only be able to access your HOME folder \"~/\" (i.e. %s)." (getenv "HOME")))
        (message (format "Your answer to the question after this one will reflect this certainty."))
        (setq answer-0 (y-or-n-p "Continue? "))
        (if (eq t answer-0)
            (message "\nYou chose option Y to continue\n")
          (d-error "\nYou chose option N to abort\n" d-program-terminated-message))
        ;;(debug)
        (progn
          (d-check-for-missing-folders (concat (d-remove-trailing-slash d-source-dir) "/dlisp/"))
          ;;(d-check-for-missing-folders (concat d-source-dir "/lisp++-projects/"))
          (d-check-for-missing-folders (concat (d-remove-trailing-slash d-source-dir) "/jtw-projects/"))
          (d-check-for-missing-folders (concat (d-remove-trailing-slash d-source-dir) "/working/"))
          (d-check-for-missing-folders (concat (d-remove-trailing-slash d-source-dir) "/install"))
          (d-check-for-missing-folders (concat (d-remove-trailing-slash d-source-dir) "/uninstall"))
          )
        )
      (setq load-path (cons (concat
                             (d-remove-trailing-slash d-source-dir)
                             "/dlisp")
                            load-path))
      (cond
       (os-type-linux-p
        (setq d-usr-share "/usr/share")
        (setq d-usr       "/usr")
        )
       (os-type-mswindows-p
        (setq d-usr-share "c:/Program Files")
        (setq d-usr       "c:/")))
      (setq d-source-dir (d-add-trailing-slash d-source-dir))
      )
    (let (answer-1 done prompt) ;; (setq *user-name* "www")
      (setq done nil)
      (setq prompt (format
                    (concat
                     "Type in a folder name such as \"%s ENTER\" or type \"y ENTER\" for\n"
                     "%s (the HOME folder of the user %s) or type \"n ENTER\" for %s> ")
                    d-usr-share
                    (expand-file-name (concat "~" *user-name*))
                    *user-name*
                    d-usr))
      ;; --------------------------------------------------
      (setq *same-dir* nil)
      (setq done nil)
      (while (not done)
        (setq answer-1 (yes2y-no2n (read-from-minibuffer prompt)))
        (setq d-target-dir (cond
                            ((string= "y" answer-1)
                             (setq *same-dir* t)
                             (setq done t)
                             d-source-dir) ;; a.k.a. (getenv "PWD")
                            ((string= "n" answer-1)
                             (setq *same-dir* nil)
                             (setq done t)
                             d-usr)
                            ((string= "" answer-1) ;; (setq prompt "")
                             (setq prompt
                                   (if (not (string-match "Please enter " prompt))
                                       (concat
                                        "Please enter \"y ENTER\" or type in the"
                                        (format "name of an existing folder (such as %s or\n"
                                                d-usr-share)
                                        "\"n ENTER\" to choose an option.\n"
                                        prompt)))
                             )
                            (t
                             (setq done t)
                             answer-1)
                            ))
        (if (not (file-exists-p d-target-dir))
            (d-error (concat "Error folder " (prin1-to-string d-target-dir)
                             " does not exist.  Please re-run\n"
                             "the intaller module and enter the name of an existing\n"
                             "folder.\n" d-program-terminated-message "\n"))))
      )
    (let (answer-2 done prompt) ;; d-source-dir
      ;;(d-debug "Dire Straits / Industrial Disease")
      (message "*** Emma Kirkby / d-source-dir=%s\n" d-source-dir)
      ;;(d-error "Smegulator")
      ;;(setq d-target-dir (d-remove-trailing-slash d-target-dir))
      ;;(when (not (string-match "preprocessors$" d-target-dir))
      (when (not (string-match "/preprocessors$" d-target-dir))
        (setq d-target-dir (expand-file-name
                            (concat d-target-dir "/preprocessors"))))
      )
    (let (answer-3 str-3 done prompt)
      (setq d-target-dir (d-add-trailing-slash d-target-dir))
      (if (string-match "\\(/preprocessors/\\)\\(preprocessors/\\)" d-target-dir)
          (setq d-target-dir (substring d-target-dir 0 (match-beginning 2))))
      ;; (setq *user-name* "www")
      ;; ---------------------------------------------------------------
      (setq str-3 (concat "Install just Davin's jtw-mode instead of Davin's Full Version of GNU Emacs?\n"
                          "Press \"y ENTER\" for yes and \"n ENTER\" to install Davin's Full Version\n"
                          "of GNU Emacs> "))
      (setq answer-3 nil)
      ;; ---------------------------------------------------------------
      (while (not answer-3)
        (setq answer-3 (yes2y-no2n (read-from-minibuffer str-3)))
        (setq str-3 "Please enter y on n> ")
        (cond
         ((string= "y" answer-3)
          (setq *full-emacs* nil))
         ((string= "n" answer-3)
          (setq *full-emacs* t))
         (t
          (setq answer-3 nil)))))
    ;; (d-debug "Stevie Wonder / He's Misstra Know-It-All")
    ;; BEGIN MAIN COPIER COMMAND! goes here
    (message "\n*** Suzanne Vega / Institution Green d-source-dir=%s d-target-dir=%s"
             d-source-dir d-target-dir)
    (if (not *same-dir*)
        (condition-case err
            (progn
              (setq d-source-dir (d-add-trailing-slash d-source-dir))
              (setq d-target-dir (d-add-trailing-slash d-target-dir))
              (message "Prior to modification to d-target-dir=%s" d-target-dir)
              (save-match-data
                (when (and (not (string-match "preprocessors/$" d-target-dir))
                           (string-match "^.*/preprocessors/.*$" d-target-dir))
                  (setq d-target-dir (expand-file-name (concat d-target-dir "/../")))))
              (message "Removing preprocessor from d-target-dir=%s" d-target-dir)
              (d-shell-command (format "mkdir -pv %s/dlisp"        d-target-dir))
              (d-shell-command (format "mkdir -pv %s/jtw-projects" d-target-dir))
              (d-shell-command (format "mkdir -pv %s/"             d-target-dir))
              (d-shell-command (format "cp -pruv %s/dlisp/Makefile.mk %s/jtw-projects/Makefile"))
              (setq d-source-dir (d-remove-trailing-slash d-source-dir))
              (setq d-target-dir (d-remove-trailing-slash d-target-dir))
              (message "MOLASSES! d-source-dir=%s d-target-dir=%s" d-source-dir d-target-dir)
              (d-shell-command (format "chmod 755 %s/dlisp" d-target-dir))
              (d-shell-command (format "cp -pruv %s/dlisp/*.el %s/dlisp"
                                       d-source-dir
                                       (expand-file-name
                                        (concat d-target-dir "/../dlisp"))))
              (d-shell-command (format "cp -pruv %s/dlisp/imported-tuff/*.el %s/dlisp/imported-tuff/" d-source-dir d-target-dir))
              (d-shell-command (format "cp -pruv %s/dlisp/caps/* %s" d-source-dir d-target-dir))
              (d-shell-command (format "cp -pruv %s/jtw-projects/MyFirstProgram.jtw %s/preprocessors" d-source-dir d-target-dir))
              (d-shell-command (format "cp -pruv %s/jtw-projects/Makefile %s/preprocessors" d-source-dir d-target-dir))
              (d-shell-command (format "cp -pruv %s/jtw-projects/Makefile.mk %s/preprocessors" d-source-dir d-target-dir))
              ;;(d-shell-command (format "cp -pruv %s/lisp++-projects %s" d-source-dir d-target-dir))
              (d-shell-command (format "chown %s $(find %s -type d)"      *user-name* d-target-dir))
              (d-shell-command (format "chown %s $(find %s -not -type d)" *user-name* d-target-dir))
              (d-shell-command (format "chmod 755 $(find %s -type d)"                 d-target-dir))
              (d-shell-command (format "chmod 644 $(find %s -not -type d)"            d-target-dir))
              (d-shell-command (format "chown %s $(find %s -type d)"      *user-name* d-source-dir))
              (d-shell-command (format "chown %s $(find %s -not -type d)" *user-name* d-source-dir))
              (d-shell-command (format "chmod 755 $(find %s -type d)"                 d-source-dir))
              (d-shell-command (format "chmod 644 $(find %s -not -type d)"            d-source-dir))
              (d-assert (file-exists-p d-target-dir))
              )
          (error "Error acessing subdirs of d-target-dir=%s d-source-dir=%s" d-target-dir)))
    (when *same-dir*
      (d-shell-command (format "chown %s $(find %s -type d)"      *user-name* d-target-dir))
      (d-shell-command (format "chown %s $(find %s -not -type d)" *user-name* d-target-dir))
      (d-shell-command (format "chmod 755 $(find %s -type d)"                 d-target-dir))
      (d-shell-command (format "chmod 644 $(find %s -not -type d)"            d-target-dir))
      )
    ;; END MAIN COPIER COMMAND! here
    ;; (setq *user-name* "www")
    (setq d-source-dir (d-add-trailing-slash d-source-dir))
    (setq d-target-dir (d-add-trailing-slash d-target-dir))
    (when (string-match "\\(/preprocessors\\)\\(/preprocessors\\)/" d-target-dir)
      (setq d-target-dir (concat (substring d-target-dir 0 (match-beginning 0))
                                 (substring d-target-dir (match-beginning 2) -1))))
    (setq user-dir-dotemacs (expand-file-name (concat "~" *user-name* "/.emacs")))
    (d-remove-from-dotemacs user-dir-dotemacs)
    (d-add-to-dotemacs      user-dir-dotemacs)
    (d-shell-command         (concat "chown " *user-name* " ~" *user-name* "/.emacs"))
    (d-shell-command         (concat "chmod 644 ~" *user-name* "/.emacs"))
    (setq d-target-dir (d-remove-trailing-slash d-target-dir))
    (when (string-match "preprocessors\\(/preprocessors\\)$" d-target-dir)
      (setq d-target-dir (substring d-target-dir 0 (match-beginning 1)))
      (message "*** Found doubled-up preprocessors/preprocessors")
      )
    (save-excursion
      (when (file-exists-p (concat d-target-dir "/jtw-projects/class-db.el"))
        (d-shell-command
           (concat "chown " *user-name* " " d-target-dir "/jtw-projects/class-db.el"))
        (d-shell-command
         (concat "chgrp 644 " d-target-dir "/jtw-projects/class-db.el"))
        ) ;; END WHEN!
      )   ;; END SAVE-EXCURSION!
    (message "*** ANDRE Ice Cold 3000 d-target-dir=%s" d-target-dir)
    (setq d-target-dir (d-remove-trailing-slash d-target-dir))
    (find-file (expand-file-name (concat d-target-dir "/jtw-projects/Makefile")))
    (read-only-mode -1)
    (goto-char (point-min))
    (flush-lines "include Makefile.mk")
    (goto-char (point-min))
    (insert "include Makefile.mk\n")
    ;;(condition-case err
    (and 'cant-buy-me-love (save-buffer))
    ;;  (error "An error ocurred while attempting to save the makefile")
    ;;  )
    (kill-buffer)
    (message "\nNote that the folder jtw-projects is located at the following location...\n%s/jtw-projects" d-target-dir)
    ;;(message "(2) Note that the folder lisp++-projects is located at the following location...\n%s/lisp++-projects" d-target-dir)
    (save-excursion
      (set-buffer "*Shell Command Output*")
      (kill-buffer))
    (message "Finished the preprocessors.el installer module.")
    (message d-program-terminated-message)
    ) ;; END LET! (str pwd user-dir)
  )   ;; END DEFUN! doit

